/**
  test_vacuumExtension.ctl contains all the code used for testing the vacuumExtension functionality
*/

#uses "vacuumExtension/vacuumExtension.ctl"
#uses "fwUnitTestComponentAsserts.ctl"
#uses "unVacObjects/cpcVV_S.ctl"


const string VACUUM_TEST_CPC_VV_S_DEVICE_NAME = "test";
const dyn_string VACUUM_TEST_CPC_VV_S_DPES = makeDynString(
    VACUUM_TEST_CPC_VV_S_DEVICE_NAME + ".frontend.alarm", 
    VACUUM_TEST_CPC_VV_S_DEVICE_NAME + ".frontend.enabled",
    VACUUM_TEST_CPC_VV_S_DEVICE_NAME + ".ProcessInput.EvStsReg01",
    VACUUM_TEST_CPC_VV_S_DEVICE_NAME + ".ProcessInput.EvStsReg01:_online.._invalid",
    VACUUM_TEST_CPC_VV_S_DEVICE_NAME + ".ProcessInput.EvStsReg02",
    VACUUM_TEST_CPC_VV_S_DEVICE_NAME + ".ProcessInput.EvStsReg02:_online.._invalid",    
    VACUUM_TEST_CPC_VV_S_DEVICE_NAME + ".ProcessInput.EvStsReg03",
    VACUUM_TEST_CPC_VV_S_DEVICE_NAME + ".ProcessInput.EvStsReg03:_online.._invalid",
    VACUUM_TEST_CPC_VV_S_DEVICE_NAME + ".ProcessInput.ObjectSt",
    VACUUM_TEST_CPC_VV_S_DEVICE_NAME + ".ProcessInput.ObjectSt:_online.._invalid",
    VACUUM_TEST_CPC_VV_S_DEVICE_NAME + ".disabled"
);
/**
  Tests the state dpe's dispatching and defaults values

  @covers CPC_VV_S_StateDPEs  
*/
void test_CPC_VV_S_StateDPEs(){
  dyn_string dsDpes;
  CPC_VV_S_StateDPEs(VACUUM_TEST_CPC_VV_S_DEVICE_NAME, dsDpes);
  dyn_string dsExp = unVacObject_dynSlice(VACUUM_TEST_CPC_VV_S_DPES, 3);
  // Assert that the expected datapoints VACUUM_TEST_CPC_VV_S_DPES (without the system integretity) is equal to the result
  assertEqual(dsExp, dsDpes);
}

/** 
  Tests if the non valid animation is correct and if it has the highest priority.
  This is done by testing the different valid dpes with all the bits high of the status register.
  And by verifying it's not selected when all the validness values are false
*/
void test_CPC_VV_S_StateMainColorAndString_NotValid(){
  string sColor, sState; // The result color and string  

  // The list of dpe names  
  dyn_string dsDpes = VACUUM_TEST_CPC_VV_S_DPES;
  // The keys in the dpe value array(daValues) that correspond to validness
  dyn_int diKeys = makeDynInt(4, 6, 8, 10);
  // All bits on, worst case sceneario
  bit32 bit32StsReg = 4294967295;
  
  // The mocked dpe values
  dyn_anytype daValues = makeDynAnytype(false, true, bit32StsReg, false, bit32StsReg, false, bit32StsReg, false, 0, false, false);   
  
  // Test when everything is valid
  CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  assertFalse("unDataNotValid" == sColor);
  assertFalse(CPC_STATE_INVALID == sState);
  
  // Loop over all the validness key positions in daValues and check them all seperately  
  for(int i = 1; i<=dynlen(diKeys); i++){
    daValues[diKeys[i]] = true; // Set invalid to true
    sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);
    daValues[diKeys[i]] = false; // Reset dpe mocked values
    
    // Assert invalid color 
    assertEqual("unDataNotValid", sColor);
    assertEqual(CPC_STATE_INVALID, sState);
  }
}


/**
  Tests the state main color for alarms

  @covers vacuumExtension_StateMainColorAndString  
*/
void test_CPC_VV_S_StateMainColorAndString_AlarmUnack(){
  string sColor, sState; // The result color and string  
  dyn_string dsDpes = VACUUM_TEST_CPC_VV_S_DPES;
  bit32 bit32StsReg01;
  setBit(bit32StsReg01, CPC_StsReg01_ALUNACK, true);
  
  // Test with all bits high 
  dyn_anytype daValues = makeDynAnytype(false, true, 4294967295, false, 4294967295, false, 4294967295, false, 0, false, false); 
  CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  
  assertFalse("unWidget_AlarmNotAck" == sColor);
  assertFalse(CPC_STATE_ALARMUNACK == sState);
 
  // Test with only CPC_StsReg01_ALUNACK bit high 
  dyn_anytype daValues = makeDynAnytype(false, true, bit32StsReg01, false, 0, false, 0, false, 0, false, false); 
  CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);
    
  assertFalse("unWidget_AlarmNotAck" == sColor);
  assertFalse(CPC_STATE_ALARMUNACK == sState);
  
  // Test with no bits high 
  dyn_anytype daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 0, false, false); 
  CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);
    
  assertFalse("unWidget_AlarmNotAck" == sColor);
  assertFalse(CPC_STATE_ALARMUNACK == sState);
  
  // Test with UNICOS acknowledge mechanism
  dyn_anytype daValues = makeDynAnytype(false, true, bit32StsReg01, false, 0, false, 0, false, 0, false, false, 0, 1, 1, 0); 
  CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState, false, true);
    
  assertEqual("unWidget_AlarmNotAck", sColor);
  
  // Test with UNICOS acknowledge mechanism without CPC_StsReg01_ALUNACK bit
  dyn_anytype daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 0, false, false, 0, 1, 1, 0); 
  CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState, false, true);
    
  assertFalse("unWidget_AlarmNotAck" == sColor);
}

/**
  Tests the state main color for alarms

  @covers vacuumExtension_StateMainColorAndString  
*/
void test_CPC_VV_S_StateMainColorAndString_Alarm(){
  string sColor, sState; // Resulting color

  dyn_string dsDpes = VACUUM_TEST_CPC_VV_S_DPES; // List of DPE names
  dyn_anytype daValues; // The corresponding DPE values

  bit32 bit32StsReg01, bit32StsReg02, bit32StsReg03;

  // No interlock active  
  // Build dpe values
  daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 0, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  assertFalse("unFaceplate_AlarmActive" == sColor);
  assertFalse(CPC_STATE_ALARM == sState);

  // Only first interlock active    
  setBit(bit32StsReg03, CPC_StsReg03_LCL1ITLST, true);  // Set the local interlock bit
  // Build dpe values
  daValues = makeDynAnytype(false, true, 0, false, 0, false, bit32StsReg03, false, 0, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  
  assertEqual("unFaceplate_AlarmActive", sColor);
  assertEqual(CPC_STATE_ALARM, sState);

  // Both interlock active
  setBit(bit32StsReg03, CPC_StsReg03_LCL2ITLST, true);  // Set the second local interlock bit
  // Build dpe values
  daValues = makeDynAnytype(false, true, 0, false, 0, false, bit32StsReg03, false, 0, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  
  //assertEqual("unFaceplate_AlarmActive", sColor);  
  
  // Only second interlock active   
  setBit(bit32StsReg03, CPC_StsReg03_LCL1ITLST, false);  // Unset the first local interlock bit
  // Build dpe values
  daValues = makeDynAnytype(false, true, 0, false, 0, false, bit32StsReg03, false, 0, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);   
    
  //assertEqual("unFaceplate_AlarmActive", sColor);
  
  // CPC_StsReg01_TSTOPIST active   
  setBit(bit32StsReg01, CPC_StsReg01_TSTOPIST, true);  // Set bit
  // Build dpe values
  daValues = makeDynAnytype(false, true, bit32StsReg01, false, 0, false, 0, false, 0, false, false); 
  // Test function
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);     
    
  assertEqual("unFaceplate_AlarmActive", sColor);  
  assertEqual(CPC_STATE_ALARM, sState);  
  setBit(bit32StsReg01, CPC_StsReg01_TSTOPIST, false);  // Unset bit

  // CPC_StsReg01_TSTOPIST active   
  setBit(bit32StsReg02, CPC_StsReg02_FUSTOPIST, true);  // Set bit
  // Build dpe values  
  daValues = makeDynAnytype(false, true, 0, false, bit32StsReg02, false, 0, false, 0, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
    
  assertEqual("unFaceplate_AlarmActive", sColor);
  assertEqual(CPC_STATE_ALARM, sState);  
  setBit(bit32StsReg02, CPC_StsReg02_FUSTOPIST, false);  // set bit
  
   // CPC_StsReg01_ALST active   
  setBit(bit32StsReg01, CPC_StsReg01_ALST, true);  // Unset the first local interlock bit
  // Build dpe values  
  daValues = makeDynAnytype(false, true, bit32StsReg01, false, 0, false, 0, false, 0, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);   
    
  assertEqual("unFaceplate_AlarmActive", sColor);
  assertEqual(CPC_STATE_ALARM, sState);  
  setBit(bit32StsReg01, CPC_StsReg01_ALST, false);  // Unset the first local interlock bit
}

/**
  Test the disabled color of the widget/state
  
  @covers vacuumExtension_StateMainColorAndString
*/
void test_CPC_VV_S_StateMainColorAndString_Disabled(){
  string sColor, sState; // Resulting color

  dyn_string dsDpes = VACUUM_TEST_CPC_VV_S_DPES; // List of DPE names
  dyn_anytype daValues; // The corresponding DPE values  

  // No interlock active  
  // Build dpe values
  daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 0, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  assertFalse("FwEquipmentDisabled" == sColor);
  assertFalse(CPC_STATE_DISABLED == sState);  
  
  daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 0, false, true); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  assertEqual("FwEquipmentDisabled", sColor);
  assertEqual(CPC_STATE_DISABLED, sState);  
}

/**
  Tests the state main color for warnings

  @covers vacuumExtension_StateMainColorAndString  
*/
void test_CPC_VV_S_StateMainColorAndString_Warning(){
  string sColor, sState; // Resulting color

  dyn_string dsDpes = VACUUM_TEST_CPC_VV_S_DPES; // List of DPE names
  dyn_anytype daValues; // The corresponding DPE values    
  
  bit32 bit32StsReg01, bit32StsReg02, bit32StsReg03; // Status registers
  
  // Test valve in OK status
  daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 1, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertFalse("cpcColor_Widget_Warning" == sColor);
  assertFalse(CPC_STATE_WARNING == sState);  
  
  // Test valve in warning status (ObjectSt == 0)
  daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 0, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertEqual("cpcColor_Widget_Warning", sColor);
  assertEqual(CPC_STATE_WARNING, sState);  
  
  // Test valve in CPC_StsReg01_IOERRORW status
  setBit(bit32StsReg01, CPC_StsReg01_IOERRORW, true);
  daValues = makeDynAnytype(false, true, bit32StsReg01, false, 0, false, 0, false, 0, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertEqual("cpcColor_Widget_Warning", sColor);
  assertEqual(CPC_STATE_WARNING, sState);  
  setBit(bit32StsReg01, CPC_StsReg01_IOERRORW, false);

  // Test valve in CPC_StsReg03_GLBBEFOREITLST status
  setBit(bit32StsReg03, CPC_StsReg03_GLBBEFOREITLST, true);
  daValues = makeDynAnytype(false, true, 0, false, 0, false, bit32StsReg03, false, 0, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertEqual("cpcColor_Widget_Warning", sColor);
  assertEqual(CPC_STATE_WARNING, sState);  
  setBit(bit32StsReg01, CPC_StsReg03_GLBBEFOREITLST, false);

  // Test valve in CPC_StsReg03_GLBAFTERITLST status
  setBit(bit32StsReg03, CPC_StsReg03_GLBAFTERITLST, true);
  daValues = makeDynAnytype(false, true, 0, false, 0, false, bit32StsReg03, false, 0, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertEqual("cpcColor_Widget_Warning", sColor);
  assertEqual(CPC_STATE_WARNING, sState);  
  setBit(bit32StsReg01, CPC_StsReg03_GLBAFTERITLST, false);
}

/**
  Tests the state main color for alarms

  @covers vacuumExtension_StateMainColorAndString  
*/
void test_CPC_VV_S_StateMainColorAndString_Forced(){
  string sColor, sState; // Resulting color

  dyn_string dsDpes = VACUUM_TEST_CPC_VV_S_DPES; // List of DPE names
  dyn_anytype daValues; // The corresponding DPE values    
  
  bit32 bit32StsReg01, bit32StsReg02, bit32StsReg03; // Status registers
  
  // Test valve in OK status
  daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 1, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertFalse("unWidget_ControlStateForced" == sColor);
  assertFalse(CPC_STATE_FORCED == sState);    
    
  // Test valve in CPC_StsReg01_FoMoSt status
  setBit(bit32StsReg01, CPC_StsReg01_FoMoSt, true);
  daValues = makeDynAnytype(false, true, bit32StsReg01, false, 0, false, 0, false, 1, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertEqual("unWidget_ControlStateForced", sColor);
  assertEqual(CPC_STATE_FORCED, sState);  
  setBit(bit32StsReg01, CPC_StsReg01_FoMoSt, false);

  // Test valve in undefined status (ObjectSt == 4) for StateHistory(bState is true). This animation differs for the widget
  daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 4, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertEqual("unWidget_ControlStateForced", sColor);
  assertEqual(CPC_STATE_UNDEFINED, sState);  
  
  // Test valve in undefined status (ObjectSt == 4) for StateHistory(bState is true). This animation differs for the widget
  daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 4, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState, false);  
  assertFalse("unWidget_ControlStateForced" == sColor);
}

/**
  Tests the state main color for off state in the state history

  @covers vacuumExtension_StateMainColorAndString  
*/
void test_CPC_VV_S_StateMainColorAndString_OffState(){
  string sColor, sState; // Resulting color

  dyn_string dsDpes = VACUUM_TEST_CPC_VV_S_DPES; // List of DPE names
  dyn_anytype daValues; // The corresponding DPE values    
    
  // Test valve in ON status
  daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 1, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertFalse("unEventList_ForeColor" == sColor);
  assertFalse(CPC_STATE_OFF == sState);  
  
  // Test valve in OK status as widget
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState, false);  
  assertFalse("unEventList_ForeColor" == sColor);
  assertFalse(CPC_STATE_OFF == sState);  
  
  // Test valve in OFF status as widget
  daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 2, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState, false);  
  assertFalse("unEventList_ForeColor" == sColor);   
  assertFalse(CPC_STATE_OFF == sState);  
  
    // Test valve in OFF status in state history
  daValues = makeDynAnytype(false, true, 0, false, 0, false, 0, false, 2, false, false); 
  sColor = CPC_VV_S_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertEqual("unEventList_ForeColor", sColor);
  assertEqual(CPC_STATE_OFF, sState);  
}
