/**
  Unit tests for the unVacTrending library
  
*/
#uses "fwUnitTestComponentAsserts.ctl"
//#uses "unVacTrending/unVacTrending.ctl"
  

/**
  Test the guessing of the scale type, to make sure we always use the correct scale 
*/
void test_unVacTrending_guessScaleType(){
  // Use numbers to do assertion as importing enum is impossible apparently 
  assertEqual(10, (int) unVacTrending_guessScaleType("dist_1:un-CFP_VAC-VAC-CPC_VP_TP-00001.ProcessInput.PosSt")); // Percentage
  assertEqual(5, (int) unVacTrending_guessScaleType("dist_1:un-CFP_VAC-VAC-CPC_VP_TP-00001.ProcessInput.MotPosSt")); // Amps Medium
  assertEqual(1, (int) unVacTrending_guessScaleType("dist_1:un-CFP_VAC-VAC-CPC_VG_PT-00001.ProcessInput.PosSt")); // Millibar vacuum scale
  assertEqual(1, (int) unVacTrending_guessScaleType("dist_1:un-CFP_VAC-VAC-CPC_VP_I-00001.ProcessInput.PosSt")); // Millibar vacuum scale
  assertEqual(4, (int) unVacTrending_guessScaleType("dist_1:un-CFP_VAC-VAC-CPC_VP_I-00001.ProcessInput.APosSt")); // Ampere low scale
  assertEqual(9, (int) unVacTrending_guessScaleType("dist_1:un-CFP_VAC-VAC-CPC_VP_I-00001.ProcessInput.VPosSt")); // Volt high scale
  assertEqual(11, (int) unVacTrending_guessScaleType("dist_1:un-CFP_VAC-VAC-CPC_VP_I-00001.ProcessInput.ObjectSt")); // Object St
  assertEqual(14, (int) unVacTrending_guessScaleType("dist_1:un-CFP_VAC-VAC-CPC_OnOff-00001.ProcessInput.OnSt")); // Bool status OnSt
}


/**
  Test the sanitize name functionality, to make sure always the same name is used
*/
void test_unVacTrending_sanitizeName(){
  assertEqual("un-CFP_VAC-VAC-CPC_VG_PT-00008.ProcessInput.ObjectSt", unVacTrending_sanitizeName("dist_1:un-CFP_VAC-VAC-CPC_VG_PT-00008.ProcessInput.ObjectSt:_online.._value"));
  assertEqual("un-CFP_VAC-VAC-CPC_VG_PT-00008.ProcessInput.ObjectSt", unVacTrending_sanitizeName("dist_1:un-CFP_VAC-VAC-CPC_VG_PT-00008.ProcessInput.ObjectSt"));
  assertEqual("un-CFP_VAC-VAC-CPC_VG_PT-00008.ProcessInput.ObjectSt", unVacTrending_sanitizeName("un-CFP_VAC-VAC-CPC_VG_PT-00008.ProcessInput.ObjectSt"));
  assertEqual("un-CFP_VAC-VAC-CPC_VG_PT-00008.ProcessInput.ObjectSt", unVacTrending_sanitizeName("un-CFP_VAC-VAC-CPC_VG_PT-00008.ProcessInput.ObjectSt:_online.._value"));
}
  
