/**@file

// cpcVP_GConfig.ctl
This library contains the import and export function of the CPC_VP_G.

@par Creation Date
  24/05/2016

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author
  Sebastien Blanchard (TE-VSC)

@copyright
        &copy;Copyright CERN 2016 - all rights reserved
*/
//@{



// Constants
const string UN_CONFIG_CPC_VP_G_DPT_NAME = "CPC_VP_G";
//begin_TAG_SCRIPT_DEVICE_TYPE_constants

// import row: import row config line description : data value separated by ';'
//-------------
// // 10 GENERIC (CPC_VP_G;deviceNumber;Alias[,DeviceLinkList];Description;Diagnostics;WWWLink;Synoptic;Domain;Nature;WidgetType;) 
// // 4(of 22) MAIN (MotPosStUnit;MotPosStFormat;ArchiveMode;TimeFilter;) 
// // 9(of 22) MAIN ADDRESS (addr_StsReg01;addr_StsReg02;addr_EvStsReg01;addr_EvStsReg02;addr_PosSt;addr_MotPosSt;addr_ManReg01;addr_ObjectSt;addr_ErrCoSt;)
// // 4(of 22) MAIN LIMIT (posStHHLimit;posStHLimit;posStLLimit;posStLLLimit;)
// // 6(of 22) MAIN ALARM and MAPPING (AlarmActive;SMSCat;AlarmMessage;AlarmAck;mapping_ObjectSt;)
// // 3 ARCHIVE (BooleanArchive;AnalogArchive;EventArchive;)
// // 10 ADDITIONAL (MaskEvent;Parameters;MasterDevice;parents;children;type;secondAlias;ControllerModel;TMPModel;mapping_ErrCoSt;)


const unsigned UN_CONFIG_CPC_VP_G_NORMALPOSITION	    = 1; 
const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_STSREG01	  = 2;
const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_STSREG02	  = 3;
const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_EVSTSREG01 	= 4;
const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_EVSTSREG02 	= 5;
const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_AUOPMOST   	= 6;
const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_OPMOST      = 7;
const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_CUSTEPST   	= 8;
const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_CUTRST   	  = 9;

const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_OBJECTST   	= 10;
const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_ERRCOST    	= 11;
const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_WARNCOST   	= 12;
const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_MANREG01   	= 13;
const unsigned UN_CONFIG_CPC_VP_G_ADDRESS_MOPMOR     	= 14;

const unsigned UN_CONFIG_CPC_VP_G_MODE_NAME_1 		= 15;
const unsigned UN_CONFIG_CPC_VP_G_MODE_NAME_2 		= 16;
const unsigned UN_CONFIG_CPC_VP_G_MODE_NAME_3 		= 17;
const unsigned UN_CONFIG_CPC_VP_G_MODE_NAME_4 		= 18;
const unsigned UN_CONFIG_CPC_VP_G_MODE_NAME_5 		= 19;
const unsigned UN_CONFIG_CPC_VP_G_MODE_NAME_6 		= 20;
const unsigned UN_CONFIG_CPC_VP_G_MODE_NAME_7 		= 21;
const unsigned UN_CONFIG_CPC_VP_G_MODE_NAME_8 		= 22;
                                                      
const unsigned UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_1 	= 23;
const unsigned UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_2 	= 24;
const unsigned UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_3 	= 25;
const unsigned UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_4 	= 26;
const unsigned UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_5 	= 27;
const unsigned UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_6 	= 28;
const unsigned UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_7 	= 29;
const unsigned UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_8 	= 30;
                                                      
const unsigned UN_CONFIG_CPC_VP_G_OBJECTST_MAPPING	= 31;
const unsigned UN_CONFIG_CPC_VP_G_ERRCOST_MAPPING	  = 32;
const unsigned UN_CONFIG_CPC_VP_G_WARNCOST_MAPPING	= 33;

const unsigned UN_CONFIG_CPC_VP_G_LENGTH            = 33;

const unsigned UN_CONFIG_CPC_VP_G_ARCHIVE_BOOLEAN_ARCHIVE  = 1;
const unsigned UN_CONFIG_CPC_VP_G_ARCHIVE_ANALOG_ARCHIVE   = 2;
const unsigned UN_CONFIG_CPC_VP_G_ARCHIVE_EVENT_ARCHIVE    = 3;

const unsigned UN_CONFIG_CPC_VP_G_ARCHIVE_LENGTH           = 3;

const unsigned UN_CONFIG_CPC_VP_G_ADDITIONAL_MASKEVENT	= 1;
const unsigned UN_CONFIG_CPC_VP_G_ADDITIONAL_PARAMETERS = 2;
const unsigned UN_CONFIG_CPC_VP_G_ADDITIONAL_MASTER_NAME= 3;
const unsigned UN_CONFIG_CPC_VP_G_ADDITIONAL_PARENTS    = 4;
const unsigned UN_CONFIG_CPC_VP_G_ADDITIONAL_CHILDREN   = 5;
const unsigned UN_CONFIG_CPC_VP_G_ADDITIONAL_TYPE       = 6;
const unsigned UN_CONFIG_CPC_VP_G_ADDITIONAL_SECOND_ALIAS  = 7;


const unsigned UN_CONFIG_CPC_VP_G_ADDITIONAL_LENGTH        = 7;
//end_TAG_SCRIPT_DEVICE_TYPE_constants

//------------------------------------------------------
// NOTES :
// (1) ARCHIVE_ACTIVE
//N for NO archiving
//VA,0.1,N for Absolute Value archiving with 0.1 deadband and NO time
//VR,0.1,N as above, but Relative Deadband
//Y for Time-only archiving
//VA,0.1,A for Absolute Value archiving with 0.1 deadband AND Time
//VR,0.1,A for Relative Value archiving with 0.1 deadband AND Time  DEFAULT ARCHIVE SETUP FOR VACUUM ANALOG VALUE
//VA,0.1,O for Absolute Value archiving with 0.1 deadband OR Time
//VR,0.1,O for Relative Value archiving with 0.1 deadband OR Time
//A for Old/new comparison AND time
//O for Old/new comparison OR time



/**
DPE configuration
TODO: use fw info instead
*/
mapping CPC_VP_GConfig_getConfig() {
    mapping config;
    mapping props;

//begin_TAG_SCRIPT_DEVICE_TYPE_deviceConfig
// 9 ADDRESS (addr_StsReg01;addr_StsReg02;addr_EvStsReg01;addr_EvStsReg02;addr_PosSt;addr_MotPosSt;addr_ManReg01;addr_ObjectSt;addr_ErrCoSt;)
// 1
    mappingClear(props);
    props["address"] 		  = ".ProcessInput.StsReg01";
    props["dataType"] 		= CPC_UINT16;
    config["StsReg01"] 		= props;
// 2
    mappingClear(props);
    props["address"] 		  = ".ProcessInput.StsReg02";
    props["dataType"] 		= CPC_UINT16;
    config["StsReg02"] 		= props;
// 3
    mappingClear(props);
    props["hasArchive"] 	= true;
    props["address"] 		  = ".ProcessInput.evStsReg01";
    props["dataType"] 		= CPC_INT32;
    config["evStsReg01"] 	= props;
// 4
    mappingClear(props);
    props["hasArchive"] 	= true;
    props["address"] 		  = ".ProcessInput.evStsReg02";
    props["dataType"] 		= CPC_INT32;
    config["evStsReg02"] 	= props;
// 5
	  mappingClear(props);
	  props["hasMapping"]   = true;
    //props["hasSmooth"] 	  = false; does not work
	  //props["hasArchive"]   = true; archive setting done in custom function
    props["address"]      = ".ProcessInput.ObjectSt";
    props["dataType"]     = CPC_UINT16;
    config["ObjectSt"]    = props;
// 6
    mappingClear(props);
 	  props["hasMapping"]   = true;
	  //props["hasArchive"]   = true; archive setting done in custom function
    //props["hasSmooth"]   	= false; does not work
    props["address"] 		  = ".ProcessInput.ErrCoSt";
    props["dataType"] 		= CPC_UINT16;
    config["ErrCoSt"] 		= props;	
// 7
    mappingClear(props);
	  props["hasMapping"]   = true;
	  //props["hasArchive"]   = true; archive setting done in custom function
    //props["hasSmooth"]   	= false; does not work
    props["address"] 		  = ".ProcessInput.WarnCoSt";
    props["dataType"] 		= CPC_UINT16;
    config["WarnCoSt"] 		= props;	
// 8
    mappingClear(props);
    //props["hasSmooth"]   	= false; does not work
	  //props["hasArchive"]   = true;
    props["address"] 		  = ".ProcessInput.AuOpMoSt";
    props["dataType"] 		= CPC_FLOAT;
    config["AuOpMoSt"] 		= props;	
// 9
    mappingClear(props);
    //props["hasSmooth"]   	= false; does not work
	  //props["hasArchive"]   = true; archive setting done in custom function
    props["address"] 		  = ".ProcessInput.OpMoSt";
    props["dataType"] 		= CPC_FLOAT;
    config["OpMoSt"] 		= props;	
// 10
    mappingClear(props);
	  //props["hasArchive"]   = true;
    props["address"] 		  = ".ProcessInput.CuStepSt";
    props["dataType"] 		= CPC_INT32;
    config["CuStepSt"] 		= props;
// 11
    mappingClear(props);
	  //props["hasArchive"]   = true;
    props["address"] 		  = ".ProcessInput.CuTrSt";
    props["dataType"] 		= CPC_INT32;
    config["CuTrSt"] 		= props;	
// 12
    mappingClear(props);
    props["address"] 		  = ".ProcessInput.StartISt";
    props["dpe"]			    = "StsReg01";
    props["bitPosition"]	= CPC_StsReg01_STARTIST;
    props["isAlarm"]		  = true;
    config["StartISt"] 		= props;
// 13
    mappingClear(props);
    props["address"] 		  = ".ProcessInput.TStopISt";
    props["dpe"]			    = "StsReg01";
    props["bitPosition"]	= CPC_StsReg01_TSTOPIST;
    props["isAlarm"]		  = true;
    config["TStopISt"] 		= props;
// 14
    mappingClear(props);
    props["address"] 		  = ".ProcessInput.FuStopISt";
    props["dpe"]			    = "StsReg02";
    props["bitPosition"]	= CPC_StsReg01_FUSTOPIST;
    props["isAlarm"]		  = true;
    config["FuStopISt"] 	= props;
// 15
    mappingClear(props);
    props["address"]      = ".ProcessInput.AlSt";
    props["dpe"]          = "StsReg01";
    props["bitPosition"]  = CPC_StsReg01_ALST;
    props["isAlarm"]      = true;
    config["AlSt"]        = props;
// 16
    mappingClear(props);
    props["address"]      = ".ProcessInput.AlUnAck";
    props["dpe"]          = "StsReg01";
    props["bitPosition"]  = CPC_StsReg01_ALUNACK;
    config["AlUnAck"]     = props;
// 17
    mappingClear(props);
    props["address"] 		  = ".ProcessOutput.ManReg01";
    props["dataType"] 		= CPC_UINT16;
    config["ManReg01"] 		= props;
// 18
    props["address"] 		  = ".ProcessOutput.MOpMoR";
    props["dataType"] 		= CPC_FLOAT;
    config["MOpMoR"] 		  = props;



//end_TAG_SCRIPT_DEVICE_TYPE_deviceConfig

    return config;
}

/**
Returns the list of parameter names that passed via config line.
*/
dyn_string CPC_VP_GConfig_getParamNames() {
//begin_TAG_SCRIPT_DEVICE_TYPE_getParamNames
    // TODO add gauge?
    return makeDynString(CPC_PARAMS_MRESTART, CPC_PARAMS_RSTARTFS, CPC_PARAMS_VVP,CPC_PARAMS_VPT, CPC_PARAMS_VVI,CPC_PARAMS_VVD, CPC_PARAMS_VVT,CPC_PARAMS_VVP, CPC_PARAMS_VGF,CPC_PARAMS_VVR1,CPC_PARAMS_VVR2,CPC_PARAMS_VVR3);
//end_TAG_SCRIPT_DEVICE_TYPE_getParamNames
}

/** Check custom configuration
*/
CPC_VP_GConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
  for (int i = UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_1; i <= UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_8; i++) {
    string allowance = configLine[UN_CONFIG_COMMON_LENGTH + i];
    if ((strlen(allowance) != 0) && (strlen(allowance) != 8)) {
      fwException_raise(exceptionInfo, "ERROR", "CPC_VP_GConfig_checkCustomConfig: " + getCatStr("unGeneration", "BADALLOWANCE"), "");
    } else {
      strreplace(allowance, "1", "");
      strreplace(allowance, "0", "");
      if (allowance != "") {
        fwException_raise(exceptionInfo, "ERROR", "CPC_VP_GConfig_checkCustomConfig: " + getCatStr("unGeneration", "BADALLOWANCE"), "");
      }
    }
  }
  
//end_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
}

/** Set custom configuration
*/
CPC_VP_GConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig
  cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.TStopISt");
  cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.FuStopISt");
  cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.StartISt");
  cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.AlSt");

	//unGenericDpFunctions_setKeyDeviceConfiguration(sDeviceDpName, UN_CPC_DEVICE_PROPERTY_PREFIX + propertyName, makeDynString(propertyValue), exceptionInfo);
	//int iConfigDataIndex = UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_ARCHIVE_LENGTH + UN_CONFIG_CPC_VP_G_LENGTH + UN_CONFIG_CPC_VP_G_ERRCOST_MAPPING;
	//unGenericDpFunctions_setKeyDeviceConfiguration(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], "DESC_ErrCoSt", dsConfigs[iConfigDataIndex], exceptionInfo);
  
  int iRes;
  CPC_ProcessControlObject_setOptionModes(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME],
                                            makeDynString(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_NAME_1],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_NAME_2],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_NAME_3],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_NAME_4],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_NAME_5],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_NAME_6],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_NAME_7],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_NAME_8]),
                                            makeDynString(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_1],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_2],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_3],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_4],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_5],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_6],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_7],
                                                    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_G_MODE_ALLOWANCE_8]),
                                            iRes);
  if (iRes < 0) {
    fwException_raise(exceptionInfo, "ERROR", "_UnPlc_CPC_ProcessControlObject_setConfig: set option modes failed", "");
  }

	//Archive setting for ObjectSt and ErrCoSt
	//fwArchive_set(string dpe, string archiveClassName, int archiveType, int smoothProcedure, float deadband, float timeInterval, dyn_string &exceptionInfo, bool checkClass = TRUE)
  
	fwArchive_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.ObjectSt",
                  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
                  DPATTR_ARCH_PROC_SIMPLESM,
                  DPATTR_COMPARE_OLD_NEW,
                  0,
                  0,
                  exceptionInfo);
	fwArchive_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.ErrCoSt",
                  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
                  DPATTR_ARCH_PROC_SIMPLESM,
                  DPATTR_COMPARE_OLD_NEW,
                  0,
                  0,
                  exceptionInfo);
  
  fwArchive_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.WarnCoSt",
                  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
                  DPATTR_ARCH_PROC_SIMPLESM,
                  DPATTR_COMPARE_OLD_NEW,
                  0,
                  0,
                  exceptionInfo);
  fwArchive_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.OpMoSt",
                  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
                  DPATTR_ARCH_PROC_SIMPLESM,
                  DPATTR_COMPARE_OLD_NEW,
                  0,
                  0,
                  exceptionInfo);
	
//end_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig
}

/**
Purpose: Export CPC_VP_G Devices

Usage: External function

PVSS manager usage: NG, NV
*/
CPC_VP_GConfig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_export
    string sObject = UN_CONFIG_CPC_VP_G_DPT_NAME;
    int amount = dynlen(dsDpList);
    for (int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if (!dpExists(currentDP)) { continue; }

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);

        cpcExportGenericFunctions_getDigitalNormalPosition(currentDP, sObject, makeDynString(".ProcessInput.StartISt", ".ProcessInput.TStopISt", ".ProcessInput.FuStopISt"), dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg02", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg02", true, dsDpParameters);
 		    cpcExportGenericFunctions_processAddress(currentDP, sObject, "AuOpMoSt",  true, dsDpParameters);
 		    cpcExportGenericFunctions_processAddress(currentDP, sObject, "OpMoSt",    true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "CuStepSt",  true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "CuTrSt",  true, dsDpParameters);
		    cpcExportGenericFunctions_processAddress(currentDP, sObject, "ObjectSt",  true, dsDpParameters);
		    cpcExportGenericFunctions_processAddress(currentDP, sObject, "ErrCoSt",   true, dsDpParameters);
 		    cpcExportGenericFunctions_processAddress(currentDP, sObject, "WarnCoSt",  true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "MOpMoR", 		false, dsDpParameters);
        
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString("ProcessInput.OnSt", "ProcessInput.ObjectSt", "ProcessInput.ErrCoSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt", "ProcessInput.MotPosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01", "ProcessInput.evStsReg02"), dsDpParameters);
        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_export

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

/**
Purpose: Export CPC_VP_G Devices for S7_PLC front-end

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_CPC_VP_G_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VP_GConfig_ExportConfig(dsDpList, exceptionInfo);
}

/**
Purpose: Export CPC_VP_G Devices for _UnPlc front-end

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_CPC_VP_G_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VP_GConfig_ExportConfig(dsDpList, exceptionInfo);
}

OPCUA_CPC_VP_G_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VP_GConfig_ExportConfig(dsDpList, exceptionInfo);
}

BACnet_CPC_VP_G_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VP_GConfig_ExportConfig(dsDpList, exceptionInfo);
}

//@}
