/**
@file
// cpcVP_I.ctl
This library contains the widget, faceplate, etc. functions of VP_I.

@par Creation Date
  10/10/2016

@par Modification History

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author
  T. van Winden (TE-VSC)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{

#uses "vacuumExtension/classes/DeviceListCriteria.ctl"


/** Returns the list of VP_I DPE with alarm config that can be acknowledged

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name DP name
@param dpType input, device type
@param dsNeedAck output, the lsit of DPE
*/
CPC_VP_I_AcknowledgeAlarm(string deviceName, string dpType, dyn_string &dsNeedAck) {
//begin_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
    dsNeedAck = makeDynString("ProcessInput.StartISt", "ProcessInput.TStopISt", "ProcessInput.FuStopISt", "ProcessInput.AlSt", UN_ACKNOWLEDGE_PLC);
//end_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
}

/** Function called from snapshot utility of the treeDeviceOverview to get the time and value

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name
@param deviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
CPC_VP_I_ObjectListGetValueTime(string deviceName, string deviceType, dyn_string &dsReturnData) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
    float value;
    string sTime;

    dpGet(deviceName + ".ProcessInput.PosSt", value,
          deviceName + ".ProcessInput.PosSt:_online.._stime", sTime);

    dsReturnData[1] = sTime;
    dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(deviceName + ".ProcessInput.PosSt"), value) + " " + dpGetUnit(deviceName + ".ProcessInput.PosSt"); // formated value with unit
//end_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
}





// POPUP MENU
//--------------------------------

/** pop-up menu

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param dsAccessOk input, the access control
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
CPC_VP_I_MenuConfiguration(string deviceName, string dpType, dyn_string dsAccessOk, dyn_string &menuList) {
//begin_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
    dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT, UN_POPUPMENU_TREND_TEXT, CPC_POPUPMENU_ALLOW_TO_RESTART_TEXT);
   
    // Reverse order for proper ordering in context menu
    mapping mDpe;
    mDpe["Voltage"] = "VPosSt";
    mDpe["Current"] = "APosSt";
    mDpe["Pressure"] = "PosSt";
    
    // Add device context menu items
    unVacObject_addAliasActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    unVacObject_addAnalogActionToMenu(deviceName, dpType, mDpe, dsAccessOk, menuList);
    
    unVacObject_addDisableEnableActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    unVacObject_addItemToPopupMenu(menuList, "Controller", CPC_POPUPMENU_CONTROLLER, true);
    int iPos = unVacObject_addVacuumExtensionActionToMenu(deviceName, dpType,  dsAccessOk, menuList);
    iPos = unVacObject_addProcessActionToMenu(deviceName, dpType,  dsMenuConfig, dsAccessOk, menuList, iPos+1);
  
//end_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
}

/** handle the answer of the popup menu

@par Constraints
  None
  
@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param menuList input, the access control
@param menuAnswer input, selected menu value
*/
CPC_VP_I_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer) {
    dyn_string exceptionInfo;
    //begin_TAG_SCRIPT_DEVICE_TYPE_HandleMenu
    
    // The help file
    string helpFile = unVacObject_getHelpLink(dpType);
    
    string controllerName = unVacObject_getControllerName(deviceName);
    
    // Handle the specific to this device 
    switch(menuAnswer){
      case CPC_POPUPMENU_CONTROLLER:
        if(controllerName != ""){
          string sDpName = unGenericDpFunctions_getWidgetDpName(unGenericDpFunctions_getSystemName(deviceName) + controllerName);
          unVacObject_OpenFaceplate(sDpName, makeDynString("$sRefDpName:"+deviceName), exceptionInfo);
        }
      break;
      // Use default vacuum handling. If not found use default UNICOS handling
      default:
      if(!unVacObject_handleCustomUnicosMenu(deviceName, dpType, menuList, menuAnswer, helpFile, exceptionInfo)){
        cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
      }
    }
    
    if (dynlen(exceptionInfo) > 0) {
        unSendMessage_toExpertException("Right Click", exceptionInfo);
    }
}

/** Init static values which are used in widget animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_VP_I_WidgetInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
    dyn_string exceptionInfo;
    g_params[CPC_PARAMS_HALFFULLLIMIT] = (float) cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_HALFFULLLIMIT, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
}

/** 
Returns the list of VP_I DPEs which should be connected on widget animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_VP_I_WidgetDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
    dynAppend(dpes, deviceName + ".eventMask");                                // 6
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");                    // 7
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");  // 8
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");                    // 9
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");  // 10
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");                    // 11
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid");  // 12
    dynAppend(dpes, deviceName + ".ProcessInput.PosSt");                       // 13
    dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_online.._invalid");     // 14
    dynAppend(dpes, deviceName + ".disabled");                                 // 15
    
    if (g_bBoolAlert) {
        dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");  // 16
        dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");  // 17
        dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state"); // 18
        dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");      // 19
    }
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
}

/** callback function on the device data

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

Datapoint elements and related values
  1  int	    frontend.alarm
  2  bool 		frontend.enabled
  3  bool    	device.statusInformation.selectedManager:_lock._original._locked,
  4  string  	device.statusInformation.selectedManager,
  5  bool    	device.statusInformation.deviceLogActive
  6  int 		  device.eventMask
  7  bit32		device.ProcessInput.StsReg01
  8  bool 		device.ProcessInput.StsReg01:_online.._invalid
  9  bit32		device.ProcessInput.StsReg02
  10 bool		  device.ProcessInput.StsReg02:_online.._invalid
  11 float		device.ProcessInput.PosSt
  12 bool		  device.ProcessInput.PosSt:_online.._invalid
  13 float		device.ProcessInput.APosSt
  14 bool		  device.ProcessInput.APosSt:_online.._invalid
  15 float		device.ProcessInput.VPosSt
  16 bool		  device.ProcessInput.VPosSt:_online.._invalid
  17 int		  device.ProcessInput.ObjectSt
  18 int		  device.ProcessInput.ObjectSt:_online.._invalid 
  19 bool		  device.disabled
  20 int		  device.ProcessInput.StartISt:_alert_hdl.._act_state
  21 int		  device.ProcessInput.TStopISt:_alert_hdl.._act_state
  22 int		  device.ProcessInput.FuStopISt:_alert_hdl.._act_state
  23 int		  device.ProcessInput.AlSt:_alert_hdl.._act_state	
  
  @see For the stsReg bit meaning check the specifications file: https://svn.cern.ch/reps/en-ice-svn/trunk/applications/vacuum/UNICOS_VAC_DEVICES/Design/Object_ManReg_StsReg_Mapping.xls
  @see Other usufull documentation: https://edms.cern.ch/ui/file/1228441/1.6.0/UCPC6_RP1.6.0_Manual.pdf

*/
CPC_VP_I_WidgetAnimation(dyn_string dpes, dyn_anytype values, string sWidgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation

// For the stsReg bit meaning check the specifications file: https://svn.cern.ch/reps/en-ice-svn/trunk/applications/vacuum/UNICOS_VAC_DEVICES/Design/Object_ManReg_StsReg_Mapping.xls
// Other usufull documentation: https://edms.cern.ch/ui/file/1228441/1.6.0/UCPC6_RP1.6.0_Manual.pdf

  // Do nothing if system is not connected  
	if (!g_bSystemConnected){
		return;
	}
  
// INIT VARIABLES
//----------------
  int iSystemIntegrityAlarmValue = values[1];
  bool bSystemIntegrityAlarmEnabled = values[2];
  bool bLocked = values[3];
  string sSelectedManager = values[4];
  bool bComment = values[5];
  int iMask = values[6];
  bit32 bit32StsReg01 = values[7];
  bit32 bit32StsReg02 = values[9];
  int iObjectSt = values[11];
  float fPosSt = values[13];

  // Determine operation status animation
  bool bInvalid = values[8] && values[10] && values[12] && values[14];

  bool bOn = (iObjectSt == 3 || iObjectSt == 2);  
  bool bIsRestartNeeded = g_bMRestart && getBit(bit32StsReg02, CPC_StsReg02_NEEDRESTART) == 0;
  bool bCompact = sWidgetType == "VP_I";
   
  // Bits loading based on the type of widget displayed
  dyn_string dsWarningBits = (bCompact) ? VAC_WARNING_BITS : CPC_WARNING_BITS;
  dyn_string dsAlarmBits = (bCompact) ? VAC_ALARM_BITS : CPC_ALARM_BITS;
  dyn_string dsControlBits = (bCompact) ? VAC_CONTROL_BITS : CPC_CONTROL_BITS;
 
  // Create list the status registers
  dyn_bit32 dbit32StsRegs = makeDynBit32(bit32StsReg01, bit32StsReg02);
    
   // Generic vacuum animation
	unVacObject_WidgetLetterAnimation(sWidgetType, UN_CONFIG_CPC_VP_I_DPT_NAME,
			bLocked, sSelectedManager, dbit32StsRegs,
      unVacObject_fetchBitsFromStsRegs(dbit32StsRegs, dsWarningBits),    // Default vacuum warning bits
      CPC_WARNING_LETTERS, // Default vacuum warning letters
      unVacObject_fetchBitsFromStsRegs(dbit32StsRegs, dsAlarmBits),     // Default vacuum alarm bits
      CPC_ALARM_LETTERS, // Default vacuum alarm letters
      unVacObject_fetchBitsFromStsRegs(dbit32StsRegs, dsControlBits) ,    // Default vacuum control bits
      CPC_CONTROL_LETTERS, // Default vacuum control letters
			bInvalid, bIsRestartNeeded, bComment,
			iSystemIntegrityAlarmValue, bSystemIntegrityAlarmEnabled);
  				  
  // Determine the body color based on the state
  dyn_int diKeyFilter = makeDynInt(3, 4, 5, 13, 14); 
  // Filter dpes based on index to be consistent with the StateHistory dpe's
  string sBodyColor, sState;
  dyn_string filteredDpes = unVacObject_dynWithoutIndex(dpes, diKeyFilter);
  dyn_anytype filteredValues = unVacObject_dynWithoutIndex(values, diKeyFilter);
  CPC_VP_I_ColorAndString("", filteredDpes, filteredValues,
      sBodyColor,
      sState,
      false,
      g_bBoolAlert,
      bIsRestartNeeded      
  );  
  
  // Determine callback name
	string sFunction = "CPC_"+sWidgetType+"_WidgetBodyAnimation";
	if(!isFunctionDefined(sFunction) || sFunction == ""){
		sFunction = "CPC_VP_I_WidgetBodyAnimation";	
	} 
  
  // OBJECT ANIMATION (animation according to ObjectSt dpe)
  callFunction(sFunction, bOn, false, sBodyColor, fPosSt);
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
}

/**
@author Thijs van Winden

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI
@param bOn Determine if the device is on
@param bUndefined Undefined state of the device
@param sBodyColor The body color of the device
*/
CPC_VP_I_WidgetBodyAnimation(bool bOn, bool bHalf, string sBodyColor, float fPosSt){
  setMultiValue("BodyLineExt1", "foreCol", sBodyColor);
  if(bHalf){    
      setMultiValue(
        "BodyFillBottom", "foreCol", sBodyColor,
		    "BodyFillBottom", "backCol", sBodyColor,
    		"BodyRoofRight", "foreCol", sBodyColor,
    		"BodyRoofLeft", "foreCol", sBodyColor,
    		"BodyLineInt1", "foreCol", sBodyColor,
        "BodyLineInt2", "foreCol", "unWidget_Background",
        "BodyLineInt3", "foreCol", sBodyColor,
        "BodyLineInt4", "foreCol", "unWidget_Background",
     		"BodyLineExt1", "backCol", "unWidget_Background"
      );
  } else if(bOn){
      setMultiValue(
        "BodyFillBottom", "foreCol", "_Transparent",
 		    "BodyFillBottom", "backCol", "_Transparent",
    		"BodyRoofRight", "foreCol", "unWidget_Background",
    		"BodyRoofLeft", "foreCol", "unWidget_Background",
    		"BodyLineInt1", "foreCol", "unWidget_Background",
        "BodyLineInt2", "foreCol", "unWidget_Background",
        "BodyLineInt3", "foreCol", "unWidget_Background",
        "BodyLineInt4", "foreCol", "unWidget_Background",
     		"BodyLineExt1", "backCol", sBodyColor
  	);
  } else {
    setMultiValue(
        "BodyFillBottom", "foreCol", "_Transparent",
 		    "BodyFillBottom", "backCol", "_Transparent",
    		"BodyRoofRight", "foreCol", sBodyColor,
    		"BodyRoofLeft", "foreCol", sBodyColor,
    		"BodyLineInt1", "foreCol", sBodyColor,
        "BodyLineInt2", "foreCol", sBodyColor,
        "BodyLineInt3", "foreCol", sBodyColor,
        "BodyLineInt4", "foreCol", sBodyColor,
     		"BodyLineExt1", "backCol", "unWidget_Background"
  	);      
  }
  
  if(!shapeExists("PosSt")) return;
  // Animate PosSt if the shape exists
  CPC_VP_I_Analog_WidgetBodyAnimation(bOn, bHalf, sBodyColor, fPosSt);
  
      
}


/**
  
Widget body animation for analog widget

@author Thijs van Winden

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI
@param bOn Determine if the device is on
@param bUndefined Undefined state of the device
@param sBodyColor The body color of the device
@param fPosSt The current analog status of the pump
*/
void CPC_VP_I_Analog_WidgetBodyAnimation(bool bOn, bool bHalf, string sBodyColor, float fPosSt) {     
  string sPosSt = (!bOn) ? "Off" : unGenericObject_FormatValue(CPC_VAC_ANALOG_FORMAT, fPosSt); // + " " + g_sPosStUnit;
  setMultiValue(
      "PosSt",            "foreCol",  sBodyColor,             
      "PosSt",            "text",  sPosSt,
      "WidgetArea",       "visible",  true
  );
}
  

/** Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.

*/
CPC_VP_I_WidgetDisconnection(string sWidgetType) {
    //begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
    string sFunction = "CPC_"+sWidgetType+"_WidgetBodyAnimation";
  	if(!isFunctionDefined(sFunction) || sFunction == ""){
  		sFunction = "CPC_VG_PT_WidgetBodyAnimation";	
  	} 
  
    // OBJECT ANIMATION (animation according to ObjectSt dpe)
    callFunction(sFunction, false, false, "unDataNoAccess", 0.0);

    setMultiValue(
   	  "WarningText", "text", "", "AlarmText", "text", "", "ControlStateText", "text", "",
                  "SelectArea", "foreCol", "", "LockBmp", "visible", false, "WidgetArea", "visible", false,
                  "eventState", "visible", false);
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
}




// BUTTONS IN FACEPLATE
//--------------------------------

/** Return button configuration including access level

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
mapping CPC_VP_I_ButtonConfig(string deviceName) {
    mapping buttons;
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
    dyn_string exceptionInfo;

	  buttons[UN_FACEPLATE_BUTTON_RESETERRWARN_REQUEST] = UN_ACCESS_RIGHTS_EXPERT; 
	  buttons[UN_FACEPLATE_BUTTON_OPERATION_MODES] = UN_ACCESS_RIGHTS_OPERATOR; 

    buttons[CPC_FACEPLATE_BUTTON_ALLOW_RESTART] = UN_ACCESS_RIGHTS_OPERATOR;
    unVacObject_genericButtonConfig(deviceName, buttons);
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
    return buttons;
}

/** Configure the list of dpes that needs for buttons animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the name of device
@param dpes input, the list of dpes to connect
*/
CPC_VP_I_ButtonDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
}

/** Set the state of the contextual button of the device

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param dpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
CPC_VP_I_ButtonSetState(string deviceName, string dpType, dyn_string dsUserAccess, dyn_string dsData) {
    bool bSelected, buttonEnabled, bit1, bit2, bit3;
    string localManager;
    dyn_string exceptionInfo;

    bool bLocked = dsData[1];
    string sSelectedUIManager = dsData[2];

    localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
    bSelected = (localManager == "S"); // Selection state
    dyn_string dsButtons = unSimpleAnimation_ButtonList(deviceName);

//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
    mixed stsReg01Value = dsData[3];
    bool bStsReg01Bad = dsData[4];

    for (int i = 1; i <= dynlen(dsButtons); i++) {
        buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0); // User access
        switch (dsButtons[i]) {
            case UN_FACEPLATE_BUTTON_FORCED_MODE:
                bit1 = !(getBit(stsReg01Value, CPC_StsReg01_FOMOST) == 0);
                bit2 = !(getBit(stsReg01Value, CPC_StsReg01_AUINHFMO) == 0);
                buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2) && !bStsReg01Bad;
                break;
            case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
            //case UN_FACEPLATE_BUTTON_ONFORCED_REQUEST: // 
            case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
            case UN_FACEPLATE_BUTTON_AUTO_MODE:
            case UN_FACEPLATE_BUTTON_SET_VALUE:
                bit1 = !(getBit(stsReg01Value, CPC_StsReg01_AUMOST) == 0);
                buttonEnabled = buttonEnabled && bSelected && !(bit1) && !bStsReg01Bad;
                break;
            case UN_FACEPLATE_BUTTON_MANUAL_MODE:
                buttonEnabled = buttonEnabled && bSelected &&
                                !getBit(stsReg01Value, CPC_StsReg01_MMOST) &&
                                !getBit(stsReg01Value, CPC_StsReg01_AUINHMMO) &&
                                !bStsReg01Bad;
                break;                
            case UN_FACEPLATE_BUTTON_OPERATION_MODES:
                buttonEnabled = buttonEnabled && bSelected && 
                                getBit(stsReg01Value, CPC_StsReg01_MMOST) && 
                                !bStsReg01Bad;                
                break;
			case UN_FACEPLATE_BUTTON_RESETERRWARN_REQUEST:
				buttonEnabled = buttonEnabled && bSelected &&
                                !bStsReg01Bad;
            default:
                break;
        }
        if (dsButtons[i] == UN_FACEPLATE_BUTTON_SELECT) {
            unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
        } else {
            cpcButton_setButtonState(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], buttonEnabled);
        }
    }
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
}






// FACEPLATE
//--------------------------------



/** Init static values which are used in faceplate animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_VP_I_FaceplateInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
    int iAlertType1, iAlertType2, iAlertType3, iAlertType4, iRangeType;
    dyn_string exceptionInfo;
    int iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
                     deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
                     deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
                     deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
    g_params["BoolAlert"] = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));
        
    g_params["MRestart"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);	
	  g_params["PumpModel"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_INSTRUMENT_MODEL, exceptionInfo);
	  g_params["ControllerModel"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_CONTROLLER_MODEL, exceptionInfo);
    g_params["Controller"] = unVacObject_getControllerName(deviceName);
    
    // Update the faceplate once    
    setMultiValue(
        "ControllerBtn", "enabled", g_params["Controller"] != "",
        "ControllerBtn", "toolTipText", g_params["Controller"],
        "PumpModel", "text", g_params["PumpModel"],
        "ControllerModel", "text", g_params["ControllerModel"]
  );
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
}



/** Returns the list of VP_I DPEs which should be connected on faceplate animation
// dyn_string dpes value[1]: event Mask
// dyn_string dpes value[2]: StsReg01
// dyn_string dpes value[1]: StsReg01:_online.._invalid
// dyn_string dpes value[1]: event Mask
// dyn_string dpes value[1]: event Mask
// dyn_string dpes value[1]: event Mask

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_VP_I_FaceplateDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
    dynAppend(dpes, deviceName + ".eventMask");  										
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.PosSt");
    dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.APosSt");
    dynAppend(dpes, deviceName + ".ProcessInput.APosSt:_online.._invalid");    
    dynAppend(dpes, deviceName + ".ProcessInput.VPosSt");
    dynAppend(dpes, deviceName + ".ProcessInput.VPosSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid");
    if (g_params["BoolAlert"]) {
        dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");
    }
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
}


/** callback function on the device data

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
CPC_VP_I_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
  
    // Device online value animation. PostSt, APosSt, VPosSt
    unVacObject_animateFaceplateOnlineValues(dpes, values, makeDynString("PosSt", "APosSt", "VPosSt"));
      
    // Device status animation
    unVacObject_animateFaceplateObjectStatus(dpes, values);

    // Do the on/off animation based on the object st
    if (dynlen(dpes) > 0) { // online animation
      bool invalid, onSt;
      int value;
      unFaceplate_fetchAnimationCBOnlineValue(dpes, values, "ObjectSt", value, invalid);
      unGenericObject_ColorBoxAnimate("ONST", (value==2 || value==3), true, "cpcColor_Faceplate_Status", invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
      unGenericObject_ColorBoxAnimate("OFFST", !(value==2 || value==3), true, "cpcColor_Faceplate_Status", invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
    } else {
      unGenericObject_ColorBoxDisconnect(makeDynString("ONST", "OFFST"));
    }
    
    // Animates operation modes, like auto, manual, forced and their inhibit modes
    unVacObject_animateFaceplateOperationModes(dpes, values);
    
    // Request status animation
    unVacObject_animateStatusMultiBit(dpes, values, makeDynString("CPC_StsReg02_MSTPMRST", "CPC_StsReg02_AUSTPMRST"), true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_OUTONOVST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_KV3RST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_KV5RST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_KV7RST", true, "cpcColor_Faceplate_Status");
	
    // Animates auto on/off/manual/force mode request status
    unVacObject_animateFaceplateRequestStatus(dpes, values);
    
    // Animates the start, stop, full stop, alarm and acknowledge interlocks
    unVacObject_animateFaceplateInterlocks(dpes, values);
    
    // Default UNICOS animations
    cpcGenericObject_animateMaskEvent(dpes, values);
    cpcGenericObject_animateNeedRestart(dpes, values);
    
    // Warning animations
    unVacObject_animateFaceplateWarnings(dpes, values);
    
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
}

/**
  Function to load the possible operation modes
  @par Constraints
  None

  @par Usage
  Public

  @par PVSS managers
  Ui, CTRL
  
  @return dyn_string List of operation modes
*/
dyn_string CPC_VP_I_getOperationModes(){
  dyn_string modes;
  dynAppend(modes, CPC_VP_I_MODE_FIXED);
  dynAppend(modes, CPC_VP_I_MODE_STEP);
 
  return modes;
}

/** Function to load the current operation modes
@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL
@param string deviceName The name of the current device
@return string Current operation mode name
*/
string CPC_VP_I_getCurrentOperationMode(string deviceName){
  uint stsReg02, stsReg01;
  dpGet(deviceName + ".ProcessInput.StsReg01",stsReg01);
  dpGet(deviceName + ".ProcessInput.StsReg02",stsReg02);
  if(getBit(stsReg01, CPC_StsReg01_KV3RST) == 1 || 
     getBit(stsReg02, CPC_StsReg02_KV5RST) == 1 || 
     getBit(stsReg02, CPC_StsReg02_KV7RST) == 1 ||
     getBit(stsReg02, CPC_StsReg02_MFIXMRST) == 1 ||
     getBit(stsReg02, CPC_StsReg02_AUFIXMRST) == 1){
    return CPC_VP_I_MODE_FIXED;
  } else if (getBit(stsReg02, CPC_StsReg02_MSTPMRST) == 1 ||
             getBit(stsReg02, CPC_StsReg02_AUSTPMRST) == 1){
    return CPC_VP_I_MODE_STEP;
  }
 
  return "";
}

/** Function to load the power modes
@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL
@param string deviceName The name of the current device
@return dyn_string The power mode names
*/

dyn_string CPC_VP_I_getFixedModes(){
  dyn_string modes;
  dynAppend(modes, CPC_VP_I_MODE_FIXED_3KV);
  dynAppend(modes, CPC_VP_I_MODE_FIXED_5KV);
  dynAppend(modes, CPC_VP_I_MODE_FIXED_7KV);
 
  return modes;
}

/** Function to load the current power mode
@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL
@param string deviceName The name of the current device
@return string Current power mode name
*/

string CPC_VP_I_getCurrentFixedMode(string deviceName){
  uint stsReg02, stsReg01;
  dpGet(deviceName + ".ProcessInput.StsReg01",stsReg01);
  dpGet(deviceName + ".ProcessInput.StsReg02",stsReg02);
  if(getBit(stsReg01, CPC_StsReg01_KV3RST) == 1){
    return CPC_VP_I_MODE_FIXED_3KV;
  } else if (getBit(stsReg02, CPC_StsReg02_KV5RST) == 1){
    return CPC_VP_I_MODE_FIXED_5KV;
  } else if (getBit(stsReg02, CPC_StsReg02_KV7RST) == 1){
    return CPC_VP_I_MODE_FIXED_7KV;
  }
 
  return "";
}


/** Set the current operation mode
@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL
@param string deviceName The name of the current device
@return string Current power mode name
*/
bool CPC_VP_I_setOperationMode(string deviceName, string operationalMode, string fixedMode, dyn_string exceptionInfo, bool alreadySent){
  if(operationalMode == CPC_VP_I_MODE_FIXED){
    	unGenericObject_SendManRegPulse(deviceName, CPC_ManReg01_MFIXMR, exceptionInfo, alreadySent);
      if(fixedMode == CPC_VP_I_MODE_FIXED_3KV){
       	unGenericObject_SendManRegPulse(deviceName, CPC_ManReg01_M3KVR, exceptionInfo, alreadySent);
      } else if(fixedMode == CPC_VP_I_MODE_FIXED_5KV){
       	unGenericObject_SendManRegPulse(deviceName, CPC_ManReg01_M5KVR, exceptionInfo, alreadySent);
      } else if(fixedMode == CPC_VP_I_MODE_FIXED_7KV){
       	unGenericObject_SendManRegPulse(deviceName, CPC_ManReg01_M7KVR, exceptionInfo, alreadySent);        
      }
   } else if (operationalMode == CPC_VP_I_MODE_STEP){
     	unGenericObject_SendManRegPulse(deviceName, CPC_ManReg01_MSTPMR, exceptionInfo, alreadySent);         
   }
}
//@}



/**
  Lists the dpes required for building the state of the device
  
  @param string sDeviceName The name of the device which is in UNICOS the same as the dpname
  @param dyn_string dsDpes The list of datapoints elements required for building the state
  @param bool bForHistory The DPEs will be used for state history, i.e. only DPEs with _archive
                            config are of interest
  
*/
CPC_VP_I_StateDPEs(string sDeviceName, dyn_string &dsDpes, bool bForHistory = true) { 
    dynAppend(dsDpes, sDeviceName + ".eventMask");                                // 3
    if(bForHistory) {
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg01");                  // 4
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg01:_online.._invalid");// 5
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg02");                  // 6
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg02:_online.._invalid");// 7
    }
    else {
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg01");                  // 4
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg01:_online.._invalid");// 5
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg02");                  // 6
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg02:_online.._invalid");// 7
    }
    dynAppend(dsDpes, sDeviceName + ".ProcessInput.ObjectSt");                    // 8
    dynAppend(dsDpes, sDeviceName + ".ProcessInput.ObjectSt:_online.._invalid");  // 9
    dynAppend(dsDpes, sDeviceName + ".disabled");                                 // 10
    
    if (g_bBoolAlert) {
        dynAppend(dsDpes, sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");  // 11
        dynAppend(dsDpes, sDeviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");  // 12
        dynAppend(dsDpes, sDeviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state"); // 13
        dynAppend(dsDpes, sDeviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");      // 14
    }
}

/**
  Returns the main color associated with the device state. This color is based on 
  the dpes listed in the CPC_VV_S_StateDPEs for the dedicated device type. In case of the 
  state history the EvStsReg## is used because this is archived, while for determining 
  the current state the StsReg## is used.

  @param dyn_string dsDpes The list of dpe names used to determine the color
  @param dyn_anytype daValues The list of dpe values used to determine the color
  
  Datapoint elements and related values
  1  int	    frontend.alarm
  2  bool 		frontend.enabled
  3  bit32		device.ProcessInput.StsReg01
  4  bool 		device.ProcessInput.StsReg01:_online.._invalid
  5  bit32		device.ProcessInput.StsReg02
  6  bool		  device.ProcessInput.StsReg02:_online.._invalid
  7  float		device.ProcessInput.PosSt
  8  bool		  device.ProcessInput.PosSt:_online.._invalid
  9  float		device.ProcessInput.APosSt
  10 bool		  device.ProcessInput.APosSt:_online.._invalid
  11 float		device.ProcessInput.VPosSt
  12 bool		  device.ProcessInput.VPosSt:_online.._invalid
  13 int		  device.ProcessInput.ObjectSt
  14 int		  device.ProcessInput.ObjectSt:_online.._invalid 
  15 bool		  device.disabled
  16 int		  device.ProcessInput.StartISt:_alert_hdl.._act_state
  17 int		  device.ProcessInput.TStopISt:_alert_hdl.._act_state
  18 int		  device.ProcessInput.FuStopISt:_alert_hdl.._act_state
  19 int		  device.ProcessInput.AlSt:_alert_hdl.._act_state	
  
  @param string sColor The color of the widget/state box
  @param string sString The main state string corresponding to the object state
  @param bool bState Whether this color is used for the StateHistory or the widget color, only supplied in case of WidgetColor
  @param bool bAlert Whether there is a realtime alert according to UNICOS alert manager, only supplied in case of WidgetColor
  @param bool bRestart Whether there is a restart needed according to UNICOS device manager, only supplied in case of WidgetColor  
  
  @see For the stsReg bit meaning check the specifications file: https://svn.cern.ch/reps/en-ice-svn/trunk/applications/vacuum/UNICOS_VAC_DEVICES/Design/Object_ManReg_StsReg_Mapping.xls
  @see Other usufull documentation: https://edms.cern.ch/ui/file/1228441/1.6.0/UCPC6_RP1.6.0_Manual.pdf

  
  @return The main color corresponding to the state of the object
*/

int CPC_VP_I_ColorAndString(string sDeviceName, dyn_string dsDpes, dyn_anytype daValues,
                               string &sColor, string &sState, bool bUseOffColor = true,
                               bool bBoolAlert = false, bool bIsRestartNeeded = false){   

  // --- Get device data value
  // --------------------------	   
  // Read system integrity values   
  int iSystemIntegrityAlarmValue = daValues[1];
  bool bSystemIntegrityAlarmEnabled = daValues[2];
  
  // Object status
  bit32 bit32StsReg01 = daValues[4];
  bit32 bit32StsReg02 = daValues[6];
  int iObjectSt = daValues[8];
  bool bDisabled = daValues[10];  
  // Software Interlock status
  int iStartIStState, iTStopIStState, iFuStopIStState, iAlSt;
  if (bBoolAlert) {
    iStartIStState = daValues[11];
    iTStopIStState = daValues[12];
    iFuStopIStState = daValues[13];
    iAlSt = daValues[14];
  }
  
  // Status valid (At least one of the DPE is invalid) 
  bool bInvalid = daValues[5] || daValues[7] || daValues[9];
  
  // --- Get main state string
  //--------------------------
  if (sDeviceName!="") {
     sState = unVacObject_getStateString(sDeviceName, iObjectSt);
  }
  else {
     sState = "Unknown";
  }

  // Determine operation status animation
  bool bOn = (iObjectSt == 3 || iObjectSt == 2);
  bool bOff = iObjectSt == 0;
  
  // Device status for color determination
  bool bUnderRange = iObjectSt == 3;
  bool bWarning = iObjectSt == 1 || iObjectSt >= 4 || getBit(bit32StsReg01, CPC_StsReg01_IOERRORW) == 1;
  bool bAlarmUnack = getBit(bit32StsReg01, CPC_StsReg01_ALUNACK) == 1 && 
             (iTStopIStState == 1 || 
              iTStopIStState == 3 || 
              iFuStopIStState == 1 || 
              iFuStopIStState == 3);
  bool bAlarm  = getBit(bit32StsReg01, CPC_StsReg01_TSTOPIST) == 1 || getBit(bit32StsReg02, CPC_StsReg02_FUSTOPIST) == 1 || getBit(bit32StsReg01, CPC_StsReg01_ALST) == 1;
  bool bForcedMode = getBit(bit32StsReg01, CPC_StsReg01_FoMoSt) == 1;
  
  // Create boolen array for corresponding colors
  dyn_bool dbStateBool = makeDynBool(
      bInvalid,      // Invalid color
      bAlarmUnack,   // Alarm Unacknowledged color
      bAlarm,        // Alarm active color
      bDisabled,     // Device disabled color
      bWarning,      // Warning color
      bForcedMode,   // Forced mode color,
      false,         // No undefined animation
      bOff && bUseOffColor,// Off color (optional color)
      bUnderRange,   // Under Range color
      bOn || (bOff && !bUseOffColor)   // OK color, when device is on or if device is off and we don't use an off color
  );
  
  // Fetch the body color from CPC_BODY_COLORS based on the lowest index value of the array dbBodyColorBool is true 
  unVacObject_WidgetBoolMapping(dbStateBool, CPC_BODY_COLORS, sColor); 
  
  return 0;
}  

int CPC_VP_I_ColorFillAndString(string sDeviceName, dyn_string dsDpes, dyn_anytype daValues, string &sColor, bool &bFill, string &sState) {
  bFill = false;
  int coco = CPC_VP_I_ColorAndString(sDeviceName, dsDpes, daValues, sColor, sState, true);
  if(coco >= 0) {
    int iObjectSt = daValues[8];
    bFill = (iObjectSt == 3 || iObjectSt == 2);  
  }
  return coco;
}

bool CPC_VP_I_MatchesCriteria(string sDpName, dyn_string dsDpes, dyn_anytype daDpeValues, shared_ptr<DeviceListCriteria> criteria,
                                string &sStateString, string &sStateColor, string &sValueString) {
  
  if(!CPC_VP_I_CheckCriteria(sDpName, dsDpes, daDpeValues, criteria)) {
    return false;
  }

  // List of DPEs/values shall be filtered out for state color/string calculation
  // Determine the body color based on the state
  float fPosSt = daDpeValues[13];
  dyn_int diKeyFilter = makeDynInt(3, 4, 5, 13, 14);
  dyn_string filteredDpes = unVacObject_dynWithoutIndex(dsDpes, diKeyFilter);
  dyn_anytype filteredValues = unVacObject_dynWithoutIndex(daDpeValues, diKeyFilter);
  CPC_VP_I_ColorAndString(sDpName, filteredDpes, filteredValues, sStateColor, sStateString, true);
  sValueString = unGenericObject_FormatValue(dpGetFormat(sDpName + ".ProcessInput.PosSt"), fPosSt) +
                 " " + dpGetUnit(sDpName + ".ProcessInput.PosSt"); // formated value with unit fPosSt);
  return true;
}

bool CPC_VP_I_CheckCriteria(string sDpName, dyn_string dsDpes, dyn_anytype daDpeValues, shared_ptr<DeviceListCriteria> criteria) {
  
  if(criteria.isEmpty()) {
    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "vacDeviceList",
                                "CPC_VP_I_CheckCriteria(" + sDpName +
                                "): match empty criteria");
    return true;  // Any device state matches empty criteria
  }
  criteria.resetResult();  // Make sure check result is reset in criteria

  // Further checks require interpretation of DPE values
 
  // Next - check NotConnected state: if device is not connected then it can only match
  // 'not connected' criteria, not any other
  int iSystemIntegrityAlarmValue = daDpeValues[1];
  bool bSystemIntegrityAlarmEnabled = daDpeValues[2];
  if(bSystemIntegrityAlarmEnabled && (iSystemIntegrityAlarmValue != 0)) {
    criteria.matchNotConnected();
    return criteria.isResult();
  }

  // Invalid DPE values - again, device can only match 'not connected' criteria
  if(daDpeValues[8] || daDpeValues[10] || daDpeValues[12] || daDpeValues[14]) {
    criteria.matchNotConnected();
    return criteria.isResult();
  }

  // Next - check device access errors. If device has access error problems - its state is,
  // most probably, unknown, and it can only match 'access error' criteria
  int iObjectSt = daDpeValues[11];
  // Access error can be found in ObjectSt value... may be also in StsReg0x?
  switch(iObjectSt) {
    case 4:  // Open Cable
    case 5:  // Error
    case 6:  // Profibus Slave Error or Not present
      criteria.macthAccessErr();
      return criteria.isResult();
    }

  // Next - check normal/abnormal status. For VPI use the criteria:
  //  ON = normal state
  //  OFF = abnormal state
  bool bNormal = false, bAbnormal = false;
  switch(iObjectSt) {
    case 0:  // Off without error
      bAbnormal = true;
      break;
    case 1:  // Over Current
    case 2:  // Measurement OK
    case 3:  // Under Range
      bNormal = true;
      break;
    }
  if(criteria.matchNormalState(bNormal, bAbnormal)) {
    return criteria.isResult();
  }

  // Next - check other criteria (errors/warnings). Not sure what exactly shall be checked here
  uint uStsReg01 = daDpeValues[7];
  if((uStsReg01 & 0x00C0)) { // IOErrorW +  IOSimuW
    if(criteria.matchError()) {
      return criteria.isResult();
    }
  }

  // Next - check pressure value
  if(criteria.isUsePressureLimit()) {
    float fPosSt = daDpeValues[13];
    if(fPosSt > criteria.getPressureLimit()) {
      if(criteria.matchPressureLimit()) {
        return criteria.isResult();
      }
    }
  }

  // Next - check pressure grow limit
  if(vacDeviceList_checkPressureGrowCriteria(sDpName + ".ProcessInput.PosSt", criteria)) {
    return true;
  }

  // If we've reached this point - device does not match criteria
  return false;
}

