/**@file

// cpcVP_TP.ctl
This library contains the widget, faceplate, etc. functions of VP_TP.

@par Creation Date
  22/06/2016

@par Modification History
  23/09/2016: body color orange when IO Error
  11/05/2017: Revisited body color calculation for state history
  15/06/2017: Device type Functions for Vac functionalities {sblancha}	
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author
  S. Blanchard (TE-VSC)
  T. van Winden (TE-VSC)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{
#uses "vacuumExtension/classes/DeviceListCriteria.ctl"

/** Returns the list of VP_TP DPE with alarm config that can be acknowledged

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name DP name
@param dpType input, device type
@param dsNeedAck output, the lsit of DPE
*/
CPC_VP_TP_AcknowledgeAlarm(string deviceName, string dpType, dyn_string &dsNeedAck) {
//begin_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
    dsNeedAck = makeDynString("ProcessInput.StartISt", "ProcessInput.TStopISt", "ProcessInput.FuStopISt", "ProcessInput.AlSt", UN_ACKNOWLEDGE_PLC);
//end_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
}

/** Function called from snapshot utility of the treeDeviceOverview to get the time and value

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name
@param deviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
CPC_VP_TP_ObjectListGetValueTime(string deviceName, string deviceType, dyn_string &dsReturnData) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
    float value;
    string sTime;

    dpGet(deviceName + ".ProcessInput.PosSt", value,
          deviceName + ".ProcessInput.PosSt:_online.._stime", sTime);

    dsReturnData[1] = sTime;
    dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(deviceName + ".ProcessInput.PosSt"), value) + " " + dpGetUnit(deviceName + ".ProcessInput.PosSt"); // formated value with unit
//end_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
}


/** POPUP MENU

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param dsAccessOk input, the access control
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
CPC_VP_TP_MenuConfiguration(string deviceName, string dpType, dyn_string dsAccessOk, dyn_string &menuList) {
//begin_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
    dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT, UN_POPUPMENU_TREND_TEXT, CPC_POPUPMENU_ALLOW_TO_RESTART_TEXT);
    
    mapping mDpe;
    mDpe["Rotation"] = "PosSt";
    
    // Add device context menu items
    unVacObject_addAliasActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    unVacObject_addAnalogActionToMenu(deviceName, dpType, mDpe, dsAccessOk, menuList);
    
    unVacObject_addDisableEnableActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    int iPos = unVacObject_addVacuumExtensionActionToMenu(deviceName, dpType,  dsAccessOk, menuList);
    iPos = unVacObject_addProcessActionToMenu(deviceName, dpType,  dsMenuConfig, dsAccessOk, menuList, iPos+1);
//end_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
}

/** handle the answer of the popup menu

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param menuList input, the access control
@param menuAnswer input, selected menu value
*/
CPC_VP_TP_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer) {
    dyn_string exceptionInfo;
    //begin_TAG_SCRIPT_DEVICE_TYPE_HandleMenu
    // The help file
    string helpFile = unVacObject_getHelpLink(dpType);
    
    if(!unVacObject_handleCustomUnicosMenu(deviceName, dpType, menuList, menuAnswer, helpFile, exceptionInfo)){
      cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
    }
    
    if (dynlen(exceptionInfo) > 0) {
        unSendMessage_toExpertException("Right Click", exceptionInfo);
    }
}

/** Init static values which are used in widget animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_VP_TP_WidgetInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
    dyn_string exceptionInfo;

    g_sPosStFormat = "##.#"; // usualy = dpGetFormat(deviceName + ".ProcessInput.PosSt");
    g_sPosStUnit = "%"; // usualy = dpGetUnit(deviceName + ".ProcessInput.PosSt");
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
}

/** 
Returns the list of VP_TP DPEs which should be connected on widget animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_VP_TP_WidgetDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
    dynAppend(dpes, deviceName + ".eventMask");                               //value[offset + 1]
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");                   //value[offset + 2]
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid"); //value[offset + 3]
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");                   //value[offset + 4]
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid"); //value[offset + 5]
    //dynAppend(dpes, deviceName + ".ProcessInput.PosSt");                      //value[offset + ]
    //dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_online.._invalid");    //value[offset + ]
    //dynAppend(dpes, deviceName + ".ProcessInput.MotPosSt");                   //value[offset + ]
    //dynAppend(dpes, deviceName + ".ProcessInput.MotPosSt:_online.._invalid"); //value[offset + ]
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");                   //value[offset + 6]
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid"); //value[offset + 7]
    dynAppend(dpes, deviceName + ".disabled");                                //value[offset + 8]
        
    if (g_bBoolAlert) {
        dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");  //value[offset + 9]
        dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");  //value[offset +10]
        dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state"); //value[offset +11]
        dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");      //value[offset +12]
    }
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
}

/** callback function on the device data

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

Datapoint elements and related values CPC_VP_TP_WidgetDPEs() dpe with offset=5
  1  int	frontend.alarm
  2  bool 	frontend.enabled
  3  bool    	device.statusInformation.selectedManager:_lock._original._locked,
  4  string  	device.statusInformation.selectedManager,
  5  bool       device.statusInformation.deviceLogActive
  6  int 	device.eventMask
  7  bit32	device.ProcessInput.StsReg01
  8  bool 	device.ProcessInput.StsReg01:_online.._invalid
  9  bit32	device.ProcessInput.StsReg02
  10 bool	device.ProcessInput.StsReg02:_online.._invalid
  11 int	device.ProcessInput.ObjectSt
  12 int	device.ProcessInput.ObjectSt:_online.._invalid
  13 bool	device.disabled
  14 int	device.ProcessInput.StartISt:_alert_hdl.._act_state
  15 int	device.ProcessInput.TStopISt:_alert_hdl.._act_state
  16 int	device.ProcessInput.FuStopISt:_alert_hdl.._act_state
  17 int	device.ProcessInput.AlSt:_alert_hdl.._act_state
  
  @see For the stsReg bit meaning check the specifications file: https://svn.cern.ch/reps/en-ice-svn/trunk/applications/vacuum/UNICOS_VAC_DEVICES/Design/Object_ManReg_StsReg_Mapping.xls
  @see Other usufull documentation: https://edms.cern.ch/ui/file/1228441/1.6.0/UCPC6_RP1.6.0_Manual.pdf

*/
CPC_VP_TP_WidgetAnimation(dyn_string dpes, dyn_anytype values, string widgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation

// INIT VARIABLES
//----------------
  int iSystemIntegrityAlarmValue = values[1];
  bool bSystemIntegrityAlarmEnabled = values[2];
  bool bLocked = values[3];
  string sSelectedManager = values[4];
  bool bComment = values[5];
  int iMask = values[6];
  bit32 bit32StsReg01 = values[7];
  bit32 bit32StsReg02 = values[9];
  int iObjectSt = values[11];
    
  // Do nothing if system is not connected
	if (!g_bSystemConnected){
		return;
	}
  
  // Check all invalid dpes for invalidness.
  bool bInvalid = values[8] || values[10] || values[12];
  
  // Create list the status registers
  dyn_bit32 dbit32StsRegs = makeDynBit32(bit32StsReg01, bit32StsReg02);
  
  // Determine widget type
  bool bCompact = widgetType == "VP_TP";
  
  // Base letter bits set on the widget type
  dyn_string dsWarningBits = (bCompact) ? VAC_WARNING_BITS : CPC_WARNING_BITS;
  dyn_string dsAlarmBits = (bCompact) ? VAC_ALARM_BITS : CPC_ALARM_BITS;
  dyn_string dsControlBits = (bCompact) ? VAC_CONTROL_BITS : CPC_CONTROL_BITS; 
  
  // Use default warning letter to list  
  dyn_bool dbWarningBools = unVacObject_fetchBitsFromStsRegs(dbit32StsRegs, dsWarningBits);
  // Display warning letter when object is deccelerating/recovering
  dbWarningBools[5] = iObjectSt == 3 || iObjectSt == 4; 

  // Use default alarm letters
  dyn_bool dbAlarmBools = unVacObject_fetchBitsFromStsRegs(dbit32StsRegs, dsAlarmBits);     // Default vacuum alarm bits
  // Check whether the device needs a restart
  bool bIsRestartNeeded = g_bMRestart && getBit(bit32StsReg02, CPC_StsReg02_NEEDRESTART) == 0;
  dbAlarmBools[5] = bIsRestartNeeded;
  
  // Generic vacuum letter animation
  unVacObject_WidgetLetterAnimation(widgetType, UN_CONFIG_CPC_VP_TP_DPT_NAME,
			bLocked, sSelectedManager, dbit32StsRegs,
                        dbWarningBools,    // Default vacuum warning bits
                        CPC_WARNING_LETTERS, // Default vacuum warning letters
                        dbAlarmBools,
                        CPC_ALARM_LETTERS, // Default vacuum alarm letters
                        unVacObject_fetchBitsFromStsRegs(dbit32StsRegs, dsControlBits) ,    // Default vacuum control bits
                        CPC_CONTROL_LETTERS, // Default vacuum control letters
			bInvalid, bIsRestartNeeded, bComment,
			iSystemIntegrityAlarmValue, bSystemIntegrityAlarmEnabled);
  				  
  // Determine the body color based on the state
  dyn_int diKeyFilter = makeDynInt(3, 4, 5);
  
  // Filter dpes based on index (remove dpe not used for color or string animations)
  string sBodyColor, sState;
  dyn_string filteredDpes = unVacObject_dynWithoutIndex(dpes, diKeyFilter);
  dyn_anytype filteredValues = unVacObject_dynWithoutIndex(values, diKeyFilter);
  CPC_VP_TP_ColorAndString("", filteredDpes, filteredValues, sBodyColor, sState, false, g_bBoolAlert, bIsRestartNeeded);

  // OBJECT ANIMATION (animation according to ObjectSt dpe)
  CPC_VP_TP_WidgetBodyAnimation(sBodyColor, iObjectSt);
  
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
}

/** 
  Animate the body color for the turbopump widget based on the ObjectSt
  
  @param sBodyColor The body color 
  @param iObjectSt The object status
  
  @return void
*/
void CPC_VP_TP_WidgetBodyAnimation(string sBodyColor, int iObjectSt){  
  setMultiValue("BodyLineExt1", "foreCol", sBodyColor);
  if (iObjectSt == 0 || iObjectSt > 4) // Turbo pump is off or The controller is in error 
    setMultiValue("BodyFillBottom1", "backCol", "unWidget_Background",
        				  "BodyLineBottom1", "foreCol", sBodyColor,
    						  "BodyFillMiddle1", "backCol", "unWidget_Background",
    						  "BodyFillMiddle1", "foreCol", "unWidget_Background",
    						  "BodyLineMiddle1", "foreCol", sBodyColor,
    						  "BodyLineMiddle2", "foreCol", sBodyColor,
    						  "BodyLineMiddle3", "foreCol", sBodyColor,
    						  "BodyLineMiddle4", "foreCol", sBodyColor,
    						  "BodyLineMiddle5", "foreCol", sBodyColor,
    						  "BodyFillTop1",    "backCol", "unWidget_Background",
    						  "BodyLineTop1",    "foreCol", sBodyColor);
  else if (iObjectSt == 1 || iObjectSt == 4) // Turbo pump is accelerating or decelerating 
    setMultiValue("BodyFillBottom1", "backCol", sBodyColor,
    						  "BodyLineBottom1", "foreCol", "unWidget_Background",
    						  "BodyFillMiddle1", "backCol", "unWidget_Background",
    						  "BodyFillMiddle1", "foreCol", "unWidget_Background",
    						  "BodyLineMiddle1", "foreCol", sBodyColor,
    						  "BodyLineMiddle2", "foreCol", sBodyColor,
    						  "BodyLineMiddle3", "foreCol", sBodyColor,
    						  "BodyLineMiddle4", "foreCol", sBodyColor,
    						  "BodyLineMiddle5", "foreCol", sBodyColor,
    						  "BodyFillTop1",    "backCol", "unWidget_Background",
    						  "BodyLineTop1",    "foreCol", sBodyColor);
  else if (iObjectSt == 2) // Turbo pump is nominal speed
	  setMultiValue("BodyFillBottom1", "backCol", sBodyColor,
    						  "BodyLineBottom1", "foreCol", "unWidget_Background",
    						  "BodyFillMiddle1", "backCol", sBodyColor,
    						  "BodyFillMiddle1", "foreCol", sBodyColor,
    						  "BodyLineMiddle1", "foreCol", "unWidget_Background",
    						  "BodyLineMiddle2", "foreCol", "unWidget_Background",
    						  "BodyLineMiddle3", "foreCol", "unWidget_Background",
    						  "BodyLineMiddle4", "foreCol", "unWidget_Background",
    						  "BodyLineMiddle5", "foreCol", "unWidget_Background",
    						  "BodyFillTop1",    "backCol", sBodyColor,
    						  "BodyLineTop1",    "foreCol", "unWidget_Background");
	else // iObjectSt == 3 : Turbo pump is recovering 
		setMultiValue("BodyFillBottom1", "backCol", sBodyColor,
    						  "BodyLineBottom1", "foreCol", "unWidget_Background",
    						  "BodyFillMiddle1", "backCol", sBodyColor,
    						  "BodyFillMiddle1", "foreCol", sBodyColor,
    						  "BodyLineMiddle1", "foreCol", "unWidget_Background",
    						  "BodyLineMiddle2", "foreCol", "unWidget_Background",
    						  "BodyLineMiddle3", "foreCol", "unWidget_Background",
    						  "BodyLineMiddle4", "foreCol", "unWidget_Background",
    						  "BodyLineMiddle5", "foreCol", "unWidget_Background",
    						  "BodyFillTop1",    "backCol", "unWidget_Background",
    						  "BodyLineTop1",    "foreCol", sBodyColor);
}

/** Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.

*/
CPC_VP_TP_WidgetDisconnection(string sWidgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
    string sBodyColor = "unDataNoAccess";
    CPC_VP_TP_WidgetBodyAnimation(sBodyColor, 0);
	  setMultiValue(				  
          "WarningText", "text", "", 
          "AlarmText", "text", "", 
          "ControlStateText", "text", "",
          "SelectArea", "foreCol", "",
          "LockBmp", "visible", false,
          "WidgetArea", "visible", false,
          "eventState", "visible", false);
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
}




// BUTTONS IN FACEPLATE
//--------------------------------

/** Return button configuration including access level

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
mapping CPC_VP_TP_ButtonConfig(string deviceName) {
    mapping buttons;
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
    dyn_string exceptionInfo;

    buttons[UN_FACEPLATE_BUTTON_SELECT] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_AUTO_MODE] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_MANUAL_MODE] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_FORCED_MODE] = UN_ACCESS_RIGHTS_EXPERT;

    //buttons[UN_FACEPLATE_BUTTON_SET_VALUE] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_ONOPEN_REQUEST] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST] = UN_ACCESS_RIGHTS_OPERATOR;
	buttons[UN_FACEPLATE_BUTTON_RESETERRWARN_REQUEST] = UN_ACCESS_RIGHTS_EXPERT; 

    buttons[UN_FACEPLATE_BUTTON_ACK_ALARM] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[CPC_FACEPLATE_BUTTON_ALLOW_RESTART] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_MASKEVENT] = UN_ACCESS_RIGHTS_EXPERT;
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
    return buttons;
}

/** Configure the list of dpes that needs for buttons animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the name of device
@param dpes input, the list of dpes to connect
*/
CPC_VP_TP_ButtonDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
}

/** Set the state of the contextual button of the device

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param dpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
CPC_VP_TP_ButtonSetState(string deviceName, string dpType, dyn_string dsUserAccess, dyn_string dsData) {
    bool bSelected, buttonEnabled, bit1, bit2, bit3;
    string localManager;
    dyn_string exceptionInfo;

    bool bLocked = dsData[1];
    string sSelectedUIManager = dsData[2];

    localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
    bSelected = (localManager == "S"); // Selection state
    dyn_string dsButtons = unSimpleAnimation_ButtonList(deviceName);

//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
    mixed stsReg01Value = dsData[3];
    bool bStsReg01Bad = dsData[4];

    for (int i = 1; i <= dynlen(dsButtons); i++) {
        buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0); // User access
        switch (dsButtons[i]) {
            case UN_FACEPLATE_BUTTON_FORCED_MODE:
                bit1 = !(getBit(stsReg01Value, CPC_StsReg01_FOMOST) == 0);
                bit2 = !(getBit(stsReg01Value, CPC_StsReg01_AUINHFMO) == 0);
                buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2) && !bStsReg01Bad;
                break;
            case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
            //case UN_FACEPLATE_BUTTON_ONFORCED_REQUEST: // 
            case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
            case UN_FACEPLATE_BUTTON_AUTO_MODE:
            case UN_FACEPLATE_BUTTON_SET_VALUE:
                bit1 = !(getBit(stsReg01Value, CPC_StsReg01_AUMOST) == 0);
                buttonEnabled = buttonEnabled && bSelected && !(bit1) && !bStsReg01Bad;
                break;
            case UN_FACEPLATE_BUTTON_MANUAL_MODE:
                buttonEnabled = buttonEnabled && bSelected &&
                                !getBit(stsReg01Value, CPC_StsReg01_MMOST) &&
                                !getBit(stsReg01Value, CPC_StsReg01_AUINHMMO) &&
                                !bStsReg01Bad;
                break;
			case UN_FACEPLATE_BUTTON_RESETERRWARN_REQUEST:
				buttonEnabled = buttonEnabled && bSelected &&
                                !bStsReg01Bad;
            default:
                break;
        }
        if (dsButtons[i] == UN_FACEPLATE_BUTTON_SELECT) {
            unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
        } else {
            cpcButton_setButtonState(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], buttonEnabled);
        }
    }
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
}






// FACEPLATE
//--------------------------------



/** Init static values which are used in faceplate animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_VP_TP_FaceplateInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
    int iAlertType1, iAlertType2, iAlertType3, iAlertType4, iRangeType;
    dyn_string exceptionInfo;
    int iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
                     deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
                     deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
                     deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
    g_params["BoolAlert"] = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));
    g_params["PosStFormat"] = dpGetFormat(deviceName + ".ProcessInput.PosSt");
    g_params["PosStUnit"] = dpGetUnit(deviceName + ".ProcessInput.PosSt");
    g_params["HFStFormat"] = dpGetFormat(deviceName + ".ProcessInput.HFSt");
    g_params["HFStUnit"] = dpGetUnit(deviceName + ".ProcessInput.HFSt");
    g_params["MRestart"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);
	
	  g_params["PumpModel"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_INSTRUMENT_MODEL, exceptionInfo);
    g_params[CPC_PROP_NAME_INTERFACE_TYPE] = (int) cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_INTERFACE_TYPE, exceptionInfo);
	  setValue("PumpModel", "text", g_params["PumpModel"]);
	  g_params["ControllerModel"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_CONTROLLER_MODEL, exceptionInfo);
	  setValue("ControllerModel", "text", g_params["ControllerModel"]);
    
    // Disable PosSt and MotPosSt in case of wiring I/O interface 
    if( (g_params[CPC_PROP_NAME_INTERFACE_TYPE] == 0) || (g_params[CPC_PROP_NAME_INTERFACE_TYPE] == 1) ){
      setMultiValue("PosStLabel", "enabled", false, "PosSt", "enabled", false, "MotPosStLabel", "enabled", false, "MotPosSt", "enabled", false);
    }      
		
	  mapping objectStLabels;
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.ObjectSt"), objectStLabels);
    g_params["ObjectStLabels"] = objectStLabels;
	
	  mapping errCoStLabels;
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.ErrCoSt"), errCoStLabels);
    g_params["ErrCoStLabels"] = errCoStLabels;


//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
}



/** Returns the list of VP_TP DPEs which should be connected on faceplate animation
// dyn_string dpes value[1]: event Mask
// dyn_string dpes value[2]: StsReg01
// dyn_string dpes value[1]: StsReg01:_online.._invalid
// dyn_string dpes value[1]: event Mask
// dyn_string dpes value[1]: event Mask
// dyn_string dpes value[1]: event Mask

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_VP_TP_FaceplateDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
    dynAppend(dpes, deviceName + ".eventMask");  										
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.PosSt");
    dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.MotPosSt");
    dynAppend(dpes, deviceName + ".ProcessInput.MotPosSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.ErrCoSt");
    dynAppend(dpes, deviceName + ".ProcessInput.ErrCoSt:_online.._invalid");
    if (g_params["BoolAlert"]) {
        dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");
    }
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
}


/** callback function on the device data

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
CPC_VP_TP_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
    // Stop animation in case of wiring I/O interface
    if(g_params[CPC_PROP_NAME_INTERFACE_TYPE] != 1){
      unFaceplate_animateOnlineValue(dpes, values, "PosSt", g_params["PosStUnit"], g_params["PosStFormat"], "cpcColor_Faceplate_Status");
      unFaceplate_animateOnlineValue(dpes, values, "MotPosSt", "Amps", g_params["PosStFormat"], "cpcColor_Faceplate_Status");      
    }      
    unFaceplate_animateTranslatedValue(dpes, values, "ObjectSt", "", "", g_params["ObjectStLabels"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "ErrCoSt", "", "", g_params["ErrCoStLabels"], "cpcColor_Faceplate_Status");

    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_ONST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_OFFST", true, "cpcColor_Faceplate_Status");
	
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUMOST", true, "cpcColor_Faceplate_Status");   
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_MMOST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUINHMMO", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_FOMOST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUINHFMO", true, "cpcColor_Faceplate_Status");

    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_OUTONOVST", true, "cpcColor_Faceplate_Status");
	
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_AUOFFPRST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_AUONPRST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_MOFFPRST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_MONPRST", true, "cpcColor_Faceplate_Status");
    
    //Stand-by and Reset Requests
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_AUSBPRST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_MSBPRST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_AURSTPRST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_MRSTPRST", true, "cpcColor_Faceplate_Status");

    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg01_STARTIST",   "StartISt",  "CPC_StsReg01_ALUNACK");
    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg01_TSTOPIST",  "TStopISt",  "CPC_StsReg01_ALUNACK");
    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg02_FUSTOPIST", "FuStopISt", "CPC_StsReg01_ALUNACK");
    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg01_ALST",      "AlSt",      "CPC_StsReg01_ALUNACK");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_ALUNACK", true, "cpcColor_Alarm_Bad");

    cpcGenericObject_animateMaskEvent(dpes, values);
    cpcGenericObject_animateNeedRestart(dpes, values);
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOERRORW", true, "cpcColor_Faceplate_Warning");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOWARNINGW", true, "cpcColor_Faceplate_Warning");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOSIMUW", true, "cpcColor_Faceplate_Warning");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_FODIPRO", true, "cpcColor_Faceplate_Warning");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_MIOERBRST", true, "cpcColor_Faceplate_Warning");

    //Reset Order Value Status    
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_OUTRSTOVST", true, "cpcColor_Faceplate_Status");
        
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_ALBW", true, "cpcColor_Faceplate_Warning");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_OUTOFFOVST", true, "cpcColor_Faceplate_Status");
    
    //Stand-by Order Value Status
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_OUTSBOVST", true, "cpcColor_Faceplate_Status");

    
    
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
}

//@}




/**
  Lists the dpes required for building the state of the device
  
  @param string sDeviceName The name of the device which is in UNICOS the same as the dpname
  @param dyn_string dsDpes The list of datapoints elements required for building the state
  @param bool bForHistory The DPEs will be used for state history, i.e. only DPEs with _archive
                            config are of interest
  
*/
CPC_VP_TP_StateDPEs(string sDeviceName, dyn_string &dsDpes, bool bForHistory = true) {
    dynAppend(dsDpes, sDeviceName + ".eventMask");                                  //idx 3
    if(bForHistory) {
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg01");                    //idx 4
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg01:_online.._invalid");  //idx 5
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg02");                    //idx 6
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg02:_online.._invalid");  //idx 7
    }
    else {
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg01");                    //idx 4
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg01:_online.._invalid");  //idx 5
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg02");                    //idx 6
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg02:_online.._invalid");  //idx 7
    }
    dynAppend(dsDpes, sDeviceName + ".ProcessInput.ObjectSt");                      //idx 8
    dynAppend(dsDpes, sDeviceName + ".ProcessInput.ObjectSt:_online.._invalid");    //idx 9
    dynAppend(dsDpes, sDeviceName + ".disabled");                                   //idx10
}

/**
  Returns the main color associated with the device state. This color is based on 
  the dpes listed in the CPC_VV_S_StateDPEs for the dedicated device type. In case of the 
  state history the EvStsReg## is used because this is archived, while for determining 
  the current state the StsReg## is used.
  @param string sDeviceName, imput, device dp name to get objectSt meaning
  @param dyn_string dsDpes The list of dpe names used to determine the color
  @param dyn_anytype daValues The list of dpe values used to determine the color
  
  1  int 		  frontend.alarm
  2  bool 		  frontend.enabled
  3  bool                 device.eventMask
  4  bit32		  device.ProcessInput.(Ev)StsReg01
  5  bool 		  device.ProcessInput.(Ev)StsReg01:_online.._invalid
  6  bit32		  device.ProcessInput.(Ev)StsReg02
  7  bool		  device.ProcessInput.(Ev)StsReg02:_online.._invalid
  8  int		  device.ProcessInput.ObjectSt
  9  int		  device.ProcessInput.ObjectSt:_online.._invalid
  10  bool		  device.disabled
  11 int		  device.ProcessInput.StartISt:_alert_hdl.._act_state
  12 int		  device.ProcessInput.TStopISt:_alert_hdl.._act_state
  13 int		  device.ProcessInput.FuStopISt:_alert_hdl.._act_state
  14 int		  device.ProcessInput.AlSt:_alert_hdl.._act_state
  
  @param string sColor, output, the main color string of device
  @param string sString, output, the main state string of device
  @param bool bUseOffColor, input, Off color is return in &sColor when device is Off, false in case of WidgetColor
  @param bool bBoolAlert, input, realtime alert according to UNICOS alert manager

  @param bool bIsRestartNeeded, input, there is a restart needed according to UNICOS device manager
  
  @see For the stsReg bit meaning check the specifications file: https://svn.cern.ch/reps/en-ice-svn/trunk/applications/vacuum/UNICOS_VAC_DEVICES/Design/Object_ManReg_StsReg_Mapping.xls
  @see Other usufull documentation: https://edms.cern.ch/ui/file/1228441/1.6.0/UCPC6_RP1.6.0_Manual.pdf  
  
  @return 'o' when function proceed successfully, '-1' when function proceed with error
*/
  
int CPC_VP_TP_ColorAndString(string sDeviceName, dyn_string dsDpes, dyn_anytype daValues, string &sColor, string &sState, bool bUseOffColor, bool bBoolAlert = false, bool bIsRestartNeeded = false){
  int iRetVal = -1;

  // --- Get device data value
  // --------------------------	   
  // Read system integrity values   
  int iSystemIntegrityAlarmValue = daValues[1];
  bool bSystemIntegrityAlarmEnabled = daValues[2];
  bit32 bit32StsReg01 = daValues[4];
  bit32 bit32StsReg02 = daValues[6];
  // Object status
  int iObjectSt = daValues[8];
  bool bDisabled = daValues[10];
  // Software Interlock status
  int iStartIStState, iTStopIStState, iFuStopIStState, iAlSt;
  if (bBoolAlert) {
    iStartIStState = daValues[11];
    iTStopIStState = daValues[12];
    iFuStopIStState = daValues[13];
    iAlSt = daValues[14];}
  // Status valid (At least one of the DPE is invalid) 
  bool bInvalid = daValues[5] || daValues[7] || daValues[9];

  // --- Get main state string
  //--------------------------
  if (sDeviceName!="") {
     sState = unVacObject_getStateString(sDeviceName, iObjectSt);
  }
  else {
     sState = "Unknown";
  }

  // --- Get main color string
  // --------------------------
  // Device status for color determination
  bool bWarning = bIsRestartNeeded || getBit(bit32StsReg01, CPC_StsReg01_IOERRORW) == 1;
  bool bAlarmUnack = getBit(bit32StsReg01, CPC_StsReg01_ALUNACK) == 1 && 
             (iTStopIStState  == 1 ||
              iTStopIStState  == 3 ||
              iFuStopIStState == 1 || 
              iFuStopIStState == 3);
  bool bAlarm  = getBit(bit32StsReg01, CPC_StsReg01_TSTOPIST) == 1 || getBit(bit32StsReg02, CPC_StsReg02_FUSTOPIST) == 1 || getBit(bit32StsReg01, CPC_StsReg01_ALST) == 1;
  bool bForcedMode = getBit(bit32StsReg01, CPC_StsReg01_FoMoSt) == 1;

  bool bOn = getBit(bit32StsReg01, CPC_StsReg01_ONST) == 1;
  bool bOff = getBit(bit32StsReg01, CPC_StsReg01_OFFST) == 1;
  bool bUndefined = bOff == bOn;
  // Create boolen array for corresponding colors
  dyn_bool dbStateBool = makeDynBool(
      bInvalid,              // Invalid color
      bAlarmUnack,           // Alarm Unacknowledged color
      bAlarm,                // Alarm active color
      bDisabled,             // Device disabled color
      bWarning,              // Warning color
      bForcedMode,           // Forced mode color
      bUndefined && bUseOffColor,  // Undefined color
      bOff && bUseOffColor,  // Off color (optional color)
      false,                 // Under Range color
      true                   // OK color, if device is on or device is off and not using an off color
  );
  // Fetch the body color from CPC_BODY_COLORS based on the lowest index value of the array dbBodyColorBool is true 
  unVacObject_WidgetBoolMapping(dbStateBool, CPC_BODY_COLORS, sColor);

  iRetVal = 0;
  return iRetVal;
}  

int CPC_VP_TP_ColorFillAndString(string sDeviceName, dyn_string dsDpes, dyn_anytype daValues, string &sColor, bool &bFill, string &sState) {
  bFill = false;
  int coco = CPC_VP_TP_ColorAndString(sDeviceName, dsDpes, daValues, sColor, sState, true);
  if(coco >= 0) {
    int iObjectSt = daValues[11];
    bFill = iObjectSt == 2; // Turbo pump is nominal speed
  }
  return coco;
}
