/**@file

 // cpcVR_AH.ctl
 This library contains the widget, faceplate, etc. functions of VR_AH.

 @par Creation Date
 10/10/2016

 @par Modification History

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @author
 T. van Winden (TE-VSC)

 @copyright
 &copy;Copyright CERN 2013 - all rights reserved
 */
//@{

#uses "vacuumExtension/classes/DeviceListCriteria.ctl"

/** Returns the list of VR_AH DPE with alarm config that can be acknowledged

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @param deviceName  input, device name DP name
 @param dpType input, device type
 @param dsNeedAck output, the lsit of DPE
 */
//CPC_VR_AH_AcknowledgeAlarm(string deviceName, string dpType, dyn_string &dsNeedAck) {}
/** Function called from snapshot utility of the treeDeviceOverview to get the time and value

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @param deviceName  input, device name
 @param deviceType input, device type
 @param dsReturnData output, return data, array of 5 strings
 */
CPC_VR_AH_ObjectListGetValueTime(string deviceName, string deviceType, dyn_string &dsReturnData) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
	float value;
	string sTime;

	dpGet(deviceName + ".ProcessInput.ObjectSt", value,
			deviceName + ".ProcessInput.ObjectSt:_online.._stime", sTime);

	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(deviceName + ".ProcessInput.ObjectSt"), value) + " " + dpGetUnit(deviceName + ".ProcessInput.ObjectSt");// formated value with unit
//end_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
}

// POPUP MENU
//--------------------------------

/** pop-up menu

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @param deviceName input, device DP name
 @param dpType input, device type
 @param dsAccessOk input, the access control
 @param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
 */
CPC_VR_AH_MenuConfiguration(string deviceName, string dpType, dyn_string dsAccessOk, dyn_string &menuList) {
//begin_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT, UN_POPUPMENU_TREND_TEXT, CPC_POPUPMENU_ALLOW_TO_RESTART_TEXT);

  // Add device context menu items
  unVacObject_addAliasActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    
  unVacObject_addDisableEnableActionToMenu(deviceName, dpType, dsAccessOk, menuList);
  int iPos = unVacObject_addVacuumExtensionActionToMenu(deviceName, dpType,  dsAccessOk, menuList);
  iPos = unVacObject_addProcessActionToMenu(deviceName, dpType,  dsMenuConfig, dsAccessOk, menuList, iPos+1);
//end_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
}

/** handle the answer of the popup menu

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @param deviceName input, device DP name
 @param dpType input, device type
 @param menuList input, the access control
 @param menuAnswer input, selected menu value
 */
CPC_VR_AH_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer) {
	dyn_string exceptionInfo;
	//begin_TAG_SCRIPT_DEVICE_TYPE_HandleMenu

    // The help file
    string helpFile = unVacObject_getHelpLink(dpType);

	if(!unVacObject_handleCustomUnicosMenu(deviceName, dpType, menuList, menuAnswer, helpFile, exceptionInfo)) {
		cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
	}

	if (dynlen(exceptionInfo) > 0) {
		unSendMessage_toExpertException("Right Click", exceptionInfo);
	}
	//end_TAG_SCRIPT_DEVICE_TYPE_HandleMenu
}

/** Init static values which are used in widget animation

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @param deviceName input, the device name

 */
CPC_VR_AH_WidgetInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
	dyn_string exceptionInfo, dsChannels;  
  string sSystemName, sControllerAlias, sControllerDpName, sChannelConstant;
  
	g_params[CPC_PROP_NAME_MACHINE_TYPE] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_MACHINE_TYPE, exceptionInfo);
	g_params[CPC_PROP_NAME_CONFIG_TYPE] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_CONFIG_TYPE, exceptionInfo);
  g_params[CPC_PROP_NAME_SOURCE1] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_SOURCE1, exceptionInfo);
  g_params[CPC_PROP_NAME_SOURCE2] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_SOURCE2, exceptionInfo);
  
  // Load the instrument name from the controller
  if(g_params[CPC_PROP_NAME_CONFIG_TYPE] == "1" || g_params[CPC_PROP_NAME_CONFIG_TYPE] == "2") {
    // Build up the controller dp name
    sSystemName = unGenericDpFunctions_getSystemName(deviceName);      
    sControllerAlias = sSystemName+g_params[CPC_PROP_NAME_SOURCE1];     
    sControllerDpName = unGenericDpFunctions_getWidgetDpName(sControllerAlias);

    // Get the controller channels for both gauge and ionpump controller
    for(int i = 1;i<=4;i++){
      // Load the parameter constant name
      sChannelConstant = (g_params[CPC_PROP_NAME_CONFIG_TYPE] == "1") ? "CPC_PARAMS_GAUGE"+i+"_NAME" : "CPC_PARAMS_PUMP"+i+"_NAME";
      // Load the channel name from the contant
      dsChannels[i] = cpcGenericDpFunctions_getDeviceProperty(sControllerDpName, cpcConfigGenericFunctions_getStringConstantByName(sChannelConstant), exceptionInfo);
    }
    // Set the channel instruments
    g_params[CPC_PROP_NAME_CHANNELS]	 = dsChannels;
  }
}

/** 

  Returns the list of VR_AH DPEs which should be connected on widget animation

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @param deviceName input, the device name
 @param dpes input/output, dpe list

 */
CPC_VR_AH_WidgetDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
	dynAppend(dpes, deviceName + ".eventMask");                                // 6
	dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");                    // 7
	dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");  // 8
	dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");                    // 9
	dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid");  // 10
	dynAppend(dpes, deviceName + ".ProcessInput.SrcChanSt");                   // 11
	dynAppend(dpes, deviceName + ".ProcessInput.SrcChanSt:_online.._invalid"); // 12
	dynAppend(dpes, deviceName + ".disabled");                                 // 13
  //end_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
}

/** callback function on the device data

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @param dpes input, the dpe names
 @param values input, the dpe values
 
 Datapoint elements and related values
  1  int     frontend.alarm
  2  bool    frontend.enabled
  3  bool    device.statusInformation.selectedManager:_lock._original._locked,
  4  string  device.statusInformation.selectedManager,
  5  bool    device.statusInformation.deviceLogActive
  6  int     device.eventMask
  7  bit32   device.ProcessInput.StsReg01
  8  bool    device.ProcessInput.StsReg01:_online.._invalid
  9  int     device.ProcessInput.ObjectSt
  10 bool     device.ProcessInput.ObjectSt:_online.._invalid 
  11 int     device.ProcessInput.SrcChanSt
  12 bool     device.ProcessInput.SrcChanSt:_online.._invalid 
  13 bool    device.disabled

 
 @see For the stsReg bit meaning check the specifications file: https://svn.cern.ch/reps/en-ice-svn/trunk/applications/vacuum/UNICOS_VAC_DEVICES/Design/Object_ManReg_StsReg_Mapping.xls
 @see Other usefull documentation: https://edms.cern.ch/ui/file/1228441/1.6.0/UCPC6_RP1.6.0_Manual.pdf
 */
CPC_VR_AH_WidgetAnimation(dyn_string dpes, dyn_anytype values, string sWidgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation

  // Do nothing if system is not connected
	if (!g_bSystemConnected){
		return;
	}
  
// INIT VARIABLES
//----------------
	int iSystemIntegrityAlarmValue = values[1];
	bool bSystemIntegrityAlarmEnabled = values[2];
	bool bLocked = values[3];
	string sSelectedManager = values[4];
	bool bComment = values[5];
	int iMask = values[6];
	int iObjectSt = values[9];
  int iChannel = values[11];
  
  // Create a list of registers
  dyn_bit32 dbit32StsRegs = makeDynBit32(values[7]);
   
	// Warning boolean
	bool bInvalid = values[8] || values[10] || values[12];
	bool bOn = iObjectSt == 0 || iObjectSt == 1;
  
  // Generic vacuum animation
	unVacObject_WidgetLetterAnimation(sWidgetType, UN_CONFIG_CPC_VR_AH_DPT_NAME,
			bLocked, sSelectedManager, dbit32StsRegs,
			makeDynString(),// Default vacuum warning bits
			makeDynString(),// Default vacuum warning letters
			makeDynString(),// Default vacuum alarm bits
			makeDynString(),// Default vacuum alarm letters
			makeDynString(),// Default vacuum control bits
			makeDynString(),// Default vacuum control letters
			bInvalid, false, bComment,
			iSystemIntegrityAlarmValue, bSystemIntegrityAlarmEnabled);
  
  // Determine the body color based on the state
  dyn_int diKeyFilter = makeDynInt(3, 4, 5); 
  
  // Filter dpes based on index to be consistent with the StateHistory dpe's
  string sBodyColor, sState;
  dyn_string filteredDpes = unVacObject_dynWithoutIndex(dpes, diKeyFilter);
  dyn_anytype filteredValues = unVacObject_dynWithoutIndex(values, diKeyFilter);
  CPC_VR_AH_ColorAndString("", filteredDpes, filteredValues, sBodyColor, sState, false, g_bBoolAlert, false);   
  

  // Find the instrument causing the interlock
  if(1 <= iChannel && iChannel <= dynlen(g_params[CPC_PROP_NAME_CHANNELS])){
      g_params[CPC_PROP_NAME_INSTRUMENT] = g_params[CPC_PROP_NAME_CHANNELS][iChannel];
  } else {
      g_params[CPC_PROP_NAME_INSTRUMENT] = "";
  }
  
  // Arrow hide animation
  bool bHidden = (g_sSourceName != "" && g_params[CPC_PROP_NAME_INSTRUMENT] != g_sSourceName);    
  
  // Animate the true color
  CPC_VR_AH_WidgetBodyAnimation(bOn, false, sBodyColor, bHidden);  

//end_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
}

CPC_VR_AH_WidgetLetterAnimation(string sWarningLetter, string sWarningColor,
		string sAlarmLetter, string sAlarmColor, string sControlLetter, string sControlColor,
		string sColorSelect, bool bLockVisible, bool bComment) {
	setMultiValue(
			"SelectArea", "foreCol", sColorSelect
	);
}

/**
 @author Thijs van Winden

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 UI
 @param bOn Determine if the device is on
 @param bUndefined Undefined state of the device
 @param sBodyColor The body color of the device
 */
CPC_VR_AH_WidgetBodyAnimation(bool bOn, bool bHalf, string sBodyColor, bool bHidden = false) {
	setMultiValue(
			"BodyArrowUp", "backCol", sBodyColor,
			"BodyArrowUp", "foreCol", sBodyColor,
			"BodyArrowRight", "backCol", sBodyColor,
			"BodyArrowRight", "foreCol", sBodyColor,
			"BodyArrowDown", "backCol", sBodyColor,
			"BodyArrowDown", "foreCol", sBodyColor,
			"BodyArrowLeft", "backCol", sBodyColor,
			"BodyArrowLeft", "foreCol", sBodyColor,
			"BodyVertical", "backCol", sBodyColor,
			"BodyVertical", "foreCol", sBodyColor,
			"BodyHorizontal", "backCol", sBodyColor,
			"BodyHorizontal", "foreCol", sBodyColor,
      "SelectArea", "visible", !bHidden,
      "SelectedArea", "visible", !bHidden
      // Visibility animation
	);
  
  // Switch the orientation
  switch(g_sOrientation){
    case "up":
      setMultiValue("BodyArrowUp", "visible", !bHidden, "BodyVertical", "visible", !bHidden);
      break;      
    case "right":
      setMultiValue("BodyArrowRight", "visible", !bHidden, "BodyHorizontal", "visible", !bHidden);  
      break;    
    case "left":
      setMultiValue("BodyArrowLeft", "visible", !bHidden, "BodyHorizontal", "visible", !bHidden);    
      break;  
    case "down":
      setMultiValue("BodyArrowDown", "visible", !bHidden, "BodyVertical", "visible", !bHidden);    
      break;
    default:
      throwError(makeError("unVacuum", PRIO_SEVERE, ERR_CONTROL, 00002, "Expected orientation with value [up, right, left, down], but "+$sOrientation+" given"));  
  }
    
  
  

}

/** Disconnect function for the widget data

 Parameters:

 Usage: Internal

 PVSS manager usage: NG, NV

 Constraints:
 . Global variables defined in OnOff faceplate
 . PVSS version: 2.12.1
 . operating system: NT and W2000, but tested only under W2000.
 . distributed system: yes.

 */
CPC_VR_AH_WidgetDisconnection(string sWidgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
	CPC_VR_AH_WidgetBodyAnimation(false, false, "unDataNoAccess");
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
}

// BUTTONS IN FACEPLATE
//--------------------------------

/** Return button configuration including access level

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 */
mapping CPC_VR_AH_ButtonConfig(string deviceName) {
	mapping buttons;
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig

	buttons[CPC_FACEPLATE_BUTTON_ALLOW_RESTART] = UN_ACCESS_RIGHTS_OPERATOR;
	unVacObject_genericButtonConfig(deviceName, buttons);

//end_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
	return buttons;
}

/** Configure the list of dpes that needs for buttons animation

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @param deviceName input, the name of device
 @param dpes input, the list of dpes to connect
 */
CPC_VR_AH_ButtonDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
	dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
	dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
}

/** Set the state of the contextual button of the device

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @param deviceName input, the device DP name
 @param dpType input, the device type
 @param dsUserAccess input, list of allowed action on the device
 @param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
 */
CPC_VR_AH_ButtonSetState(string deviceName, string dpType, dyn_string dsUserAccess, dyn_string dsData) {
	bool bSelected, buttonEnabled, bit1, bit2, bit3;
	string localManager;
	dyn_string exceptionInfo;

	bool bLocked = dsData[1];
	string sSelectedUIManager = dsData[2];

	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
	bSelected = (localManager == "S"); // Selection state
	dyn_string dsButtons = unSimpleAnimation_ButtonList(deviceName);

//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
	mixed stsReg01Value = dsData[3];
	bool bStsReg01Bad = dsData[4];

	for (int i = 1; i <= dynlen(dsButtons); i++) {
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0); // User access
		switch (dsButtons[i]) {
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
			bit1 = !(getBit(stsReg01Value, CPC_StsReg01_FOMOST) == 0);
			bit2 = !(getBit(stsReg01Value, CPC_StsReg01_AUINHFMO) == 0);
			buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2) && !bStsReg01Bad;
			break;
			case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
			//case UN_FACEPLATE_BUTTON_ONFORCED_REQUEST: //
			case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
			case UN_FACEPLATE_BUTTON_SET_VALUE:
			bit1 = !(getBit(stsReg01Value, CPC_StsReg01_AUMOST) == 0);
			buttonEnabled = buttonEnabled && bSelected && !(bit1) && !bStsReg01Bad;
			break;
			case UN_FACEPLATE_BUTTON_MANUAL_MODE:
			buttonEnabled = buttonEnabled && bSelected &&
			!getBit(stsReg01Value, CPC_StsReg01_MMOST) &&
			!getBit(stsReg01Value, CPC_StsReg01_AUINHMMO) &&
			!bStsReg01Bad;
			break;
		}
		if (dsButtons[i] == UN_FACEPLATE_BUTTON_SELECT) {
			unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
		} else {
			cpcButton_setButtonState(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], buttonEnabled);
		}
	}
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
}

// FACEPLATE
//--------------------------------

/** Init static values which are used in faceplate animation

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @param deviceName input, the device name

 */
CPC_VR_AH_FaceplateInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
	dyn_string exceptionInfo, dsChannels;
	mapping objectStLabels;
  // Get  the system name
  string sSystemName, sControllerAlias, sControllerDpName, sChannelConstant;

  // Get the object st mapping   
 	cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.ObjectSt"), objectStLabels);
	g_params["ObjectStLabels"] = objectStLabels;
	g_params["MRestart"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);
   
  // Get the controller and instruments
	g_params[CPC_PROP_NAME_CONFIG_TYPE] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_CONFIG_TYPE, exceptionInfo);
  g_params[CPC_PROP_NAME_SOURCE1] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_SOURCE1, exceptionInfo);
  g_params[CPC_PROP_NAME_SOURCE2] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_SOURCE2, exceptionInfo);
  
  // Load the instrument name from the controller
  if(g_params[CPC_PROP_NAME_CONFIG_TYPE] == "1" || g_params[CPC_PROP_NAME_CONFIG_TYPE] == "2") {
    // Build up the controller dp name
    sSystemName = unGenericDpFunctions_getSystemName(deviceName);      
    sControllerAlias = sSystemName+g_params[CPC_PROP_NAME_SOURCE1];     
    sControllerDpName = unGenericDpFunctions_getWidgetDpName(sControllerAlias);
    // Get the controller channels for both gauge and ionpump controller
    for(int i = 1;i<=4;i++){
      // Load the parameter constant name
      sChannelConstant = (g_params[CPC_PROP_NAME_CONFIG_TYPE] == "1") ? "CPC_PARAMS_GAUGE"+i+"_NAME" : "CPC_PARAMS_PUMP"+i+"_NAME";
      // Load the channel name from the contant
      
      dsChannels[i] = cpcGenericDpFunctions_getDeviceProperty(sControllerDpName, cpcConfigGenericFunctions_getStringConstantByName(sChannelConstant), exceptionInfo);
    }
    // Set the channel instruments
    g_params[CPC_PROP_NAME_CHANNELS]	 = dsChannels;
  }
  // Apply default instrument
  g_params[CPC_PROP_NAME_INSTRUMENT] = "";
	//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
}

/** 
 Returns the list of VR_AH DPEs which should be connected on faceplate animation

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @param deviceName input, the device name
 @param dpes input/output, dpe list

 */
CPC_VR_AH_FaceplateDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
	dynAppend(dpes, deviceName + ".eventMask");
	dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
	dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
	dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");
	dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid");
	dynAppend(dpes, deviceName + ".ProcessInput.SrcChanSt");
	dynAppend(dpes, deviceName + ".ProcessInput.SrcChanSt:_online.._invalid");
	dynAppend(dpes, deviceName + ".ProcessInput.S1UpThreshSt");
	dynAppend(dpes, deviceName + ".ProcessInput.S1UpThreshSt:_online.._invalid");
	dynAppend(dpes, deviceName + ".ProcessInput.S1LoThreshSt");
	dynAppend(dpes, deviceName + ".ProcessInput.S1LoThreshSt:_online.._invalid");
	dynAppend(dpes, deviceName + ".ProcessInput.S2UpThreshSt");
	dynAppend(dpes, deviceName + ".ProcessInput.S2UpThreshSt:_online.._invalid");
	dynAppend(dpes, deviceName + ".ProcessInput.S2LoThreshSt");
	dynAppend(dpes, deviceName + ".ProcessInput.S2LoThreshSt:_online.._invalid");
	dynAppend(dpes, deviceName + ".disabled");
  
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
}

/** callback function on the device data

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @param dpes input, the dpe names
 @param values input, the dpe values

 */
CPC_VR_AH_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
  int value;
  bool invalid, bController;
  bController = g_params[CPC_PROP_NAME_CONFIG_TYPE] == "1" || g_params[CPC_PROP_NAME_CONFIG_TYPE] == "2";
  // Find the instrument causing the interlock
  if(bController){
    if(unFaceplate_fetchAnimationCBOnlineValue(dpes, values, "SrcChanSt", value, invalid) && !invalid && value <= 4 && value >= 1){
      g_params[CPC_PROP_NAME_INSTRUMENT] = g_params[CPC_PROP_NAME_CHANNELS][value];
    } else {
      g_params[CPC_PROP_NAME_INSTRUMENT] = "";
    }
  }
        
  // Animate the links to the devices
	setMultiValue(
			"ControllerButton", "toolTipText", g_params[CPC_PROP_NAME_SOURCE1],
			"ControllerButton", "visible", g_params[CPC_PROP_NAME_SOURCE1] != "" && bController,
			"InstrumentButton", "toolTipText", g_params[CPC_PROP_NAME_INSTRUMENT],
			"InstrumentButton", "visible", g_params[CPC_PROP_NAME_INSTRUMENT] != "" && bController
	);

  unFaceplate_animateTranslatedValue(dpes, values, "ObjectSt", "", "", g_params["ObjectStLabels"], "cpcColor_Faceplate_Status");

  // Animates the on/off status and objectSt panel
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_POSST", true, "cpcColor_Faceplate_Status");

	// Animates operation modes, like auto, manual, forced and their inhibit modes
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUMOST", true, "cpcColor_Faceplate_Status");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_FOMOST", true, "cpcColor_Faceplate_Status");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUINHFMO", true, "cpcColor_Faceplate_Status");

	cpcGenericObject_animateMaskEvent(dpes, values);
	cpcGenericObject_animateNeedRestart(dpes, values);

	// Animates warning modes
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOERRORW", true, "cpcColor_Faceplate_Warning");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOSIMUW", true, "cpcColor_Faceplate_Warning");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_FODIPRO", true, "cpcColor_Faceplate_Warning");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_MIOERBRST", true, "cpcColor_Faceplate_Warning");
	//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
}


/**
  Lists the dpes required for building the state of the device
  
  @param string sDeviceName The name of the device which is in UNICOS the same as the dpname
  @param dyn_string dsDpes The list of datapoints elements required for building the state
  @param bool bForHistory The DPEs will be used for state history, i.e. only DPEs with _archive
                            config are of interest

*/
CPC_VR_AH_StateDPEs(string sDeviceName, dyn_string &dsDpes, bool bForHistory = true) {
	dynAppend(dsDpes, sDeviceName + ".eventMask");                                // 3
  if(bForHistory) {
  	dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg01");                    // 4
  	dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg01:_online.._invalid");  // 5
  }
  else {
  	dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg01");                    // 4
  	dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg01:_online.._invalid");  // 5
  }
  dynAppend(dsDpes, sDeviceName + ".ProcessInput.ObjectSt");                    // 6
  dynAppend(dsDpes, sDeviceName + ".ProcessInput.ObjectSt:_online.._invalid");  // 7
	dynAppend(dsDpes, sDeviceName + ".ProcessInput.SrcChanSt");                   // 8
	dynAppend(dsDpes, sDeviceName + ".ProcessInput.SrcChanSt:_online.._invalid"); // 9
	dynAppend(dsDpes, sDeviceName + ".disabled");                                 // 10
}

/**
  Returns the main color associated with the d   evice state. This color is based on 
  the dpes listed in the CPC_VR_AH_StateDPEs for the dedicated device type. In case of the 
  state history the EvStsReg## is used because this is archived, while for determining 
  the current state the StsReg## is used.

  @param dyn_string dsDpes The list of dpe names used to determine the color
  @param dyn_anytype daValues The list of dpe values used to determine the color
  
  1  int     frontend.alarm
  2  bool    frontend.enabled
  3  bool                 device.eventMask
  4  bit32	 device.ProcessInput.(Ev)StsReg01
  5  bool 	 device.ProcessInput.(Ev)StsReg01:_online.._invalid
  6  int     device.ProcessInput.ObjectSt
  7  int     device.ProcessInput.ObjectSt:_online.._invalid
  8  int     device.ProcessInput.SrcChanSt
  9  int     device.ProcessInput.SrcChanSt:_online.._invalid
  10 bool    device.disabled
  
  @param string sColor, output, the main color string of device
  @param string sString, output, the main state string of device
  @param bool bUseOffColor, input, Off color is return in &sColor when device is Off, false in case of WidgetColor
  @param bool bBoolAlert, input, realtime alert according to UNICOS alert manager

  @param bool bIsRestartNeeded, input, there is a restart needed according to UNICOS device manager
  
  @see For the stsReg bit meaning check the specifications file: https://svn.cern.ch/reps/en-ice-svn/trunk/applications/vacuum/UNICOS_VAC_DEVICES/Design/Object_ManReg_StsReg_Mapping.xls
  @see Other usufull documentation: https://edms.cern.ch/ui/file/1228441/1.6.0/UCPC6_RP1.6.0_Manual.pdf  
  
  @return 0 when function proceed successfully, -1 when function proceed with error
*/

int CPC_VR_AH_ColorAndString(string sDeviceName, dyn_string dsDpes, dyn_anytype daValues, string &sColor,
                             string &sState, bool bUseOffColor, bool bBoolAlert = false,
                             bool bIsRestartNeeded = false){

  // --- Get device data value
  // --------------------------	   
  // Read system integrity values 
  int iSystemIntegrityAlarmValue = daValues[1];
  bool bSystemIntegrityAlarmEnabled = daValues[2];
    
  // Status valid
  bool bInvalid = daValues[5] || daValues[7] || daValues[9];

  // Object status
  bit32 bit32StsReg01 = daValues[4];
  int iObjectSt = daValues[6];
  bool bDisabled = daValues[10];
  
  // --- Get main state string
  //--------------------------
  if (sDeviceName!="") {
     sState = unVacObject_getStateString(sDeviceName, iObjectSt);
  }
  else {
     sState = "Unknown";
  }

  // Determine alarm color booleans
	bool bActive = getBit(bit32StsReg01, CPC_StsReg01_POSST) == 1;
	bool bWarning = getBit(bit32StsReg01, CPC_StsReg01_IOERRORW) == 1;

	// Mode boolean
	bool bForcedMode = getBit(bit32StsReg01, CPC_StsReg01_FoMoSt) == 1;
	bool bOn = iObjectSt == 1; // Object is on
  bool bUndefined = iObjectSt >= 3; // Undefined state of the relay
  bool bOff = iObjectSt == 0 || iObjectSt == 2;

	// Register list
	dyn_bit32 dbit32StsRegs = makeDynBit32(bit32StsReg01);

	// Create boolen priority list for corresponding colors in CPC_BODY_COLORS
	dyn_bool dbStateBool = makeDynBool(
			bInvalid,              // Invalid color
			false,                 // Alarm Unacknowledged color
			false,                 // Alarm active color
			bDisabled,             // Device disabled color
			bWarning || bActive ,  // Warning color
			bForcedMode,           // Forced mode color
      bUndefined,            // Undefined color
      bOff && bUseOffColor,  // Off color (optional color)
			false,                 // Under Range color
      true                   // OK color
	);
  
  // Fetch the body color from CPC_BODY_COLORS based on the lowest index value of the array dbBodyColorBool is true 
  unVacObject_WidgetBoolMapping(dbStateBool, CPC_BODY_COLORS, sColor);
  
  return 0;
}  


/** 
  Get the source device name of the VR_AH 
  
  @param sDeviceName The SCADA name of the hardware alarm device 
  @param iChannelNo The channel number used in the controller. If none is given this is loaded from the datapoint ".ProcessInput.SrcChanSt"
  @return The device name of the source
*/
string CPC_VR_AH_GetSourceDevice(string sDeviceName, int iChannelNo = -1){
  // The channels
  dyn_string dsChannels, exceptionInfo;  
  
  if(iChannelNo == -1){
    if(0 != dpGet(sDeviceName+".ProcessInput.SrcChanSt", iChannelNo)){
      string sErrMsg;
      sprintf(sErrMsg,"CPC_VR_AH_GetSourceDevice: Could not get DPE %s", sDeviceName+".ProcessInput.SrcChanSt");
      errClass error = makeError("unVacuum", PRIO_SEVERE, ERR_CONTROL, 107, sErrMsg);
      throw( error );	 
    }
  }
  
  // Load the sources and config types from the import configuration
  string sConfigType = cpcGenericDpFunctions_getDeviceProperty(sDeviceName, CPC_PROP_NAME_CONFIG_TYPE, exceptionInfo);
  string sSource1 = cpcGenericDpFunctions_getDeviceProperty(sDeviceName, CPC_PROP_NAME_SOURCE1, exceptionInfo);
  
  string sSystemName, sControllerAlias, sControllerDpName, sChannelConstant, sSourceDeviceName;
  
    // Check whether the channel number is in range or throw an exception
  if(iChannelNo < 1 || iChannelNo > 4){
      string sErrMsg;
      sprintf(sErrMsg, "CPC_VR_AH_GetSourceDevice: No channel found with number %d, must be one of [1,2,3,4]",iChannelNo);
      errClass error = makeError("unVacuum", PRIO_SEVERE, ERR_CONTROL, 108, sErrMsg);
      throw( error );	 
  }  
  
  //Only load the channels when a VR_PI or VR_GT is the source
  if(sConfigType == "1" || sConfigType == "2") {
      // Build up the controller dp name
      sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);      
      sControllerAlias = sSystemName+sSource1;     
      sControllerDpName = unGenericDpFunctions_getWidgetDpName(sControllerAlias);
      // Get the controller channels for both gauge and ionpump controller

      // Load the parameter constant name
      sChannelConstant = (sConfigType == "1") ? "CPC_PARAMS_GAUGE"+iChannelNo+"_NAME" : "CPC_PARAMS_PUMP"+iChannelNo+"_NAME";
      // Load the channel name from the contant
      sSourceDeviceName = cpcGenericDpFunctions_getDeviceProperty(sControllerDpName, cpcConfigGenericFunctions_getStringConstantByName(sChannelConstant), exceptionInfo);
  }
  
  return sSourceDeviceName;  
}

int CPC_R_AH_ColorFillAndString(string sDeviceName, dyn_string dsDpes, dyn_anytype daValues, string &sColor, bool &bFill, string &sState) {
  bFill = true;
  int coco = CPC_VR_AH_ColorAndString(sDeviceName, dsDpes, daValues, sColor, sState, true);
  return coco;
}
