/**@file

 // cpcVR_AHConfig.ctl
 This library contains the import and export function of the CPC_VR_AH.

 @par Creation Date
 27/10/2016

 @par Constraints
 None

 @par Usage
 Public

 @par PVSS managers
 Ui, CTRL

 @author
 Thijs van Winden (TE-VSC)

 @copyright
 &copy;Copyright CERN 2016 - all rights reserved
 */
//@{

// Constants
const string UN_CONFIG_CPC_VR_AH_DPT_NAME = "CPC_VR_AH";
//begin_TAG_SCRIPT_DEVICE_TYPE_constants

// Row imporation 
// See header line in the import file for definite mapping
// 10 GENERIC (CPC_VR_AH;deviceNumber;Alias[,DeviceLinkList];Description;Diagnostics;WWWLink;Synoptic;Domain;Nature;WidgetType;) 

// Row importation MAIN ADDRESSING (23)
const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_STSREG01 	  = 1;
const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_EVSTSREG01 	= 2;
const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_SRCCHANST   	= 3;
const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_S1UPTHRESHST = 4;
const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_S1LOTHRESHST = 5;
const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_S2UPTHRESHST = 6;
const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_S2LOTHRESHST = 7;
const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_OBJECTST 	  = 8;

const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_MANREG01 	  = 9;
const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_S1UPTHRESHR 	= 10;
const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_S1LOTHRESHR 	= 11;
const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_S2UPTHRESHR 	= 12;
const unsigned UN_CONFIG_CPC_VR_AH_ADDRESS_S2LOTHRESHR 	= 13;

const unsigned UN_CONFIG_CPC_VR_AH_DEFAULT_LOGIC 	   	= 14;
const unsigned UN_CONFIG_CPC_VR_AH_DEFAULT_UR			    = 15;		
const unsigned UN_CONFIG_CPC_VR_AH_DEFAULT_S1BELOWOK	= 16;
const unsigned UN_CONFIG_CPC_VR_AH_DEFAULT_S2BELOWOK	= 17;
const unsigned UN_CONFIG_CPC_VR_AH_DEFAULT_S1UPTHRESH	= 18;
const unsigned UN_CONFIG_CPC_VR_AH_DEFAULT_S1LOTHRESH	= 19;
const unsigned UN_CONFIG_CPC_VR_AH_DEFAULT_S2UPTHRESH	= 20;
const unsigned UN_CONFIG_CPC_VR_AH_DEFAULT_S2LOTHRESH	= 21;

// Row importation MAIN MAPPING (1)
const unsigned UN_CONFIG_CPC_VR_AH_MAPPING = 22;

// Row imporation ARCHIVE MODE (1)
const unsigned UN_CONFIG_CPC_VR_AH_ARCHIVE_ACTIVE 	= 23;

const unsigned UN_CONFIG_CPC_VR_AH_LENGTH = 23;

// Row imporatation ARCHIVE(3)
const unsigned UN_CONFIG_CPC_VR_AH_ARCHIVE_BOOLEAN_ARCHIVE = 1;
const unsigned UN_CONFIG_CPC_VR_AH_ARCHIVE_ANALOG_ARCHIVE = 2;
const unsigned UN_CONFIG_CPC_VR_AH_ARCHIVE_EVENT_ARCHIVE = 3;
const unsigned UN_CONFIG_CPC_VR_AH_ARCHIVE_LENGTH = 3;

// Row imporatation ADDITIONAL
const unsigned UN_CONFIG_CPC_VR_AH_ADDITIONAL_MASKEVENT = 1;
const unsigned UN_CONFIG_CPC_VR_AH_ADDITIONAL_PARAMETERS = 2;
const unsigned UN_CONFIG_CPC_VR_AH_ADDITIONAL_PARENTS = 3;
const unsigned UN_CONFIG_CPC_VR_AH_ADDITIONAL_CHILDREN = 4;
const unsigned UN_CONFIG_CPC_VR_AH_ADDITIONAL_TYPE = 5;
const unsigned UN_CONFIG_CPC_VR_AH_ADDITIONAL_SECOND_ALIAS = 6;

const unsigned UN_CONFIG_CPC_VR_AH_ADDITIONAL_LENGTH = 6;
//end_TAG_SCRIPT_DEVICE_TYPE_constants

//------------------------------------------------------
// NOTES :
// (1) ARCHIVE_ACTIVE
//N for NO archiving
//VA,0.1,N for Absolute Value archiving with 0.1 deadband and NO time
//VR,0.1,N as above, but Relative Deadband
//Y for Time-only archiving
//VA,0.1,A for Absolute Value archiving with 0.1 deadband AND Time
//VR,0.1,A for Relative Value archiving with 0.1 deadband AND Time  DEFAULT ARCHIVE SETUP FOR VACUUM ANALOG VALUE
//VA,0.1,O for Absolute Value archiving with 0.1 deadband OR Time
//VR,0.1,O for Relative Value archiving with 0.1 deadband OR Time
//A for Old/new comparison AND time
//O for Old/new comparison OR time

/**
 DPE configuration
 TODO: use fw info instead
 */
mapping CPC_VR_AHConfig_getConfig() {
	mapping config;
	mapping props;

	// Status adresses
	// StsReg01
	mappingClear(props);
	props["address"] = ".ProcessInput.StsReg01";
	props["dataType"] = CPC_UINT16;
	config["StsReg01"] = props;
	// evStsReg01
	mappingClear(props);
	props["hasArchive"] = true;
	props["address"] = ".ProcessInput.evStsReg01";
	props["dataType"] = CPC_INT32;
	config["evStsReg01"] = props;
	// SrcChanSt
	mappingClear(props);
	props["address"] = ".ProcessInput.SrcChanSt";
	props["dataType"] = CPC_UINT16;
	config["SrcChanSt"] = props;
	// S1UpThreshSt
	mappingClear(props);
	props["address"] = ".ProcessInput.S1UpThreshSt";
	props["dataType"] = CPC_FLOAT;
	config["S1UpThreshSt"] = props;
	// S1LoThreshSt
	mappingClear(props);
	props["address"] = ".ProcessInput.S1LoThreshSt";
	props["dataType"] = CPC_FLOAT;
	config["S1LoThreshSt"] = props;
	// S2UpThreshSt
	mappingClear(props);
	props["address"] = ".ProcessInput.S2UpThreshSt";
	props["dataType"] = CPC_FLOAT;
	config["S2UpThreshSt"] = props;
	// S2LoThreshSt
	mappingClear(props);
	props["address"] = ".ProcessInput.S2LoThreshSt";
	props["dataType"] = CPC_FLOAT;
	config["S2LoThreshSt"] = props;
	// ObjectSt
	mappingClear(props);
	props["hasMapping"] = true;
	props["hasUnit"] = false;
	props["hasFormat"] = false;
	props["address"] = ".ProcessInput.ObjectSt";
	props["dataType"] = CPC_UINT16;
	config["ObjectSt"] = props;
	
	// Request adresses
	// ManReg01
	mappingClear(props);
	props["address"] = ".ProcessOutput.ManReg01";
	props["dataType"] = CPC_UINT16;
	config["ManReg01"] = props;
	// S1UpThreshR
	mappingClear(props);
	props["address"] = ".ProcessOutput.S1UpThreshR";
	props["dataType"] = CPC_FLOAT;
	config["S1UpThreshR"] = props;
	// S1LoThreshR
	mappingClear(props);
	props["address"] = ".ProcessOutput.S1LoThreshR";
	props["dataType"] = CPC_FLOAT;
	config["S1LoThreshR"] = props;
	// S2UpThreshR
	mappingClear(props);
	props["address"] = ".ProcessOutput.S2UpThreshR";
	props["dataType"] = CPC_FLOAT;
	config["S2UpThreshR"] = props;
	// S2LoThreshR
	mappingClear(props);
	props["address"] = ".ProcessOutput.S2LoThreshR";
	props["dataType"] = CPC_FLOAT;
	config["S2LoThreshR"] = props;
  
	return config;
}

/**
 Returns the list of parameter names that passed via config line.
 */
dyn_string CPC_VR_AHConfig_getParamNames() {
	return makeDynString(CPC_PROP_NAME_SOURCE1, CPC_PROP_NAME_SOURCE2, CPC_PROP_NAME_CONFIG_TYPE);
}

/** Check custom configuration
 */
CPC_VR_AHConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
//end_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
}

/** Set custom configuration
 */
CPC_VR_AHConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig

	//Archive setting for ObjectSt
	//fwArchive_set(string dpe, string archiveClassName, int archiveType, int smoothProcedure, float deadband, float timeInterval, dyn_string &exceptionInfo, bool checkClass = TRUE)
	fwArchive_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.ObjectSt",
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
			DPATTR_ARCH_PROC_SIMPLESM,
			DPATTR_COMPARE_OLD_NEW,
			0,
			0,
			exceptionInfo);

//end_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig
}

/**
 Purpose: Export CPC_VR_AH Devices

 Usage: External function

 PVSS manager usage: NG, NV
 */
CPC_VR_AHConfig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
	string sObject = UN_CONFIG_CPC_VR_AH_DPT_NAME;
	int amount = dynlen(dsDpList);
	for (int i = 1; i <= amount; i++) {
		dyn_string dsDpParameters;
		string currentDP = dsDpList[i];

		if (!dpExists(currentDP)) {continue;}

		//begin_TAG_SCRIPT_DEVICE_TYPE_export
		unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);

		cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg02", true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg03", true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg02", true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "ObjectSt", true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", false, dsDpParameters);

		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString("ProcessInput.OnSt", "ProcessInput.ObjectSt"), dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01", "ProcessInput.evStsReg02"), dsDpParameters);
		cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
		cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);
		//end_TAG_SCRIPT_DEVICE_TYPE_export

		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	}
}

/**
 Purpose: Export CPC_VR_AH Devices for S7_PLC front-end

 Usage: External function

 PVSS manager usage: NG, NV
 */
S7_PLC_CPC_VR_AH_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
	CPC_VR_AHConfig_ExportConfig(dsDpList, exceptionInfo);
}

/**
 Purpose: Export CPC_VR_AH Devices for _UnPlc front-end

 Usage: External function

 PVSS manager usage: NG, NV
 */
_UnPlc_CPC_VR_AH_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
	CPC_VR_AHConfig_ExportConfig(dsDpList, exceptionInfo);
}

OPCUA_CPC_VR_AH_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
	CPC_VR_AHConfig_ExportConfig(dsDpList, exceptionInfo);
}

BACnet_CPC_VR_AH_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
	CPC_VR_AHConfig_ExportConfig(dsDpList, exceptionInfo);
}

//@}
