/**@file

// cpcVR_PI.ctl
This library contains the widget, faceplate, etc. functions of VR_PI.

@par Creation Date
  dd/mm/yyyy

@par Modification History

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author
  Marcin Bes (BE-ICS)
  the author (DEP-GROUP)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{

#uses "vacuumExtension/classes/DeviceListCriteria.ctl"

/** Returns the list of VR_PI DPE with alarm config that can be acknowledged

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name DP name
@param dpType input, device type
@param dsNeedAck output, the lsit of DPE
*/
CPC_VR_PI_AcknowledgeAlarm(string deviceName, string dpType, dyn_string &dsNeedAck) {
//begin_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
    dsNeedAck = makeDynString(UN_ACKNOWLEDGE_PLC);
//end_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
}

/** Function called from snapshot utility of the treeDeviceOverview to get the time and value

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name
@param deviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
CPC_VR_PI_ObjectListGetValueTime(string deviceName, string deviceType, dyn_string &dsReturnData) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
    float value;
    string sTime;

    //dpGet(deviceName + ".ProcessInput.PosSt", value,
    //      deviceName + ".ProcessInput.PosSt:_online.._stime", sTime);

    //dsReturnData[1] = sTime;
    //dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(deviceName + ".ProcessInput.PosSt"), value) + " " + dpGetUnit(deviceName + ".ProcessInput.PosSt"); // formated value with unit
//end_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
}

/** pop-up menu

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param dsAccessOk input, the access control
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
CPC_VR_PI_MenuConfiguration(string deviceName, string dpType, dyn_string dsAccessOk, dyn_string &menuList) {
    dyn_string exceptionInfo;
//begin_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
    dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT, UN_POPUPMENU_TREND_TEXT);

    // Add device context menu items
    unVacObject_addAliasActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    
    unVacObject_addDisableEnableActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    int iPos = unVacObject_addVacuumExtensionActionToMenu(deviceName, dpType,  dsAccessOk, menuList);
    iPos = unVacObject_addProcessActionToMenu(deviceName, dpType,  dsMenuConfig, dsAccessOk, menuList, iPos+1);
//end_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
}

/** handle the answer of the popup menu

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param menuList input, the access control
@param menuAnswer input, selected menu value
*/
CPC_VR_PI_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer) {
    dyn_string exceptionInfo;
    
    // The help file
    string helpFile = unVacObject_getHelpLink(dpType);

    if(!unVacObject_handleCustomUnicosMenu(deviceName, dpType, menuList, menuAnswer, helpFile, exceptionInfo)){
      cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
    }
    
    if (dynlen(exceptionInfo) > 0) {
        unSendMessage_toExpertException("Right Click", exceptionInfo);
    }
}

/** Init static values which are used in widget animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_VR_PI_WidgetInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics

//end_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
}

/** Returns the list of VR_PI DPEs which should be connected on widget animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_VR_PI_WidgetDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs

    dynAppend(dpes, deviceName + ".eventMask");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");	


//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
}

/** callback function on the device data

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
CPC_VR_PI_WidgetAnimation(dyn_string dpes, dyn_anytype values, string widgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation

    int iSystemIntegrityAlarmValue = values[1];
    bool bSystemIntegrityAlarmEnabled = values[2];
    bool bLocked = values[3];
    string sSelectedManager = values[4];
    bool bIsCommentsActive = values[5];
    
    int iMask = values[6];
    bit32 bit32StsReg01 = values[7];
    bool bStsReg01Invalid = values[8];
    
    string sWarningLetter, sWarningColor, sAlarmLetter, sAlarmColor, sControlLetter, sControlColor, sSelectColor;
       
    // 1. Select
    bool bLockVisible;
    unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sSelectColor, bLockVisible);

    // 2. Warning
    sWarningLetter = "";
    sWarningColor = "unFaceplate_WarningActive";

    if(!bStsReg01Invalid && getBit(bit32StsReg01, CPC_StsReg01_IOERRORW) == 1) {
      sWarningLetter = CPC_WIDGET_TEXT_WARNING_ERROR;
    } else if(!bStsReg01Invalid && getBit(bit32StsReg01, CPC_StsReg01_IOSIMUW) == 1) {
      sWarningLetter = CPC_WIDGET_TEXT_WARNING_SIMU;
    }

    bool bInvalidLetter = bStsReg01Invalid;
    // 2.1 if the device is not correclty running (the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)
    // or the checking is not enabled then set the letter to O and the color to not valid data
    cpcGenericObject_WidgetValidnessAnimation(bSystemIntegrityAlarmEnabled, iSystemIntegrityAlarmValue, bInvalidLetter, sWarningLetter, sWarningColor);

    bool bInvalidBody = bStsReg01Invalid;

    // 3. Alarm

    // 4. control state
    sControlLetter = "";
    sControlColor = "unWidget_ControlStateManualLocal";
    
    dyn_string sBodyColor;
    // set color of body
    if (bInvalidBody) { // blue
      sBodyColor = "unDataNotValid";
    } else if((getBit(bit32StsReg01, CPC_StsReg01_IOERRORW) == 1)){
      sBodyColor = "cpcColor_Widget_Warning";
    } else { // normal green
      sBodyColor = "cpcColor_Widget_Status";
    }

    // 5. Animate all
    if (!g_bSystemConnected) {
      return;
    }
    
    setMultiValue("Body1",            "foreCol",  sBodyColor,
                  "Body2",            "foreCol",  sBodyColor,

                  "WarningText",      "text",     sWarningLetter, 
                  "WarningText",      "foreCol",  sWarningColor,

                  "AlarmText",        "text",  "",
                  "ControlStateText", "text",  "",

                  "commentsButton",   "visible",  bIsCommentsActive,
                  "SelectArea",       "foreCol",  sSelectColor, 
                  "LockBmp",          "visible",  bLockVisible,
                  "WidgetArea",       "visible",  true,

                  "eventState",       "visible",  iMask == 0
    );
          
    
    
    
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
}

/** Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.

*/
CPC_VR_PI_WidgetDisconnection(string sWidgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
    setMultiValue("Body1",            "foreCol",  "unDataNoAccess",
                  "Body2",            "foreCol",  "unDataNoAccess",
                  "WarningText",      "text",     "", 
                  "AlarmText",        "text",     "", 
                  "ControlStateText", "text",     "",
                  "SelectArea",       "foreCol",  "", 
                  "LockBmp",          "visible",  false, 
                  "WidgetArea",       "visible",  false,
                  "eventState",       "visible",  false);
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
}

/** Return button configuration including access level

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
mapping CPC_VR_PI_ButtonConfig(string deviceName) {
    mapping buttons;
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
    dyn_string exceptionInfo;

    buttons[UN_FACEPLATE_BUTTON_SELECT]               = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_REFRESH]              = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_RESTORE_ALL_AND_WRITE]= UN_ACCESS_RIGHTS_EXPERT;
    buttons[CPC_FACEPLATE_BUTTON_PARAMETER_SET_VALUE] = UN_ACCESS_RIGHTS_EXPERT;
    buttons[UN_FACEPLATE_MASKEVENT]                   = UN_ACCESS_RIGHTS_EXPERT;
    
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
    return buttons;
}

/** Configure the list of dpes that needs for buttons animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the name of device
@param dpes input, the list of dpes to connect
*/
CPC_VR_PI_ButtonDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
}

/** Set the state of the contextual button of the device

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param dpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
CPC_VR_PI_ButtonSetState(string deviceName, string dpType, dyn_string dsUserAccess, dyn_string dsData) {
    bool buttonEnabled, bSelected;
    string localManager;
    bool bLocked = dsData[1];
    string sSelectedUIManager = dsData[2];

    localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
    bSelected = (localManager == "S"); // Selection state
    dyn_string dsButtons = unSimpleAnimation_ButtonList(deviceName);

//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
    for (int i = 1; i <= dynlen(dsButtons); i++) {
        buttonEnabled = bSelected && (dynContains(dsUserAccess, dsButtons[i]) > 0); // User access
        if (dsButtons[i] == UN_FACEPLATE_BUTTON_SELECT) {
            unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
        }
        else {
            cpcButton_setButtonState(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], buttonEnabled);
        
        }
    }
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
}

/** Init static values which are used in faceplate animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_VR_PI_FaceplateInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
    dyn_string exceptionInfo;
    
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.ObjectSt"), g_params["OBJECTST_MAPPING"]);
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.ErrCoSt"), g_params["ERRCOST_MAPPING"]);
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.Rel1St"), g_params["RELAY_MAPPING"]);
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.Rel1SrcSt"), g_params["SOURCE_MAPPING"]);
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.RelFilterSt"), g_params["RELFILTERST_MAPPING"]);

    g_params["P1"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_PUMP1_NAME, exceptionInfo);
    g_params["P2"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_PUMP2_NAME, exceptionInfo);
    g_params["P3"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_PUMP3_NAME, exceptionInfo);
    g_params["P4"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_PUMP4_NAME, exceptionInfo);
    g_params["R1"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_RELAY1_NAME, exceptionInfo);
    g_params["R2"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_RELAY2_NAME, exceptionInfo);
    g_params["R3"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_RELAY3_NAME, exceptionInfo);
    g_params["R4"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_RELAY4_NAME, exceptionInfo);
    
    g_params["PRESSURE_UNIT"] = "mbar";
    g_params["PRESSURE_FORMAT"] = "%1.2e";
    

    //cpcGenericObject_StringToMapping("0=closed normally,1=open normally,2=forced closed,3=forced open,4=unknown", g_params["RELAY_MAPPING"]);
    cpcGenericObject_StringToMapping("0=not configured,1=200 W Neg,2=200 W Pos,3=80 + 80 W Neg, 4=80 + 80 W Pos,5=80 W Neg,6=80 W Pos", g_params["HCONFSXST_MAPPING"]);
    cpcGenericObject_StringToMapping("0=Torr,1=mbar (default),2=Pa", g_params["PRUNITST_MAPPING"]);    
    cpcGenericObject_StringToMapping("0=not accepted,1=remote,2=local,3=profibus", g_params["CTRLMOST_MAPPING"]);
    cpcGenericObject_StringToMapping("0=HV Off,1=HV restart in last state", g_params["RESTARTMOST_MAPPING"]);   
    cpcGenericObject_StringToMapping("0=On when current is higher than 1mA,1=Always on", g_params["FANMOST_MAPPING"]);    
    //cpcGenericObject_StringToMapping("0=FAST (K:1),1=MEDIUM (K:2) (default),2=SLOW (K:4)", g_params["RELFILTERST_MAPPING"]);
    //cpcGenericObject_StringToMapping("0=Disabled,1=CH1,2=CH2,3=CH3,4=CH4", g_params["SOURCE_MAPPING"]);
    cpcGenericObject_StringToMapping("0=OK,1=Undefined error,2=Invalid,225=Wrong Request code,226=Wrong Parameter value,227=Request processing error", g_params["COMMST_MAPPING"]);
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
}
 
/** Returns the list of VR_PI DPEs which should be connected on faceplate animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_VR_PI_FaceplateDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
    dynAppend(dpes, deviceName + ".eventMask");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.ErrCoSt");
    dynAppend(dpes, deviceName + ".ProcessInput.ErrCoSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.CommSt");
    dynAppend(dpes, deviceName + ".ProcessInput.CommSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.RelFilterSt");
    dynAppend(dpes, deviceName + ".ProcessInput.RelFilterSt:_online.._invalid");
    
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1St");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1St:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2St");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2St:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3St");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3St:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4St");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4St:_online.._invalid");
    
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1SrcSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1SrcSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2SrcSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2SrcSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3SrcSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3SrcSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4SrcSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4SrcSt:_online.._invalid");
    
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1ThreSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1ThreSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2ThreSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2ThreSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3ThreSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3ThreSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4ThreSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4ThreSt:_online.._invalid");
    
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1HystSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1HystSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2HystSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2HystSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3HystSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3HystSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4HystSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4HystSt:_online.._invalid");
    
    dynAppend(dpes, deviceName + ".ProcessInput.ParGetCoSt");
    dynAppend(dpes, deviceName + ".ProcessInput.ParGetCoSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.ParGetValSt");
    dynAppend(dpes, deviceName + ".ProcessInput.ParGetValSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.HConfS1St");
    dynAppend(dpes, deviceName + ".ProcessInput.HConfS1St:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.HConfS2St");
    dynAppend(dpes, deviceName + ".ProcessInput.HConfS2St:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.FirmWSt");
    dynAppend(dpes, deviceName + ".ProcessInput.FirmWSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.RestartMoSt");
    dynAppend(dpes, deviceName + ".ProcessInput.RestartMoSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.CtrlMoSt");
    dynAppend(dpes, deviceName + ".ProcessInput.CtrlMoSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.PrUnitSt");
    dynAppend(dpes, deviceName + ".ProcessInput.PrUnitSt:_online.._invalid"); 
    dynAppend(dpes, deviceName + ".ProcessInput.FanMoSt");
    dynAppend(dpes, deviceName + ".ProcessInput.FanMoSt:_online.._invalid");    
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
}

/** callback function on the device data

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
CPC_VR_PI_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values) {  
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
    
    // Check whether the status parameter code has changed
    if(!mappingHasKey(g_params, "PARGETCOST_OLD") || g_params["PARGETCOST_OLD"] != values[46]){
      string sDeviceName = unGenericDpFunctions_getDpName(dpes[3]);
      bool bResult = CPC_VR_PI_handleAsyncParameters(sDeviceName, values[46], values[48], values[47] || values[49]);
      CPC_VR_PI_handleAsyncError(sDeviceName, values[46], values[48], values[47] || values[49], !bResult);
      // UPdate parameter code
      g_params["PARGETCOST_OLD"] = values[46];        
    }
    

    // General status block
    unFaceplate_animateTranslatedValue(dpes, values, "ObjectSt", "", "", g_params["OBJECTST_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "ErrCoSt", "", "", g_params["ERRCOST_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel1St", "", "", g_params["RELAY_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel2St", "", "", g_params["RELAY_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel3St", "", "", g_params["RELAY_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel4St", "", "", g_params["RELAY_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "CommSt", "", "", g_params["COMMST_MAPPING"], "cpcColor_Faceplate_Status");

    // Relay block
    unFaceplate_animateTranslatedValue(dpes, values, "Rel1SrcSt", "", "", g_params["SOURCE_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel2SrcSt", "", "", g_params["SOURCE_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel3SrcSt", "", "", g_params["SOURCE_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel4SrcSt", "", "", g_params["SOURCE_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel1ThreSt", g_params["PRESSURE_UNIT"], g_params["PRESSURE_FORMAT"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel2ThreSt", g_params["PRESSURE_UNIT"], g_params["PRESSURE_FORMAT"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel3ThreSt", g_params["PRESSURE_UNIT"], g_params["PRESSURE_FORMAT"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel4ThreSt", g_params["PRESSURE_UNIT"], g_params["PRESSURE_FORMAT"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel1HystSt", "%", "%d", "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel2HystSt", "%", "%d", "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel3HystSt", "%", "%d", "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel4HystSt", "%", "%d", "cpcColor_Faceplate_Status");    
    unFaceplate_animateTranslatedValue(dpes, values, "RelFilterSt", "", "", g_params["RELFILTERST_MAPPING"], "cpcColor_Faceplate_Status");   
    
    // Controller parameter async block
    unFaceplate_animateTranslatedValue(dpes, values, "HConfS1St", "", "", g_params["HCONFSXST_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "HConfS2St", "", "", g_params["HCONFSXST_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "PrUnitSt", "", "", g_params["PRUNITST_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "CtrlMoSt", "", "", g_params["CTRLMOST_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "RestartMoSt", "", "", g_params["RESTARTMOST_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "FanMoSt", "", "", g_params["FANMOST_MAPPING"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "CommSt", "", "", g_params["COMMST_MAPPING"], "cpcColor_Faceplate_Status");

    string refAlias;
    if(isDollarDefined("$sRefDpName") && $sRefDpName != ""){
        refAlias = unGenericDpFunctions_getAlias($sRefDpName);
        refAlias = substr(refAlias, strpos(refAlias, ":")+1);
    }     
    
    for(int i = 1; i<=4;i++){    
      CPC_VR_PI_FaceplatePumpAnimation(i, refAlias, dpes, values);
      CPC_VR_PI_FaceplateRelayAnimation(i, refAlias, dpes, values);
    }

    cpcGenericObject_animateMaskEvent(dpes, values);

    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOERRORW", true, "cpcColor_Faceplate_Warning");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOSIMUW", true, "cpcColor_Faceplate_Warning");
    
     


//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
}

/** 
  Animates a the pumps data in the faceplate
  
  @param iPumpNo The number of the pump (1,4) generally
  @param dpes The dpe names supplied to the original faceplate
  @param values The values corresponding to these dpes
  @return void
*/
CPC_VR_PI_FaceplatePumpAnimation(int iPumpNo, string refAlias, dyn_string dpes, dyn_anytype values){
    string sPumpName = "P"+iPumpNo, sColor;
    // Check whether connected
    if(g_params[sPumpName] != ""){
      // Base the relay color on the reference alias. Red if the reference is coming from this particular VR_PI
      sColor = g_params[sPumpName] == refAlias ? "red" : "black";
      // Set the name and layout
      setMultiValue(sPumpName+"Name", "toolTipText", g_params[sPumpName], sPumpName+"Name", "foreCol", sColor);
    } else {
      setMultiValue(sPumpName+"Name", "enabled", false);
    }  
}

/** 
  Animates a relay data in the faceplate
  
  @param iRelayNo The number of the relay (1,4)
  @param dpes The dpe names supplied to the original faceplate
  @param values The values corresponding to these dpes
  @return void
*/
CPC_VR_PI_FaceplateRelayAnimation(int iRelayNo, string refAlias, dyn_string dpes, dyn_anytype values){
    string sRelay = "R"+iRelayNo, sRelayColor;      
    // Check whether connected
    if(g_params[sRelay] != ""){
       // Base the relay color on the reference alias. Red if the reference is coming from this particular VR_AH
       sRelayColor = g_params[sRelay] == refAlias ? "red" : "black";
       // Set the name and layout
       setMultiValue(sRelay+"Name", "toolTipText", g_params[sRelay], sRelay+"Name", "foreCol", sRelayColor);     
     } else {
       setMultiValue(sRelay+"Name", "enabled", false);              
     }    
}

/** 
  Handle the async parameters coming from the PLC. This can be read by setting the ReadOVSt bit to 1. Will automatically be set to zero when all parameters are read
    
  @param sDeviceName The name of the device 
  @param iParameterCode The code of the parameters
  @param b32ParameterValue The parameter value before casting to the specific datatype of the DPE
  @param bInvalid Invalid parameter of the combined dpes
  @return True the dpe has been set, false otherwise
  
  @throws UnVacuumException DpeDoesNotExists => Implementation error 
  @throws UnVacuumException FailedToSetDpe   => Runtime error
*/
bool CPC_VR_PI_handleAsyncParameters(string sDeviceName, int iParameterCode, anytype fParameterValue, bool bInvalid){
  if(bInvalid) return false;
  
  // The list of DPES
  dyn_string dsDpes = makeDynString(
      "HConfS1St",
      "HConfS2St",
      "FirmWSt",
      "", //"FanTempSt",
      "", //"Ch1TempSt",
      "", //"Ch2TempSt",
      "", //"Ch3TempSt",
      "", //"Ch4TempSt",
      "RestartMoSt",
      "CtrlMoSt",
      "PrUnitSt",
      "FanMoSt",
      "Ch1ProtSt",
      "Ch2ProtSt",
      "Ch3ProtSt",
      "Ch4ProtSt"
      //"Ch1Pr100nASt",
      //"Ch1Pr10uASt",
      //"Ch1Pr10mASt",
      //"Ch1Pr400mASt",
      //"Ch2Pr100nASt",
      //"Ch2Pr10uASt",
      //"Ch2Pr10mASt",
      //"Ch2Pr400mASt",
      //"Ch3Pr100nASt",
      //"Ch3Pr10uASt",
      //"Ch3Pr10mASt",
      //"Ch3Pr400mASt",
      //"Ch4Pr100nASt",
      //"Ch4Pr10uASt",
      //"Ch4Pr10mASt",
      //"Ch4Pr400mASt"
  );   
  
  // Waiting in IDLE
  if(iParameterCode == 0){
    return true;
  } 
    
  // Check if valid parameter code
  if(iParameterCode < 1 || iParameterCode > dynlen(dsDpes)){
    return true;
  }
  
  string sName = dsDpes[iParameterCode];
  
  // If no dpe name is registered assume it's ignored
  if(sName == ""){
    return false;
  }

  // Build up the dpe name
  string sDpeName = sDeviceName+".ProcessInput."+sName;
  
  // Check if the dpe actually exists
  if(!dpExists(sDpeName)){
    throw(makeError("unVacuum", PRIO_SEVERE, ERR_IMPL, 105, "CPC_VR_PI_handleAsyncParameters: DPE name "+sDpeName +" does not exists"));     
  }
  
  // Set the dpe
  if(0 != dpSetWait(sDpeName, fParameterValue)){
    throw(makeError("unVacuum", PRIO_SEVERE, ERR_CONTROL, 106, "CPC_VR_PI_handleAsyncParameters: Failed to set DPE "+sDpeName));     
  }
  
  return true;
}

/** 
  Handle the async parameters coming from the PLC. This can be read by setting the ReadOVSt bit to 1. Will automatically be set to zero when all parameters are read
    
  @param sDeviceName The name of the device 
  @param iParameterCode The code of the parameters
  @param fParameterValue The parameter value before casting to the specific datatype of the DPE
  @param bInvalid Invalid parameter of the combined dpes
  @param bError The error detected by the setting of the parameter
  @return True the dpe has been set, false otherwise
  
  @throws UnVacuumException DpeDoesNotExists => Implementation error 
  @throws UnVacuumException FailedToSetDpe   => Runtime error
*/
bool CPC_VR_PI_handleAsyncError(string sDeviceName, int iParameterCode, anytype fParameterValue, bool bInvalid, bool bError){
    string sErrorDpeName = sDeviceName+".ProcessInput.CommSt";
    int iValue = iParameterCode;

    // Throw error of the communication state dpe does not exists
    if(!dpExists(sErrorDpeName)){
      throw(makeError("unVacuum", PRIO_SEVERE, ERR_IMPL, 105, "CPC_VR_PI_handleAsyncParameters: DPE name "+sErrorDpeName +" does not exists"));     
    }    
    
    // The error codes
    dyn_int diErrorCodes = makeDynInt(
        225, // Wrong Request code
        226, // Wrong Parameter value
        227  // Request processing error        
    );
    
    // The error status code 
    int iError = dynContains(diErrorCodes, iParameterCode);
    
    if(bError){
      if(bInvalid){
        iValue = 2;
      } else if(iError < 1){
        iValue = 1;              
      }      
    } else {
        iValue = 0;
    }
    
    if(0 != dpSetWait(sErrorDpeName, iValue)){
      throw(makeError("unVacuum", PRIO_SEVERE, ERR_CONTROL, 106, "CPC_VR_PI_handleAsyncParameters: Failed to set DPE "+sDpeName));     
    }  
    
}

//@}
