/**@file

// cpcVR_PIConfig.ctl
This library contains the import and export function of the CPC_VR_PI.

@par Creation Date
  24/05/2016

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author
  Marcin Bes (BE-ICS)

@copyright
        &copy;Copyright CERN 2016 - all rights reserved
*/
//@{


// Constants
const string UN_CONFIG_CPC_VR_PI_DPT_NAME = "CPC_VR_PI";

//begin_TAG_SCRIPT_DEVICE_TYPE_constants
const unsigned UN_CONFIG_CPC_VR_PI_REL1THREST_UNIT             = 1;
const unsigned UN_CONFIG_CPC_VR_PI_REL2THREST_UNIT             = 1;
const unsigned UN_CONFIG_CPC_VR_PI_REL3THREST_UNIT             = 1;
const unsigned UN_CONFIG_CPC_VR_PI_REL4THREST_UNIT             = 1;
const unsigned UN_CONFIG_CPC_VR_PI_REL1THREST_FORMAT             = 2;
const unsigned UN_CONFIG_CPC_VR_PI_REL2THREST_FORMAT             = 2;
const unsigned UN_CONFIG_CPC_VR_PI_REL3THREST_FORMAT             = 2;
const unsigned UN_CONFIG_CPC_VR_PI_REL4THREST_FORMAT             = 2;
const unsigned UN_CONFIG_CPC_VR_PI_DEADBAND                     = 3;
const unsigned UN_CONFIG_CPC_VR_PI_DEADBAND_TYPE                = 4;
const unsigned UN_CONFIG_CPC_VR_PI_ARCHIVE_ACTIVE            = 5; 
const unsigned UN_CONFIG_CPC_VR_PI_ARCHIVE_TIME_FILTER     = 6; 

// Device outputs                                         
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_STSREG01 	       = 7;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_EVSTSREG01 	       = 8;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL1ST 		       = 9;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL2ST 		       = 10;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL3ST 		   	= 11;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL4ST 		     = 12;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_RELFILTERST       = 13;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL1SRCST	     = 14;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL2SRCST	     = 15;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL3SRCST	     = 16;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL4SRCST	     = 17;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL1THREST	     = 18;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL2THREST	     = 19;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL3THREST	     = 20;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL4THREST	     = 21;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL1HYSTST	     = 22;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL2HYSTST	     = 23;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL3HYSTST	     = 24;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_REL4HYSTST	     = 25; 
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_OBJECTST		     = 26;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_ERRCOST		   	= 27;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_PARGETCOST	    = 28;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_PARGETVALST      = 29;
                                                             
// Manual requests                                           
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_MANREG01		     = 30;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_MANREG02		       = 31;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_PARSETCOR	   = 32;
const unsigned UN_CONFIG_CPC_VR_PI_ADDRESS_PARVALR		    = 33;	
                                                            
const unsigned UN_CONFIG_CPC_VR_PI_RELFILTERST_DEFAULT       = 34;
const unsigned UN_CONFIG_CPC_VR_PI_REL1SRC_DEFAULT 		     = 35;
const unsigned UN_CONFIG_CPC_VR_PI_REL2SRC_DEFAULT 		     = 36;
const unsigned UN_CONFIG_CPC_VR_PI_REL3SRC_DEFAULT 		     = 37;
const unsigned UN_CONFIG_CPC_VR_PI_REL4SRC_DEFAULT 		     = 38;
const unsigned UN_CONFIG_CPC_VR_PI_REL1THRES_DEFAULT 	     = 39;
const unsigned UN_CONFIG_CPC_VR_PI_REL2THRES_DEFAULT 	     = 40;
const unsigned UN_CONFIG_CPC_VR_PI_REL3THRES_DEFAULT 	     = 41;
const unsigned UN_CONFIG_CPC_VR_PI_REL4THRES_DEFAULT 	     = 42;
const unsigned UN_CONFIG_CPC_VR_PI_REL1HYST_DEFAULT		     = 43;
const unsigned UN_CONFIG_CPC_VR_PI_REL2HYST_DEFAULT		     = 44;
const unsigned UN_CONFIG_CPC_VR_PI_REL3HYST_DEFAULT		   = 45;
const unsigned UN_CONFIG_CPC_VR_PI_REL4HYST_DEFAULT		   = 46;
                                                                
// Mappings                                                     
const unsigned UN_CONFIG_CPC_VR_PI_OBJECTST_MAPPING        = 47;
const unsigned UN_CONFIG_CPC_VR_PI_ERRCOST_MAPPING         = 48;
const unsigned UN_CONFIG_CPC_VR_PI_REL1ST_MAPPING          = 49;
const unsigned UN_CONFIG_CPC_VR_PI_REL2ST_MAPPING          = 49;
const unsigned UN_CONFIG_CPC_VR_PI_REL3ST_MAPPING          = 49;
const unsigned UN_CONFIG_CPC_VR_PI_REL4ST_MAPPING          = 49;
const unsigned UN_CONFIG_CPC_VR_PI_RELFILTERST_MAPPING     = 50;
const unsigned UN_CONFIG_CPC_VR_PI_REL1SRCST_MAPPING       = 51;
const unsigned UN_CONFIG_CPC_VR_PI_REL2SRCST_MAPPING       = 51;
const unsigned UN_CONFIG_CPC_VR_PI_REL3SRCST_MAPPING       = 51;
const unsigned UN_CONFIG_CPC_VR_PI_REL4SRCST_MAPPING       = 51;

const unsigned UN_CONFIG_CPC_VR_PI_LENGTH                   = 51;

// Archive
const unsigned UN_CONFIG_CPC_VR_PI_ARCHIVE_BOOLEAN_ARCHIVE 	= 1;
const unsigned UN_CONFIG_CPC_VR_PI_ARCHIVE_ANALOG_ARCHIVE   = 2;
const unsigned UN_CONFIG_CPC_VR_PI_ARCHIVE_EVENT_ARCHIVE    = 3;
const unsigned UN_CONFIG_CPC_VP_PI_ARCHIVE_LENGTH           = 3;

// Additional
const unsigned UN_CONFIG_CPC_VR_PI_ADDITIONAL_LENGTH          = 7;
const unsigned UN_CONFIG_CPC_VR_PI_ADDITIONAL_MASKEVENT       = 1;
const unsigned UN_CONFIG_CPC_VR_PI_ADDITIONAL_PARAMETERS      = 2;
const unsigned UN_CONFIG_CPC_VR_PI_ADDITIONAL_MASTER_NAME     = 3;
const unsigned UN_CONFIG_CPC_VR_PI_ADDITIONAL_PARENTS         = 4;
const unsigned UN_CONFIG_CPC_VR_PI_ADDITIONAL_CHILDREN        = 5;
const unsigned UN_CONFIG_CPC_VR_PI_ADDITIONAL_TYPE            = 6;
const unsigned UN_CONFIG_CPC_VR_PI_ADDITIONAL_SECOND_ALIAS    = 7;

//end_TAG_SCRIPT_DEVICE_TYPE_constants

/**
DPE configuration
TODO: use fw info instead
*/
mapping CPC_VR_PIConfig_getConfig() {
    mapping config;
    mapping props;

//begin_TAG_SCRIPT_DEVICE_TYPE_deviceConfig
//
mappingClear(props);
props["address"]        = ".ProcessInput.StsReg01";
props["dataType"]       = CPC_UINT16;
config["StsReg01"]      = props;

//
mappingClear(props);
props["hasArchive"]     = true;
props["address"]        = ".ProcessInput.evStsReg01";
props["dataType"]       = CPC_INT32;
config["evStsReg01"]    = props;

// 
mappingClear(props);
props["hasArchive"]     = true;
props["hasMapping"]     = true;
props["address"]        = ".ProcessInput.ObjectSt";
props["dataType"]       = CPC_UINT16;
config["ObjectSt"]      = props;

//
mappingClear(props);
props["hasArchive"]     = true;
props["hasMapping"]     = true;
props["address"]        = ".ProcessInput.ErrCoSt";
props["dataType"]       = CPC_UINT16;
config["ErrCoSt"]        = props;

// Relay 1 Status
mappingClear(props);
props["hasArchive"]     = true;
props["hasMapping"]     = true;
props["address"]        = ".ProcessInput.Rel1St";
props["dataType"]       = CPC_UINT16;
config["Rel1St"]        = props;

// Relay 2 Status
mappingClear(props);
props["hasArchive"]     = true;
props["hasMapping"]     = true;
props["address"]        = ".ProcessInput.Rel2St";
props["dataType"]       = CPC_UINT16;
config["Rel2St"]        = props;

// Relay 3 Status
mappingClear(props);
props["hasArchive"]     = true;
props["hasMapping"]     = true;
props["address"]        = ".ProcessInput.Rel3St";
props["dataType"]       = CPC_UINT16;
config["Rel3St"]        = props;

// Relay 4 Status
mappingClear(props);
props["hasArchive"]     = true;
props["hasMapping"]     = true;
props["address"]        = ".ProcessInput.Rel4St";
props["dataType"]       = CPC_UINT16;
config["Rel4St"]        = props;

// Relay filter Status
mappingClear(props);
props["hasArchive"]     = true;
props["hasMapping"]     = true;
props["address"]        = ".ProcessInput.RelFilterSt";
props["dataType"]       = CPC_UINT16;
config["RelFilterSt"]        = props;

// Relay 1 source status
mappingClear(props);
props["hasArchive"]     = true;
props["hasMapping"]     = true;
props["address"]        = ".ProcessInput.Rel1SrcSt";
props["dataType"]       = CPC_UINT16;
config["Rel1SrcSt"]        = props;

// Relay 2 source status
mappingClear(props);
props["hasArchive"]     = true;
props["hasMapping"]     = true;
props["address"]        = ".ProcessInput.Rel2SrcSt";
props["dataType"]       = CPC_UINT16;
config["Rel2SrcSt"]        = props;

// Relay 3 source status
mappingClear(props);
props["hasArchive"]     = true;
props["hasMapping"]     = true;
props["address"]        = ".ProcessInput.Rel3SrcSt";
props["dataType"]       = CPC_UINT16;
config["Rel3SrcSt"]        = props;

// Relay 4 source status
mappingClear(props);
props["hasArchive"]     = true;
props["hasMapping"]     = true;
props["address"]        = ".ProcessInput.Rel4SrcSt";
props["dataType"]       = CPC_UINT16;
config["Rel4SrcSt"]        = props;

// Relay 1 threshold status
mappingClear(props);
props["hasArchive"]     = true;
props["hasFormat"]      = true;
props["hasUnit"]        = true;
//props["hasPvRange"]     = true; // Done manually in custom config function
props["address"]        = ".ProcessInput.Rel1ThreSt";
props["dataType"]       = CPC_FLOAT;
config["Rel1ThreSt"]        = props;

// Relay 2 threshold status
mappingClear(props);
props["hasArchive"]     = true;
props["hasFormat"]      = true;
props["hasUnit"]        = true;
//props["hasPvRange"]     = true; // Done manually in custom config function
props["address"]        = ".ProcessInput.Rel2ThreSt";
props["dataType"]       = CPC_FLOAT;
config["Rel2ThreSt"]        = props;

// Relay 3 threshold status
mappingClear(props);
props["hasArchive"]     = true;
props["hasFormat"]      = true;
props["hasUnit"]        = true;
//props["hasPvRange"]     = true; // Done manually in custom config function
props["address"]        = ".ProcessInput.Rel3ThreSt";
props["dataType"]       = CPC_FLOAT;
config["Rel3ThreSt"]        = props;

// Relay 4 threshold status
mappingClear(props);
props["hasArchive"]     = true;
props["hasFormat"]      = true;
props["hasUnit"]        = true;
//props["hasPvRange"]     = true; // Done manually in custom config function
props["address"]        = ".ProcessInput.Rel4ThreSt";
props["dataType"]       = CPC_FLOAT;
config["Rel4ThreSt"]        = props;

// Relay 1 hysteris status
mappingClear(props);
props["hasArchive"]     = true;
//props["hasPvRange"]     = true; // Done manually in custom config function
props["address"]        = ".ProcessInput.Rel1HystSt";
props["dataType"]       = CPC_UINT16;
config["Rel1HystSt"]        = props;

// Relay 2 hysteris status
mappingClear(props);
props["hasArchive"]     = true;
//props["hasPvRange"]     = true; // Done manually in custom config function
props["address"]        = ".ProcessInput.Rel2HystSt";
props["dataType"]       = CPC_UINT16;
config["Rel2HystSt"]        = props;

// Relay 3 hysteris status
mappingClear(props);
props["hasArchive"]     = true;
//props["hasPvRange"]     = true; // Done manually in custom config function
props["address"]         = ".ProcessInput.Rel3HystSt";
props["dataType"]       = CPC_UINT16;
config["Rel3HystSt"]        = props;

// Relay 4 hysteris status
mappingClear(props);
props["hasArchive"]     = true;
//props["hasPvRange"]     = true; // Done manually in custom config function
props["address"]         = ".ProcessInput.Rel4HystSt";
props["dataType"]       = CPC_UINT16;
config["Rel4HystSt"]    = props;

// Parameter get code 
mappingClear(props);
props["hasArchive"]     = false;
props["address"]        = ".ProcessInput.ParGetCoSt";
props["dataType"]       = CPC_UINT16;
config["ParGetCoSt"]    = props;

// Parameter get value 
mappingClear(props);
props["hasArchive"]     = false;
props["address"]        = ".ProcessInput.ParGetValSt";
props["dataType"]       = CPC_FLOAT;
config["ParGetValSt"]    = props;

// Outputs
mappingClear(props);
props["address"]        = ".ProcessOutput.ManReg01";
props["dataType"]       = CPC_UINT16;
config["ManReg01"]      = props;

//
mappingClear(props);
props["address"]        = ".ProcessOutput.ManReg02";
props["dataType"]       = CPC_UINT16;
config["ManReg02"]      = props;

//
mappingClear(props);
props["address"]        = ".ProcessOutput.ParSetCoR";
props["dataType"]       = CPC_UINT16;
config["ParSetCoR"]      = props;

//
mappingClear(props);
props["address"]        = ".ProcessOutput.ParValR";
props["dataType"]       = CPC_FLOAT;
config["ParValR"]     = props;


//end_TAG_SCRIPT_DEVICE_TYPE_deviceConfig

    return config;
}

/**
Returns the list of parameter names that passed via config line.
*/
dyn_string CPC_VR_PIConfig_getParamNames() {
//begin_TAG_SCRIPT_DEVICE_TYPE_getParamNames
    return makeDynString(
        CPC_PARAMS_PUMP1_NAME, CPC_PARAMS_PUMP2_NAME, CPC_PARAMS_PUMP3_NAME, CPC_PARAMS_PUMP4_NAME,
        CPC_PARAMS_RELAY1_NAME, CPC_PARAMS_RELAY2_NAME, CPC_PARAMS_RELAY3_NAME, CPC_PARAMS_RELAY4_NAME
    );
//end_TAG_SCRIPT_DEVICE_TYPE_getParamNames
}

/** Check custom configuration
*/
CPC_VR_PIConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
    
    int iValue;
    int fValue;
    
    unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_RELFILTERST_DEFAULT], iValue, exceptionInfo);
    unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL1SRC_DEFAULT], iValue, exceptionInfo);
    unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL2SRC_DEFAULT], iValue, exceptionInfo);    
    unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL3SRC_DEFAULT], iValue, exceptionInfo);
    unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL4SRC_DEFAULT], iValue, exceptionInfo);

    unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL1HYST_DEFAULT], iValue, exceptionInfo);
    unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL2HYST_DEFAULT], iValue, exceptionInfo);
    unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL3HYST_DEFAULT], iValue, exceptionInfo);
    unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL4HYST_DEFAULT], iValue, exceptionInfo);

    
    unConfigGenericFunctions_checkFloat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL1THRES_DEFAULT], fValue, exceptionInfo);
    unConfigGenericFunctions_checkFloat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL2THRES_DEFAULT], fValue, exceptionInfo);
    unConfigGenericFunctions_checkFloat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL3THRES_DEFAULT], fValue, exceptionInfo);
    unConfigGenericFunctions_checkFloat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL4THRES_DEFAULT], fValue, exceptionInfo);

//end_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
}

/** Set custom configuration
*/
CPC_VR_PIConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig
    
    // Default thresholds 
    float fThresMin = 1e-12;
    float fThresMax = 1e-4;

    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRelFilter", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_RELFILTERST_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel1Src", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL1SRC_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel2Src", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL2SRC_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel3Src", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL3SRC_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel4Src", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL4SRC_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel1Thre", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL1THRES_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel2Thre", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL2THRES_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel3Thre", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL3THRES_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel4Thre", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL4THRES_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel1Hyst", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL1HYST_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel2Hyst", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL2HYST_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel3Hyst", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL3HYST_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel4Hyst", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL4HYST_DEFAULT]);

    unConfigGenericFunctions_setPvRangeDPE(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.Rel1ThreSt", fThresMin, fThresMax, exceptionInfo);
    unConfigGenericFunctions_setPvRangeDPE(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.Rel2ThreSt", fThresMin, fThresMax, exceptionInfo);
    unConfigGenericFunctions_setPvRangeDPE(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.Rel3ThreSt", fThresMin, fThresMax, exceptionInfo);
    unConfigGenericFunctions_setPvRangeDPE(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.Rel4ThreSt", fThresMin, fThresMax, exceptionInfo);
    unConfigGenericFunctions_setPvRangeDPE(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.Rel1HystSt", 0, 100, exceptionInfo);
    unConfigGenericFunctions_setPvRangeDPE(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.Rel2HystSt", 0, 100, exceptionInfo);
    unConfigGenericFunctions_setPvRangeDPE(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.Rel3HystSt", 0, 100, exceptionInfo);
    unConfigGenericFunctions_setPvRangeDPE(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.Rel4HystSt", 0, 100, exceptionInfo);

//end_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig
}

/**
Purpose: Export CPC_VR_PI Devices

Usage: External function

PVSS manager usage: NG, NV
*/
CPC_VR_PIConfig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    string sObject = UN_CONFIG_CPC_VR_PI_DPT_NAME;
    int amount = dynlen(dsDpList);
    for (int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if (!dpExists(currentDP)) { continue; }

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
        //begin_TAG_SCRIPT_DEVICE_TYPE_export

        
        cpcExportGenericFunctions_getDeadband(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.Rel1St");
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters, ".ProcessInput.Rel1St");
        
        // Process outputs (20)
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01",        true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01",      true, dsDpParameters);        
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel1St", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel2St", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel3St", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel4St", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "RelFilterSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel1SrcSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel2SrcSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel3SrcSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel4SrcSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel1ThreSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel2ThreSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel3ThreSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel4ThreSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel1HystSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel2HystSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel3HystSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "Rel4HystSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ObjectSt", true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ErrCoSt", true, dsDpParameters);
    
        // Process inputs (4)
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", false, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg02", false, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ParSetCoR", false, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ParValR", false, dsDpParameters);
        
        // Defaults (12)
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "RelFilterSt"); 
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "Rel1SrcSt");
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "Rel2SrcSt");
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "Rel3SrcSt");
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "Rel4SrcSt");
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "Rel1ThreSt");
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "Rel2ThreSt");
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "Rel3ThreSt");
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "Rel4ThreSt");
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "Rel1HystSt"); 
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "Rel2HystSt"); 
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "Rel3HystSt"); 
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "Rel4HystSt"); 

        // Mappings
        dynAppend(dsDpParameters, cpcGenericDpFunctions_getMapping(currentDP, ".ProcessInput.ObjectSt"));
        dynAppend(dsDpParameters, cpcGenericDpFunctions_getMapping(currentDP, ".ProcessInput.ErrCoSt"));
        dynAppend(dsDpParameters, cpcGenericDpFunctions_getMapping(currentDP, ".ProcessInput.Rel1St"));
                
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString(               
            "ProcessInput.Rel1St",
            "ProcessInput.Rel2St",
            "ProcessInput.Rel3St",
            "ProcessInput.Rel4St",
            "ProcessInput.RelFilterSt",
            "ProcessInput.Rel1SrcSt",
            "ProcessInput.Rel2SrcSt",
            "ProcessInput.Rel3SrcSt",
            "ProcessInput.Rel4SrcSt",
            "ProcessInput.Rel1ThreSt",
            "ProcessInput.Rel2ThreSt",
            "ProcessInput.Rel3ThreSt",
            "ProcessInput.Rel4ThreSt",
            "ProcessInput.Rel1HystSt",
            "ProcessInput.Rel2HystSt",
            "ProcessInput.Rel3HystSt",
            "ProcessInput.Rel4HystSt",
            "ProcessInput.ObjectSt",
            "ProcessInput.ErrCoSt"
        ), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01"), dsDpParameters);

        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_export

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

/**
Purpose: Export CPC_VR_PI Devices for S7_PLC front-end

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_CPC_VR_PI_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VR_PIConfig_ExportConfig(dsDpList, exceptionInfo);
}

/**
Purpose: Export CPC_VR_PI Devices for _UnPlc front-end

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_CPC_VR_PI_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VR_PIConfig_ExportConfig(dsDpList, exceptionInfo);
}

OPCUA_CPC_VR_PI_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VR_PIConfig_ExportConfig(dsDpList, exceptionInfo);
}

BACnet_CPC_VR_PI_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VR_PIConfig_ExportConfig(dsDpList, exceptionInfo);
}

//@}
