/**@file

// cpcVV_S.ctl
This library contains the widget, faceplate, etc. functions of VV_S.

@par Creation Date
  10/10/2016

@par Modification History

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author
  T. van Winden (TE-VSC)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{

#uses "vacuumExtension/classes/DeviceListCriteria.ctl"

/** Returns the list of VV_S DPE with alarm config that can be acknowledged

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name DP name
@param dpType input, device type
@param dsNeedAck output, the lsit of DPE
*/
CPC_VV_S_AcknowledgeAlarm(string deviceName, string dpType, dyn_string &dsNeedAck) {
//begin_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
    dsNeedAck = makeDynString(
        "ProcessInput.PrItlSt",
        "ProcessInput.PrSrcItlSt",
        "ProcessInput.ClEnSt",
        "ProcessInput.PrevItlSt",
        "ProcessInput.AftItlSt",
        "ProcessInput.StartISt", 
        "ProcessInput.TStopISt", 
        "ProcessInput.FuStopISt", 
        "ProcessInput.AlSt", 
        UN_ACKNOWLEDGE_PLC
    );
//end_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
}

/** Function called from snapshot utility of the treeDeviceOverview to get the time and value

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name
@param deviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
CPC_VV_S_ObjectListGetValueTime(string deviceName, string deviceType, dyn_string &dsReturnData) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
    float value;
    string sTime;

    dpGet(deviceName + ".ProcessInput.ObjectSt", value,
          deviceName + ".ProcessInput.ObjectSt:_online.._stime", sTime);

    dsReturnData[1] = sTime;
    dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(deviceName + ".ProcessInput.ObjectSt"), value) + " " + dpGetUnit(deviceName + ".ProcessInput.ObjectSt"); // formated value with unit
//end_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
}





// POPUP MENU
//--------------------------------

/** pop-up menu

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param dsAccessOk input, the access control
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
CPC_VV_S_MenuConfiguration(string deviceName, string dpType, dyn_string dsAccessOk, dyn_string &menuList) {
//begin_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
    dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT, UN_POPUPMENU_TREND_TEXT, CPC_POPUPMENU_ALLOW_TO_RESTART_TEXT);

    // Add device context menu items
    unVacObject_addAliasActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    
    unVacObject_addDisableEnableActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    int iPos = unVacObject_addVacuumExtensionActionToMenu(deviceName, dpType,  dsAccessOk, menuList);
    iPos = unVacObject_addProcessActionToMenu(deviceName, dpType,  dsMenuConfig, dsAccessOk, menuList, iPos+1);

//end_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
}

/** handle the answer of the popup menu

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param menuList input, the access control
@param menuAnswer input, selected menu value
*/
CPC_VV_S_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer) {
    dyn_string exceptionInfo;
    //begin_TAG_SCRIPT_DEVICE_TYPE_HandleMenu
    
    // The help file
    string helpFile = unVacObject_getHelpLink(dpType);
    
    if(!unVacObject_handleCustomUnicosMenu(deviceName, dpType, menuList, menuAnswer, helpFile, exceptionInfo)){
      cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
    }
    
    if (dynlen(exceptionInfo) > 0) {
        unSendMessage_toExpertException("Right Click", exceptionInfo);
    }
    //end_TAG_SCRIPT_DEVICE_TYPE_HandleMenu
}

/** Init static values which are used in widget animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_VV_S_WidgetInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
  dyn_string exceptionInfo;
  g_params[CPC_PROP_NAME_MACHINE_TYPE] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_MACHINE_TYPE, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
}

/** 
Returns the list of VV_S DPEs which should be connected on widget animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_VV_S_WidgetDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
    dynAppend(dpes, deviceName + ".eventMask");                                // 6
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");                    // 7
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");  // 8
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");                    // 9
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");  // 10
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg03");                    // 11
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg03:_online.._invalid");  // 12
	  dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");                    // 13
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid");  // 14
    dynAppend(dpes, deviceName + ".disabled");                                 // 15
    
    if (g_bBoolAlert) {
        dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");  // 16
        dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");  // 17
        dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state"); // 18
        dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");      // 19
    }
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
}

/** callback function on the device data

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

Datapoint elements and related values
  1  int	    frontend.alarm
  2  bool 		frontend.enabled
  3  bool    	device.statusInformation.selectedManager:_lock._original._locked,
  4  string  	device.statusInformation.selectedManager,
  5  bool    	device.statusInformation.deviceLogActive
  6  int 		  device.eventMask
  7  bit32		device.ProcessInput.StsReg01
  8  bool 		device.ProcessInput.StsReg01:_online.._invalid
  9  bit32		device.ProcessInput.StsReg02
  10 bool		  device.ProcessInput.StsReg02:_online.._invalid
  11 bit32		device.ProcessInput.StsReg03
  12 bool		  device.ProcessInput.StsReg03:_online.._invalid
  13 int		  device.ProcessInput.ObjectSt
  14 int		  device.ProcessInput.ObjectSt:_online.._invalid 
  15 bool		  device.disabled
  16 int		  device.ProcessInput.StartISt:_alert_hdl.._act_state
  17 int		  device.ProcessInput.TStopISt:_alert_hdl.._act_state
  18 int		  device.ProcessInput.FuStopISt:_alert_hdl.._act_state
  19 int		  device.ProcessInput.AlSt:_alert_hdl.._act_state	
  
  @see For the stsReg bit meaning check the specifications file: https://svn.cern.ch/reps/en-ice-svn/trunk/applications/vacuum/UNICOS_VAC_DEVICES/Design/Object_ManReg_StsReg_Mapping.xls
  @see Other usufull documentation: https://edms.cern.ch/ui/file/1228441/1.6.0/UCPC6_RP1.6.0_Manual.pdf
*/
CPC_VV_S_WidgetAnimation(dyn_string dpes, dyn_anytype values, string sWidgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation

  // Do nothing if system is not connected
	if (!g_bSystemConnected){
		return;
	}

// INIT VARIABLES
//----------------
  int iSystemIntegrityAlarmValue = values[1];
  bool bSystemIntegrityAlarmEnabled = values[2];
  bool bLocked = values[3];
  string sSelectedManager = values[4];
  bool bComment = values[5];
  int iMask = values[6];
  bit32 bit32StsReg01 = values[7];
  bit32 bit32StsReg02 = values[9];
  bit32 bit32StsReg03 = values[11];
  int iObjectSt = values[13];
  bool bDisabled = values[15];

  // Determine validness based on combination of all DPES 
  bool bInvalid = values[8] || values[10] || values[12] || values[14];  
  
  // Determine widget type
  bool bCompact = sWidgetType == "VV_S";
  
  // Determine machine type    
  bool isLHC = g_params[CPC_PROP_NAME_MACHINE_TYPE] == CPC_VAC_MACHINE_TYPE_LHC;
  bool isSPS = g_params[CPC_PROP_NAME_MACHINE_TYPE] == CPC_VAC_MACHINE_TYPE_SPS;
  bool isCPS = g_params[CPC_PROP_NAME_MACHINE_TYPE] == CPC_VAC_MACHINE_TYPE_CPS;
  
  // Determine alarm letter booleans
  bool bLclItl = (isLHC) ? getBit(bit32StsReg03, CPC_StsReg03_LCL1ITLST) == 1 : getBit(bit32StsReg03, CPC_StsReg03_LCL1ITLST) == 1 || getBit(bit32StsReg03, CPC_StsReg03_LCL2ITLST) == 1;
  bool bPropItl = (isLHC || isSPS) ? getBit(bit32StsReg03, CPC_StsReg03_GLBBEFOREITLST) == 1 || getBit(bit32StsReg03, CPC_StsReg03_GLBAFTERITLST) == 1 : false;
  bool bOpenEnable = (isLHC) ? getBit(bit32StsReg03, CPC_StsReg03_OPEN1ST) == 1 && getBit(bit32StsReg03, CPC_StsReg03_TEMPITLST) == 0: getBit(bit32StsReg03, CPC_StsReg03_OPEN1ST) == 1 && getBit(bit32StsReg03, CPC_StsReg03_OPEN2ST) == 1;  
  
  // Determine valve state
  bool bOn = iObjectSt == 0 || iObjectSt == 1;
  bool bHalf = iObjectSt == 4;
  bool bIsRestartNeeded = g_bMRestart && getBit(bit32StsReg02, CPC_StsReg02_NEEDRESTART) == 0;
   
  // Bits set
  dyn_string dsWarningBits = (bCompact) ? VAC_WARNING_BITS : CPC_WARNING_BITS;
  dyn_string dsAlarmBits = (bCompact) ? VAC_ALARM_BITS : CPC_ALARM_BITS;
  dyn_string dsControlBits = (bCompact) ? VAC_CONTROL_BITS : CPC_CONTROL_BITS; 
   
  // Letters set
  dyn_string dsAlarmLetters = CPC_ALARM_LETTERS;
  dyn_string dsControlLetters = CPC_CONTROL_LETTERS;
  
  // Add LHC control/alarm letter  
  if(isLHC && bOn){
      dynAppend(dsControlLetters, CPC_WIDGET_TEXT_CONTROL_CLOSE_ENABLE);
      dynAppend(dsControlBits, "CPC_StsReg03_CLENST");
  }
  if(isLHC){
    dynAppend(dsAlarmLetters, CPC_WIDGET_TEXT_ALARM_PREVENTIVE_CLOSE);
    dynAppend(dsAlarmBits, "CPC_StsReg03_PREVCLST");  
  }
  
  // Add local control letters
  dynAppend(dsControlBits,"CPC_StsReg01_LDST");
  dynAppend(dsControlLetters, CPC_WIDGET_TEXT_CONTROL_LOCAL);
      
  // Create sts reg
  dyn_bit32 dbit32StsRegs = makeDynBit32(bit32StsReg01, bit32StsReg02, bit32StsReg03);
  
  // Apply letter logic for valve
  dyn_bool dbAlarmBools = unVacObject_fetchBitsFromStsRegs(dbit32StsRegs, dsAlarmBits);
  dynInsertAt(dsAlarmLetters, CPC_WIDGET_TEXT_ALARM_LOCAL_INT, 1); // Add local interlock sign
  dynInsertAt(dbAlarmBools, bLclItl, 1);
  dynInsertAt(dsAlarmLetters, CPC_WIDGET_TEXT_ALARM_PROPAGATED_INT, 5); // Add propagated interlock
  dynInsertAt(dbAlarmBools, bPropItl, 5);
  dynInsertAt(dsAlarmLetters, CPC_WIDGET_TEXT_ALARM_OPEN_DISABLE, 6); // Add open disable
  dynInsertAt(dbAlarmBools, !bOpenEnable, 6); 
     
   // Generic vacuum animation
	unVacObject_WidgetLetterAnimation(sWidgetType, UN_CONFIG_CPC_VV_S_DPT_NAME,
			bLocked, sSelectedManager, dbit32StsRegs,
      unVacObject_fetchBitsFromStsRegs(dbit32StsRegs, dsWarningBits),    // Default vacuum warning bits
      CPC_WARNING_LETTERS, // Default vacuum warning letters
      dbAlarmBools,
      dsAlarmLetters, // Default vacuum alarm letters
      unVacObject_fetchBitsFromStsRegs(dbit32StsRegs, dsControlBits) ,    
      dsControlLetters, // Default vacuum control letters
			bInvalid, false, bComment,
			iSystemIntegrityAlarmValue, bSystemIntegrityAlarmEnabled);

  // Determine the body color based on the state
  dyn_int diKeyFilter = makeDynInt(3, 4, 5);
  dyn_string filteredDpes = unVacObject_dynWithoutIndex(dpes, diKeyFilter);
  dyn_anytype filteredValues = unVacObject_dynWithoutIndex(values, diKeyFilter);
  // Filter dpes based on index to be consistent with the StateHistory dpe's
  string sBodyColor, sState;
  CPC_VV_S_ColorAndString("", filteredDpes, filteredValues, sBodyColor, sState, false, g_bBoolAlert, bIsRestartNeeded);  
  
  bool bFillValve = (!bOpenEnable || bLclItl); // Fill the valve when the local interlock is present
  CPC_VV_S_WidgetBodyAnimation(bOn, bHalf, sBodyColor, bFillValve); // Animate the object
						  
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
}
  
/**
@author Thijs van Winden

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI
@param bOn Determine if the device is on
@param bUndefined Undefined state of the device
@param sBodyColor The body color of the device
*/
CPC_VV_S_WidgetBodyAnimation(bool bOn, bool bHalf, string sBodyColor, bool bFillValve = false){
  setMultiValue(
  		"BodyLineExt1", "foreCol", sBodyColor,
  		"BodyLineExt2", "foreCol", sBodyColor,
      "BodyLineExt3", "foreCol", sBodyColor,
      "BodyLineExt4", "foreCol", sBodyColor,
      "BodyLineExt1", "backCol", (bOn && !bHalf) ? sBodyColor : "unWidget_Background",
  		"BodyLineExt2", "backCol", (bOn) ? sBodyColor : "unWidget_Background",
      "BodyLineExt4", "backCol", bFillValve ? sBodyColor: "unWidget_Background"
  );
}



/** Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.

*/
CPC_VV_S_WidgetDisconnection(string sWidgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
    CPC_VV_S_WidgetBodyAnimation(false, false, "unDataNoAccess");
	  setMultiValue(  		
			"WarningText", "text", "", "AlarmText", "text", "", "ControlStateText", "text", "",
      "SelectArea", "foreCol", "", "LockBmp", "visible", false, "WidgetArea", "visible", false,
      "eventState", "visible", false
   );
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
}




// BUTTONS IN FACEPLATE
//--------------------------------

/** Return button configuration including access level

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
mapping CPC_VV_S_ButtonConfig(string deviceName) {
    mapping buttons;
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig

    buttons[CPC_FACEPLATE_BUTTON_ALLOW_RESTART] = UN_ACCESS_RIGHTS_OPERATOR;
    unVacObject_genericButtonConfig(deviceName, buttons);
    
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
    return buttons;
}

/** Configure the list of dpes that needs for buttons animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the name of device
@param dpes input, the list of dpes to connect
*/
CPC_VV_S_ButtonDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");    
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg03");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg03:_online.._invalid");
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
}

/** Set the state of the contextual button of the device

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param dpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
CPC_VV_S_ButtonSetState(string deviceName, string dpType, dyn_string dsUserAccess, dyn_string dsData) {
    bool bSelected, buttonEnabled, bit1, bit2, bit3;
    string localManager;
    dyn_string exceptionInfo;

    bool bLocked = dsData[1];
    string sSelectedUIManager = dsData[2];

    localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
    bSelected = (localManager == "S"); // Selection state
    dyn_string dsButtons = unSimpleAnimation_ButtonList(deviceName);

//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
    mixed stsReg01Value = dsData[3];
    bool bStsReg01Bad = dsData[4];

    for (int i = 1; i <= dynlen(dsButtons); i++) {
        buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0); // User access
        switch (dsButtons[i]) {
            case UN_FACEPLATE_BUTTON_FORCED_MODE:
                bit1 = !(getBit(stsReg01Value, CPC_StsReg01_FOMOST) == 0);
                bit2 = !(getBit(stsReg01Value, CPC_StsReg01_AUINHFMO) == 0);
                buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2) && !bStsReg01Bad;
                break;
            case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
            //case UN_FACEPLATE_BUTTON_ONFORCED_REQUEST: // 
            case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
            case UN_FACEPLATE_BUTTON_AUTO_MODE:
            case UN_FACEPLATE_BUTTON_SET_VALUE:
                bit1 = !(getBit(stsReg01Value, CPC_StsReg01_AUMOST) == 0);
                buttonEnabled = buttonEnabled && bSelected && !(bit1) && !bStsReg01Bad;
                break;
            case UN_FACEPLATE_BUTTON_MANUAL_MODE:
                buttonEnabled = buttonEnabled && bSelected &&
                                !getBit(stsReg01Value, CPC_StsReg01_MMOST) &&
                                !getBit(stsReg01Value, CPC_StsReg01_AUINHMMO) &&
                                !bStsReg01Bad;
                break;                
            case UN_FACEPLATE_BUTTON_OPERATION_MODES:
                buttonEnabled = buttonEnabled && bSelected && 
                                getBit(stsReg01Value, CPC_StsReg01_MMOST) && 
                                !bStsReg01Bad;                
                break;
			case UN_FACEPLATE_BUTTON_RESETERRWARN_REQUEST:
				buttonEnabled = buttonEnabled && bSelected &&
                                !bStsReg01Bad;
            default:
                break;
        }
        if (dsButtons[i] == UN_FACEPLATE_BUTTON_SELECT) {
            unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
        } else {
            cpcButton_setButtonState(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], buttonEnabled);
        }
    }
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
}






// FACEPLATE
//--------------------------------



/** Init static values which are used in faceplate animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_VV_S_FaceplateInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
    int iAlertType1, iAlertType2, iAlertType3, iAlertType4, iRangeType;
    dyn_string exceptionInfo;
    int iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
                     deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
                     deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
                     deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
    g_params["BoolAlert"] = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));
    
    g_params["MRestart"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);
	
	  g_params["ControllerModel"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_CONTROLLER_MODEL, exceptionInfo);
	  setValue("ControllerModel", "text", g_params["ControllerModel"]);
		
	  mapping objectStLabels;
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.ObjectSt"), objectStLabels);
    g_params["ObjectStLabels"] = objectStLabels;
	
    // Load the machine type
    g_params[CPC_PROP_NAME_MACHINE_TYPE] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_MACHINE_TYPE, exceptionInfo);
    // Hide elements based on machine type
    dyn_string dsHiddenEl;
    switch(g_params[CPC_PROP_NAME_MACHINE_TYPE]){
      case CPC_VAC_MACHINE_TYPE_LHC:
        dsHiddenEl = makeDynString("LCL2ITLSTLABEL", "LCL2ITLST", "LCL1ITLSTLABEL", "OPEN2STLABEL", "OPEN2ST", "OPEN1STLABEL");
      break;
      case CPC_VAC_MACHINE_TYPE_SPS:
        dsHiddenEl = makeDynString("TEMPITLSTLABEL", "TEMPITLST", "PREVCLSTLABEL", "PREVCLST", "CLENSTLABEL", "CLENST", "BDRSTLABEL", "BDRST");
      break;
      case CPC_VAC_MACHINE_TYPE_CPS:
        dsHiddenEl = makeDynString("TEMPITLSTLABEL", "TEMPITLST", "PREVCLSTLABEL", "PREVCLST", "CLENSTLABEL", "CLENST", "BDRSTLABEL", "BDRST",
                                 "OPENSTLABEL", "OPEN1STLABEL", "OPEN1ST", "OPEN2STLABEL", "OPEN2ST", "GLBITLSTLABEL", "GLBBEFOREITLSTLABEL", 
                                 "GLBBEFOREITLST", "GLBAFTERITLSTLABEL", "GLBAFTERITLST", "VVSCONSTLABEL", "VVSCONST"
                                 );      
      break;
    }
    // Hide the elements    
    for(int i = 1; i<=dynlen(dsHiddenEl); i++){
      setValue(dsHiddenEl[i], "visible", false);
    }    
    
    //end_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
    
}



/** Returns the list of VV_S DPEs which should be connected on faceplate animation
// dyn_string dpes value[1]: event Mask
// dyn_string dpes value[2]: StsReg01
// dyn_string dpes value[1]: StsReg01:_online.._invalid
// dyn_string dpes value[1]: event Mask
// dyn_string dpes value[1]: event Mask
// dyn_string dpes value[1]: event Mask

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_VV_S_FaceplateDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
    dynAppend(dpes, deviceName + ".eventMask");  										
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");    
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg03");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg03:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid");
    if (g_params["BoolAlert"]) {
        dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.PrItlSt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.PrSrcItlSt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.ClEnSt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.PrevItlSt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.AftItlSt:_alert_hdl.._act_state");
    }
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
}


/** callback function on the device data

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
CPC_VV_S_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
    // Animates the on/off status and objectSt panel
    unVacObject_animateFaceplateDeviceStatus(dpes, values);
    // Animates operation modes, like auto, manual, forced and their inhibit modes
    unVacObject_animateFaceplateOperationModes(dpes, values);
    
    // Shared animations for machine types
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg03_REMCTRENST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_LDST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_OUTONOVST", true, "cpcColor_Faceplate_Status");
      
    // Animate LHC status bits, show disabled box when not supported    
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg03_CLENST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg03_BDRST", true, "cpcColor_Faceplate_Status");  
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg03_PRITLENST", true, "cpcColor_Faceplate_Status"); 
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg03_PREVCLST", true, "cpcColor_Faceplate_Status");
    // Animate LHC+SPS status bits, show disabled box when not supported  
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg03_VVSCONST", true, "cpcColor_Faceplate_Status");
    
    // Animate LHC interlocks    
    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg03_LCL1ITLST", "LCL1ITLST", "CPC_StsReg01_ALUNACK"); 
    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg03_LCL2ITLST", "LCL2ITLST", "CPC_StsReg01_ALUNACK"); 
    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg03_GLBBEFOREITLST", "GLBBEFOREITLST", "CPC_StsReg01_ALUNACK");
    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg03_GLBAFTERITLST", "GLBAFTERITLST", "CPC_StsReg01_ALUNACK");
    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg03_TEMPITLST", "TEMPITLST", "CPC_StsReg01_ALUNACK");   
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg03_OPEN1ST",  true,  "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg03_OPEN2ST",  true,  "cpcColor_Faceplate_Status");            
    
    // Animates auto on/off/manual/force mode request status
    unVacObject_animateFaceplateRequestStatus(dpes, values);

    // Animates the start, stop, full stop, alarm and acknowledge interlocks
    unVacObject_animateFaceplateInterlocks(dpes, values);

    cpcGenericObject_animateMaskEvent(dpes, values);
    cpcGenericObject_animateNeedRestart(dpes, values);
    unVacObject_animateFaceplateWarnings(dpes, values);
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
}


/**
  Lists the dpes required for building the state of the device
  
  @param string sDeviceName The name of the device which is in UNICOS the same as the dpname
  @param dyn_string dsDpes The list of datapoints elements required for building the state
  @param bool bForHistory The DPEs will be used for state history, i.e. only DPEs with _archive
                            config are of interest
  
*/
CPC_VV_S_StateDPEs(string sDeviceName, dyn_string &dsDpes, bool bForHistory = true) {
    dynAppend(dsDpes, sDeviceName + ".eventMask");                                  // 3
    if(bForHistory) {
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg01");                    // 4
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg01:_online.._invalid");  // 5
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg02");                    // 6
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg02:_online.._invalid");  // 7
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg03");                    // 8
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg03:_online.._invalid");  // 9
    }
    else {
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg01");                    // 4
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg01:_online.._invalid");  // 5
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg02");                    // 6
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg02:_online.._invalid");  // 7
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg03");                    // 8
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg03:_online.._invalid");  // 9
    }
	  dynAppend(dsDpes, sDeviceName + ".ProcessInput.ObjectSt");                      // 10
    dynAppend(dsDpes, sDeviceName + ".ProcessInput.ObjectSt:_online.._invalid");    // 11
    dynAppend(dsDpes, sDeviceName + ".disabled");                                   // 12
    
    if (g_bBoolAlert) {
        dynAppend(dsDpes, sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");  // 13
        dynAppend(dsDpes, sDeviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");  // 14
        dynAppend(dsDpes, sDeviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state"); // 15
        dynAppend(dsDpes, sDeviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");      // 16
    }
}

/**
  Returns the main color associated with the d   evice state. This color is based on 
  the dpes listed in the CPC_VV_S_StateDPEs for the dedicated device type. In case of the 
  state history the EvStsReg## is used because this is archived, while for determining 
  the current state the StsReg## is used.

  @param dyn_string dsDpes The list of dpe names used to determine the color
  @param dyn_anytype daValues The list of dpe values used to determine the color
  
  Datapoint elements and related values
  1  int	    frontend.alarm
  2  bool 		frontend.enabled
  3  bool     device.eventMask
  4  bit32		device.ProcessInput.(Ev)StsReg01
  5  bool 		device.ProcessInput.(Ev)StsReg01:_online.._invalid
  6  bit32		device.ProcessInput.(Ev)StsReg02
  7  bool		  device.ProcessInput.(Ev)StsReg02:_online.._invalid
  8  bit32		device.ProcessInput.(Ev)StsReg03
  9  bool		  device.ProcessInput.(Ev)StsReg03:_online.._invalid
  10  int		  device.ProcessInput.ObjectSt
  11 int		  device.ProcessInput.ObjectSt:_online.._invalid 
  12 bool		  device.disabled
  13 int		  device.ProcessInput.StartISt:_alert_hdl.._act_state
  14 int		  device.ProcessInput.TStopISt:_alert_hdl.._act_state
  15 int		  device.ProcessInput.FuStopISt:_alert_hdl.._act_state
  16 int		  device.ProcessInput.AlSt:_alert_hdl.._act_state	
  
  @param string sColor, output, the main color string of device
  @param string sString, output, the main state string of device
  @param bool bUseOffColor, input, Off color is return in &sColor when device is Off, false in case of WidgetColor
  @param bool bBoolAlert, input, realtime alert according to UNICOS alert manager

  @param bool bIsRestartNeeded, input, there is a restart needed according to UNICOS device manager
  
  @see For the stsReg bit meaning check the specifications file: https://svn.cern.ch/reps/en-ice-svn/trunk/applications/vacuum/UNICOS_VAC_DEVICES/Design/Object_ManReg_StsReg_Mapping.xls
  @see Other usufull documentation: https://edms.cern.ch/ui/file/1228441/1.6.0/UCPC6_RP1.6.0_Manual.pdf  
  
  @return '0' when function proceed successfully, '-1' when function proceed with error
  
*/

int CPC_VV_S_ColorAndString(string sDeviceName, dyn_string dsDpes, dyn_anytype daValues, string &sColor,
                               string &sState, bool bUseOffColor, bool bBoolAlert = false,
                               bool bIsRestartNeeded = false){
  // INIT VARIABLES
  int iSystemIntegrityAlarmValue = daValues[1];
  bool bSystemIntegrityAlarmEnabled = daValues[2];
  bit32 bit32StsReg01 = daValues[4];
  bit32 bit32StsReg02 = daValues[6];
  bit32 bit32StsReg03 = daValues[8];
  int iObjectSt = daValues[10];
  bool bDisabled = daValues[12];
    
  int iStartIStState, iTStopIStState, iFuStopIStState, iAlSt;
  if (bBoolAlert) {
    iStartIStState = daValues[13];
    iTStopIStState = daValues[14];
    iFuStopIStState = daValues[15];
    iAlSt = daValues[15];
  }
  
  // Determine valid state
  bool bInvalid = daValues[5] || daValues[7] || daValues[9] || daValues[11];
    
  // --- Get main state string
  //--------------------------
  if (sDeviceName!="") {
     sState = unVacObject_getStateString(sDeviceName, iObjectSt);
  }
  else {
     sState = "Unknown";
  }

  dyn_string dsExceptionInfo;
  string sMachineType = CPC_VAC_MACHINE_TYPE_LHC; //cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PROP_NAME_MACHINE_TYPE, dsExceptionInfo);

  // --- Get main color string
  // --------------------------
  // Determine machine type    
  bool isLHC = sMachineType == CPC_VAC_MACHINE_TYPE_LHC;
  bool isSPS = sMachineType == CPC_VAC_MACHINE_TYPE_SPS;
  bool isCPS = sMachineType == CPC_VAC_MACHINE_TYPE_CPS;
  
  // Determine alarm letter booleans
  bool bLclItl = (isLHC) ? getBit(bit32StsReg03, CPC_StsReg03_LCL1ITLST) == 1 : getBit(bit32StsReg03, CPC_StsReg03_LCL1ITLST) == 1 || getBit(bit32StsReg03, CPC_StsReg03_LCL2ITLST) == 1;
  bool bPropItl = (isLHC || isSPS) ? getBit(bit32StsReg03, CPC_StsReg03_GLBBEFOREITLST) == 1 || getBit(bit32StsReg03, CPC_StsReg03_GLBAFTERITLST) == 1 : false;
  bool bOpenEnable = (isLHC) ? getBit(bit32StsReg03, CPC_StsReg03_OPEN1ST) == 1 && getBit(bit32StsReg03, CPC_StsReg03_TEMPITLST) == 0: getBit(bit32StsReg03, CPC_StsReg03_OPEN1ST) == 1 && getBit(bit32StsReg03, CPC_StsReg03_OPEN2ST) == 1;      
  
  // Determine alarm color booleans
  bool bAlarm = bLclItl || getBit(bit32StsReg01, CPC_StsReg01_TSTOPIST) || getBit(bit32StsReg02, CPC_StsReg02_FUSTOPIST) || getBit(bit32StsReg01, CPC_StsReg01_ALST);
  bool bAlarmUnack= getBit(bit32StsReg01, CPC_StsReg01_ALUNACK) == 1 &&
             (iTStopIStState == 1 || 
              iTStopIStState == 3 || 
              iFuStopIStState == 1 || 
              iFuStopIStState == 3);
  
  // Warning boolean if Opened with Error(ObjectSt == 0) or IO Error 
  bool bWarning = iObjectSt == 0 || bPropItl || getBit(bit32StsReg01, CPC_StsReg01_IOERRORW) == 1;

  // Mode boolean
  bool bForcedMode = getBit(bit32StsReg01, CPC_StsReg01_FoMoSt) == 1;  
  // Valve is open in Opened with Error (ObjectSt == 0) and Open without error(ObjectSt == 1)
  bool bOn = iObjectSt == 0 || iObjectSt == 1; 
  // Show half animation when valve is in Undefined state (ObjectSt == 4). This means neither open nor closed
  bool bHalf = iObjectSt == 4; 
  
  // Create boolean array for corresponding colors
  dyn_bool dbBodyStateBool = makeDynBool(
      bInvalid,      // Invalid color
      bAlarmUnack,   // Alarm Unacknowledged color
      bAlarm,    // Alarm active color
      bDisabled,     // Device disabled color
      bWarning,      // Warning color
      bForcedMode,   // Forced mode color
      bUseOffColor && bHalf, // Undefined color
      (!bOn) && bUseOffColor,// Off color (optional color)
      false,                 // Under Range color
      true                   // OK color
  );
    
  // Create boolen array for corresponding state by using modified version of boolean array   
  unVacObject_WidgetBoolMapping(dbBodyStateBool, CPC_BODY_COLORS, sColor);
  
  return 0;
}

int CPC_VV_S_ColorFillAndString(string sDeviceName, dyn_string dsDpes, dyn_anytype daValues, string &sColor, bool &bFill, string &sState) {
  bFill = false;
  int coco = CPC_VV_S_ColorAndString(sDeviceName, dsDpes, daValues, sColor, sState, true);
  if(coco >= 0) {
    int iObjectSt = daValues[13];
    bFill = iObjectSt == 0 || iObjectSt == 1;
  }
  return coco;
}

bool CPC_VV_S_MatchesCriteria(string sDpName, dyn_string dsDpes, dyn_anytype daDpeValues, shared_ptr<DeviceListCriteria> criteria,
                                string &sStateString, string &sStateColor, string &sValueString) {
  
  if(!CPC_VV_S_CheckCriteria(sDpName, dsDpes, daDpeValues, criteria)) {
    return false;
  }

  // List of DPEs/values shall be filtered out for state color/string calculation
  // Determine the body color based on the state
  dyn_int diKeyFilter = makeDynInt(3, 4, 5);
  dyn_string filteredDpes = unVacObject_dynWithoutIndex(dsDpes, diKeyFilter);
  dyn_anytype filteredValues = unVacObject_dynWithoutIndex(daDpeValues, diKeyFilter);
  CPC_VV_S_ColorAndString(sDpName, filteredDpes, filteredValues, sStateColor, sStateString, true);
  return true;
}

bool CPC_VV_S_CheckCriteria(string sDpName, dyn_string dsDpes, dyn_anytype daDpeValues, shared_ptr<DeviceListCriteria> criteria) {
  
  if(criteria.isEmpty()) {
    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "vacDeviceList",
                                "CPC_VV_S_CheckCriteria(" + sDpName +
                                "): match empty criteria");
    return true;  // Any device state matches empty criteria
  }
  criteria.resetResult();  // Make sure check result is reset in criteria

  // Further checks require interpretation of DPE values
 
  // Next - check NotConnected state: if device is not connected then it can only match
  // 'not connected' criteria, not any other
  int iSystemIntegrityAlarmValue = daDpeValues[1];
  bool bSystemIntegrityAlarmEnabled = daDpeValues[2];
  if(bSystemIntegrityAlarmEnabled && (iSystemIntegrityAlarmValue != 0)) {
    criteria.matchNotConnected();
    return criteria.isResult();
  }

  // Invalid DPE values - again, device can only match 'not connected' criteria
  if(daDpeValues[8] || daDpeValues[10] || daDpeValues[12] || daDpeValues[14]) {
    criteria.matchNotConnected();
    return criteria.isResult();
  }

  // Next - check device access errors. If device has access error problems - its state is,
  // most probably, unknown, and it can only match 'access error' criteria
  int iObjectSt = daDpeValues[13];
  // Access error can be found in ObjectSt value... may be also in StsReg0x?
  switch(iObjectSt) {
    case 5:  // Profibus Slave Error or Not present
      criteria.macthAccessErr();
      return criteria.isResult();
    }

  // Next - check normal/abnormal status. For valve use the criteria:
  //  OPEN = normal state
  //  CLOSED = abnormal state
  bool bNormal = false, bAbnormal = false;
  switch(iObjectSt) {
    case 2:  // Closed without Error (Start Interlock Bad)(Full Interlock Bad)
    case 3:  // Closed with Error (Start Interlock Bad)(Full Interlock Bad)
      bAbnormal = true;
      break;
    case 0:  // Opened with error (Start Interlock Bad)
    case 1:  // Opened without error
      bNormal = true;
      break;
    case 4:  // Undefined
      // both normal and abnormal remain false
      break;
    }
  if(criteria.matchNormalState(bNormal, bAbnormal)) {
    return criteria.isResult();
  }

  // Next - check other criteria (errors/warnings). Not sure what exactly shall be checked here
  uint uStsReg01 = daDpeValues[7];
  if((uStsReg01 & 0x00C0)) { // IOErrorW +  IOSimuW
    if(criteria.matchError()) {
      return criteria.isResult();
    }
  }

  // If we've reached this point - device does not match criteria
  return false;
}

