/**@file

// cpcVV_SConfig.ctl
This library contains the import and export function of the CPC_VV_S.

@par Creation Date
  27/10/2016

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author
  Thijs van Winden (TE-VSC)

@copyright
        &copy;Copyright CERN 2016 - all rights reserved
*/
//@{



// Constants
const string UN_CONFIG_CPC_VV_S_DPT_NAME = "CPC_VV_S";
//begin_TAG_SCRIPT_DEVICE_TYPE_constants



// import row: import row config line description : data value separated by ';'
//-------------
// // 10 GENERIC (CPC_VV_S;deviceNumber;Alias[,DeviceLinkList];Description;Diagnostics;WWWLink;Synoptic;Domain;Nature;WidgetType;) 
// // 7(of 8) MAIN ADDRESS (addr_StsReg01;addr_StsReg02;addr_StsReg03;addr_EvStsReg01;addr_EvStsReg02;addr_ManReg01;addr_ObjectSt;)
// // 1(of 8) MAIN MAPPING (ObjectSt_mapping)
// // 3 ARCHIVE (BooleanArchive;AnalogArchive;EventArchive;)
// // 10 ADDITIONAL (MaskEvent;Parameters;MasterDevice;parents;children;type;secondAlias;ControllerUnit;)

const unsigned UN_CONFIG_CPC_VV_S_ADDRESS_STSREG01         	= 1;	
const unsigned UN_CONFIG_CPC_VV_S_ADDRESS_STSREG02         	= 2;	
const unsigned UN_CONFIG_CPC_VV_S_ADDRESS_STSREG03         	= 3;	
const unsigned UN_CONFIG_CPC_VV_S_ADDRESS_EVSTSREG01       	= 4;	
const unsigned UN_CONFIG_CPC_VV_S_ADDRESS_EVSTSREG02       	= 5;	
const unsigned UN_CONFIG_CPC_VV_S_ADDRESS_EVSTSREG03       	= 6;	
const unsigned UN_CONFIG_CPC_VV_S_ADDRESS_MANREG01         	= 7;	
const unsigned UN_CONFIG_CPC_VV_S_ADDRESS_OBJECTST         	= 8;	
const unsigned UN_CONFIG_CPC_VV_S_MAPPING					          = 9; // Mapping of the meaning for ObjectSt

const unsigned UN_CONFIG_CPC_VV_S_LENGTH			              = 9;


const unsigned UN_CONFIG_CPC_VV_S_ARCHIVE_BOOLEAN_ARCHIVE	  = 1;
const unsigned UN_CONFIG_CPC_VV_S_ARCHIVE_ANALOG_ARCHIVE   	= 2;
const unsigned UN_CONFIG_CPC_VV_S_ARCHIVE_EVENT_ARCHIVE    	= 3;

const unsigned UN_CONFIG_CPC_VV_S_ARCHIVE_LENGTH           	= 3;

const unsigned UN_CONFIG_CPC_VV_S_ADDITIONAL_MASKEVENT     	= 1;
const unsigned UN_CONFIG_CPC_VV_S_ADDITIONAL_PARAMETERS    	= 2;
const unsigned UN_CONFIG_CPC_VV_S_ADDITIONAL_MASTER_NAME   	= 3;
const unsigned UN_CONFIG_CPC_VV_S_ADDITIONAL_PARENTS       	= 4;
const unsigned UN_CONFIG_CPC_VV_S_ADDITIONAL_CHILDREN      	= 5;
const unsigned UN_CONFIG_CPC_VV_S_ADDITIONAL_TYPE          	= 6;
const unsigned UN_CONFIG_CPC_VV_S_ADDITIONAL_SECOND_ALIAS  	= 7;
const unsigned UN_CONFIG_CPC_VV_S_ADDITIONAL_CONTROLLER_UNIT	= 8;

const unsigned UN_CONFIG_CPC_VV_S_ADDITIONAL_LENGTH        = 8;
//end_TAG_SCRIPT_DEVICE_TYPE_constants

//------------------------------------------------------
// NOTES :
// (1) ARCHIVE_ACTIVE
//N for NO archiving
//VA,0.1,N for Absolute Value archiving with 0.1 deadband and NO time
//VR,0.1,N as above, but Relative Deadband
//Y for Time-only archiving
//VA,0.1,A for Absolute Value archiving with 0.1 deadband AND Time
//VR,0.1,A for Relative Value archiving with 0.1 deadband AND Time  DEFAULT ARCHIVE SETUP FOR VACUUM ANALOG VALUE
//VA,0.1,O for Absolute Value archiving with 0.1 deadband OR Time
//VR,0.1,O for Relative Value archiving with 0.1 deadband OR Time
//A for Old/new comparison AND time
//O for Old/new comparison OR time



/**
DPE configuration
TODO: use fw info instead
*/
mapping CPC_VV_SConfig_getConfig() {
    mapping config;
    mapping props;

//begin_TAG_SCRIPT_DEVICE_TYPE_deviceConfig
// 9 ADDRESS (addr_StsReg01;addr_StsReg02;addr_EvStsReg01;addr_EvStsReg02;addr_PosSt;addr_APosSt;addr_VPosSt;addr_ManReg01;addr_ObjectSt;)
// 1
    mappingClear(props);
    props["address"] 		= ".ProcessInput.StsReg01";
    props["dataType"] 		= CPC_UINT16;
    config["StsReg01"] 		= props;
// 2
    mappingClear(props);
    props["address"] 		= ".ProcessInput.StsReg02";
    props["dataType"] 		= CPC_UINT16;
    config["StsReg02"] 		= props;
// 3
    mappingClear(props);
    props["address"] 		= ".ProcessInput.StsReg03";
    props["dataType"] 		= CPC_UINT16;
    config["StsReg03"] 		= props;
// 4
    mappingClear(props);
    props["hasArchive"] 	= true;
    props["address"] 		= ".ProcessInput.evStsReg01";
    props["dataType"] 		= CPC_INT32;
    config["evStsReg01"] 	= props;
// 5
    mappingClear(props);
    props["hasArchive"] 	= true;
    props["address"] 		= ".ProcessInput.evStsReg02";
    props["dataType"] 		= CPC_INT32;
    config["evStsReg02"] 	= props;
// 6
    mappingClear(props);
    props["hasArchive"] 	= true;
    props["address"] 		= ".ProcessInput.evStsReg03";
    props["dataType"] 		= CPC_INT32;
    config["evStsReg03"] 	= props;
// 7
    mappingClear(props);
    props["address"] 		= ".ProcessOutput.ManReg01";
    props["dataType"] 		= CPC_UINT16;
    config["ManReg01"] 		= props;
// 8
	  mappingClear(props);
   	props["hasMapping"]     = true;
    props["hasUnit"]        = false;
    props["hasFormat"]      = false;
    //props["hasSmooth"]   	= false; does not work
	//props["hasArchive"]     = true; archive setting done in custom function
    props["address"]        = ".ProcessInput.ObjectSt";
    props["dataType"]       = CPC_UINT16;
    config["ObjectSt"]      = props;
    
// 9
    mappingClear(props);
    props["address"] 		  = ".ProcessInput.StartISt";
    props["dpe"]			    = "StsReg01";
    props["bitPosition"]	= CPC_StsReg01_STARTIST;
    props["isAlarm"]		  = true;
    config["StartISt"] 		= props;
// 10
    mappingClear(props);
    props["address"] 		  = ".ProcessInput.TStopISt";
    props["dpe"]			    = "StsReg01";
    props["bitPosition"]	= CPC_StsReg01_TSTOPIST;
    props["isAlarm"]		  = true;
    config["TStopISt"] 		= props;
// 11
    mappingClear(props);
    props["address"] 		  = ".ProcessInput.FuStopISt";
    props["dpe"]			    = "StsReg02";
    props["bitPosition"]	= CPC_StsReg02_FUSTOPIST;
    props["isAlarm"]		  = true;
    config["FuStopISt"] 	= props;
// 12
    mappingClear(props);
    props["address"]      = ".ProcessInput.AlSt";
    props["dpe"]          = "StsReg01";
    props["bitPosition"]  = CPC_StsReg01_ALST;
    props["isAlarm"]      = true;
    config["AlSt"]        = props;
// 13
    mappingClear(props);
    props["address"]      = ".ProcessInput.AlUnAck";
    props["dpe"]          = "StsReg01";
    props["bitPosition"]  = CPC_StsReg01_ALUNACK;
    config["AlUnAck"]     = props;


//end_TAG_SCRIPT_DEVICE_TYPE_deviceConfig

    return config;
}

/**
Returns the list of parameter names that passed via config line.
*/
dyn_string CPC_VV_SConfig_getParamNames() {
//begin_TAG_SCRIPT_DEVICE_TYPE_getParamNames
    // TODO add gauge?
    return makeDynString(CPC_PARAMS_MRESTART, CPC_PARAMS_RSTARTFS);
//end_TAG_SCRIPT_DEVICE_TYPE_getParamNames
}

/** Check custom configuration
*/
CPC_VV_SConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
//end_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
}

/** Set custom configuration
*/
CPC_VV_SConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig
    cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.TStopISt");
    cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.FuStopISt");
    cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.StartISt");
    cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.AlSt");
	
	//Import the properties using cpcGenericDpFunctions_setDeviceProperty(string sDeviceDpName, string propertyName, string propertyValue, dyn_string &exceptionInfo)
	//Import Property CONTROLLER MODEL
	int iConfigDataIndex = UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VV_S_ARCHIVE_LENGTH + UN_CONFIG_CPC_VV_S_LENGTH + UN_CONFIG_CPC_VV_S_ADDITIONAL_CONTROLLER_UNIT;
	cpcGenericDpFunctions_setDeviceProperty(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], CPC_PROP_NAME_CONTROLLER_MODEL, dsConfigs[iConfigDataIndex], exceptionInfo);
  iConfigDataIndex = UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VV_S_ARCHIVE_LENGTH + UN_CONFIG_CPC_VV_S_LENGTH + UN_CONFIG_CPC_VV_S_ADDITIONAL_TYPE;
	cpcGenericDpFunctions_setDeviceProperty(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], CPC_PROP_NAME_MACHINE_TYPE, dsConfigs[iConfigDataIndex], exceptionInfo);
	
	//Archive setting for ObjectSt
	//fwArchive_set(string dpe, string archiveClassName, int archiveType, int smoothProcedure, float deadband, float timeInterval, dyn_string &exceptionInfo, bool checkClass = TRUE)
	fwArchive_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.ObjectSt",
                  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
                  DPATTR_ARCH_PROC_SIMPLESM,
                  DPATTR_COMPARE_OLD_NEW,
                  0,
                  0,
                  exceptionInfo);
	
//end_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig
}

/**
Purpose: Export CPC_VV_S Devices

Usage: External function

PVSS manager usage: NG, NV
*/
CPC_VV_SConfig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    string sObject = UN_CONFIG_CPC_VV_S_DPT_NAME;
    int amount = dynlen(dsDpList);
    for (int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if (!dpExists(currentDP)) { continue; }

        //begin_TAG_SCRIPT_DEVICE_TYPE_export
        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);

        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getDigitalNormalPosition(currentDP, sObject, makeDynString(".ProcessInput.StartISt", ".ProcessInput.TStopISt", ".ProcessInput.FuStopISt"), dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg02", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg03", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg02", 	true, dsDpParameters);
		    cpcExportGenericFunctions_processAddress(currentDP, sObject, "ObjectSt",      true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);

        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString("ProcessInput.OnSt", "ProcessInput.ObjectSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01", "ProcessInput.evStsReg02"), dsDpParameters);
        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);
      //end_TAG_SCRIPT_DEVICE_TYPE_export

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

/**
Purpose: Export CPC_VV_S Devices for S7_PLC front-end

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_CPC_VV_S_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VV_SConfig_ExportConfig(dsDpList, exceptionInfo);
}

/**
Purpose: Export CPC_VV_S Devices for _UnPlc front-end

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_CPC_VV_S_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VV_SConfig_ExportConfig(dsDpList, exceptionInfo);
}

OPCUA_CPC_VV_S_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VV_SConfig_ExportConfig(dsDpList, exceptionInfo);
}

BACnet_CPC_VV_S_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VV_SConfig_ExportConfig(dsDpList, exceptionInfo);
}

//@}
