// unVacuumObject_OpenFaceplate
/**
@author Sebastien Blanchard (TE-VSC)

@version 1.0 2017-05-22

@par Description
Open dedicated vacuum faceplate for UNICOS CPC core instance object

@par Constraints
unVaccum component installed

@par Usage
External

@par PVSS managers
UI

@param sDpName string **IN** widget device name
@param sPanelFile string **IN** faceplate panel file name
@param exceptionInfo dyn_string **OUT** exception for error

*/

unVacuumObject_OpenFaceplate(string sDpName, string sPanelFile, dyn_string &exceptionInfo)
{
	string deviceName, sPanelName, sModule1, sModule2;
	int xPos, yPos, openMarge = 5;
	float widgetHeight, widgetWidth;
	dyn_int diSize;
	bool bModule1, bModule2, bPanel1, bPanel2;

	deviceName = unGenericDpFunctions_getDpName(sDpName);
  if (dpExists(deviceName))
		{
		if (sPanelFile != "") {
// Build panel name
			sPanelName = unGenericDpFunctions_getAlias(deviceName) + " " + unGenericDpFunctions_getDescription(deviceName) + "   ";
// Check if panel is already opened in unicos graphical frame
			sModule1 = myModuleName();
			if (strpos(sModule1, ";unicosGFPanel") > 0)
				{
				sModule2 = sModule1;
				strreplace(sModule2, ";unicosGFPanel", "");
				}
			else
				{
				sModule2 = sModule1 + ";unicosGFPanel";
				}
                        bModule1 = (bool)isModuleOpen(sModule1);
                        bModule2 = (bool)isModuleOpen(sModule2);
                        if(bModule1)
                          bPanel1 = unGraphicalFrame_isPanelOpen(sPanelName, sModule1);
                        if(bModule2)
                          bPanel2 = unGraphicalFrame_isPanelOpen(sPanelName, sModule2);
			if (!bPanel1 && !bPanel2)
				{
// Get open position
				if (this.name == "WidgetArea")
					{
					getValue("", "position", xPos, yPos);
					getValue("", "size", widgetWidth, widgetHeight);
					diSize = getPanelSize(sPanelFile);
					widgetWidth = widgetWidth + 15;	// Add lock bitmap width
					xPos = (int) (xPos + widgetWidth + openMarge);
					yPos = (int) (yPos + widgetHeight + openMarge);
					if (dynlen(diSize) == 2)
						{
						if ((xPos + diSize[1]) >= (UN_BASE_PANEL_LEFT + UN_BASE_PANEL_WIDTH))
							{
							xPos = (int)(xPos - diSize[1] - widgetWidth - openMarge);
							}
						if ((yPos + diSize[2]) >= (UN_BASE_PANEL_TOP + UN_BASE_PANEL_HEIGHT))
							{
							yPos = (int)(yPos - diSize[2] - widgetHeight - openMarge);
							}
						}
// 5. Open faceplate
					unGraphicalFrame_ChildPanelOn(sPanelFile, sPanelName, makeDynString("$sDpName:" + deviceName), xPos, yPos);
          }
				else
					{
					unGraphicalFrame_ChildPanelOnCentral(sPanelFile, sPanelName, makeDynString("$sDpName:" + deviceName));
					}
				}
			else
				{
				if (bPanel1)
					{
					setInputFocus(sModule1, sPanelName, "Button1");
					}
				else if(bPanel2)
					{
					setInputFocus(sModule2, sPanelName, "Button1");
					}
				}
			}
		}
	else
		{
		fwException_raise(exceptionInfo, "ERROR", "unVacuumObject_OpenFaceplate:" + getCatStr("unGeneration", "DPDOESNOTEXIST"), "");
		}
}



/**Animate widget

di$...$Bits and ds$...$Letters must have the same length. dyn arrays must be defined in priority order: lowest -> highest priority .

@author Thijs van Winden

@par Constraints
None

@par Usage
Internal

@par PVSS managers
UI

@param sWidgetType widget type = dpType + widgetType. ex: AnaDigHeater
@param sDpType object
@param bLocked for select animation
@param sSelectedManager for select animation
@param dbit32StsReg Dyn list of status registers
@param diWarningBits warning bit positions in status register.
@param dsWarningLetters associated warning letters to be displayed.
@param diAlarmBits alarm bit positions in status register (StsReg01).
@param dsAlarmLetters associated alarm letters to be displayed (StsReg01).
@param diControlBitNames list of StsReg constants for controls
@param diControlBitLetters list of StsReg constants for controls
@param onBitConstant on bit constant name
@param offBitConstant off bit constant name
@param bInvalidLetter true if invalid letter as to be shown
@param bInvalidBody invalid animation on body ?
@param bAlarmUnack If the alarm is unacknowledged
@param bIsRestartNeeded True if a restart is needed
@param bDisabled True if the device is disabled
@param iSystemIntegrityAlarmValue the value of the systemIntegrity alarm for the PLC:
@param bSystemIntegrityAlarmEnabled the systemIntegrity alarm is enabled
*/
void unVacObject_WidgetAnimation(string sWidgetType, string sDpType,
bool bLocked, string sSelectedManager, dyn_bit32 dbit32StsRegs,
dyn_bool dbBodyColorBools, dyn_string dsBodyColors,
dyn_bool dbWarningBits, dyn_string dsWarningLetters,
dyn_bool dbAlarmBits, dyn_string dsAlarmLetters,
dyn_bool dbControlBits, dyn_string dsControlBitLetters,
bool bOn, bool bUndefined, bool bInvalidLetter, bool bInvalidBody, bool bAlarmUnack, bool bIsRestartNeeded, bool bDisabled, bool bComment,
int iSystemIntegrityAlarmValue, bool bSystemIntegrityAlarmEnabled) {
	string sColorSelect, sWarningLetter, sWarningColor, sAlarmLetter, sAlarmColor, sControlLetter, sControlColor, sBodyColor, sFunction;
	bool bLockVisible;
	dyn_string dsFunctions, exceptionInfo;

	// Animate the letters of the widget
	unVacObject_WidgetLetterAnimation(sWidgetType, sDpType, bLocked, sSelectedManager,  dbit32StsRegs,
		dbWarningBits, dsWarningLetters,
		dbAlarmBits, dsAlarmLetters,
		dbControlBits, dsControlBitLetters,
		bInvalidLetter, bIsRestartNeeded,  bComment,
		iSystemIntegrityAlarmValue,  bSystemIntegrityAlarmEnabled
	);

  unVacObject_WidgetBodyAnimation(sWidgetType, sDpType, dbit32StsRegs, dbBodyColorBools, dsBodyColors, bOn, bUndefined);
}

/**Animate widget

di$...$Bits and ds$...$Letters must have the same length. dyn arrays must be defined in priority order: lowest -> highest priority .

@author Thijs van Winden

@par Constraints
None

@par Usage
Internal

@par PVSS managers
UI

@param sWidgetType widget type = dpType + widgetType. ex: AnaDigHeater
@param sDpType object
@param bLocked for select animation
@param sSelectedManager for select animation
@param dbit32StsReg Dyn list of status registers
@param dbWarningBits List of boolean related with the warning letters
@param dsWarningLetters associated warning letters to be displayed.
@param dbAlarmBits Alarm bit positions in status register (StsReg01).
@param dsAlarmLetters associated alarm letters to be displayed (StsReg01).
@param dbControlBits list of boolean related with the letters
@param diControlBitLetters list of StsReg constants for controls
@param bInvalid true if invalid letter as to be shown
@param bAlarmUnack If the alarm is unacknowledged
@param bIsRestartNeeded True if a restart is needed
@param iSystemIntegrityAlarmValue the value of the systemIntegrity alarm for the PLC:
@param bSystemIntegrityAlarmEnabled the systemIntegrity alarm is enabled
*/
void unVacObject_WidgetLetterAnimation(string sWidgetType, string sDpType,
bool bLocked, string sSelectedManager, dyn_bit32 dbit32StsRegs,
dyn_bool dbWarningBits, dyn_string dsWarningLetters,
dyn_bool dbAlarmBits, dyn_string dsAlarmLetters,
dyn_bool dbControlBits, dyn_string dsControlBitLetters,
bool bInvalid, bool bIsRestartNeeded, bool bComment,
int iSystemIntegrityAlarmValue, bool bSystemIntegrityAlarmEnabled) {
	string sColorSelect, sWarningLetter, sWarningColor, sAlarmLetter, sAlarmColor, sControlLetter, sControlColor, sBodyColor, sFunction;
	bool bLockVisible;

	// Do nothing if not connected
	if (!g_bSystemConnected){
		return;
	}

	// 1. Select
	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sColorSelect, bLockVisible);
	// 2. Warning (if the stsreg02 exists then the treatment is different)
	unVacObject_WidgetWarningTextAnimation(dbWarningBits, dsWarningLetters, sWarningLetter, sWarningColor);
	cpcGenericObject_WidgetValidnessAnimation(bSystemIntegrityAlarmEnabled, iSystemIntegrityAlarmValue, bInvalid, sWarningLetter, sWarningColor);

	// 3. Alarm
	if (!bInvalid) {
		unVacObject_WidgetAlarmRestartAnimation(bIsRestartNeeded, sAlarmLetter, sAlarmColor);
		unVacObject_WidgetAlarmTextAnimation(dbAlarmBits, dsAlarmLetters, sAlarmLetter, sAlarmColor);
	}

	// 4. Control state
	if (!bInvalid) {
		unVacObject_WidgetControlStateAnimation(dbControlBits, dsControlBitLetters, sControlLetter, sControlColor);
	}

	// Determine callback name
	sFunction = sDpType+"_WidgetLetterAnimation";
	if(!isFunctionDefined(sFunction) || sFunction == ""){
		sFunction = "unVacObject_animateWidgetStatusLetters";
	}

	// Exec callback and default if there is none
	execScript("main(string sWarningLetter, string sWarningColor, "+
		"string sAlarmLetter, string sAlarmColor, string sControlLetter, string sControlColor, "+
		"string sColorSelect, bool bLockVisible, bool bComment, dyn_bit32 dbit32StsRegs) {" +
		sFunction + "(sWarningLetter, sWarningColor, sAlarmLetter, sAlarmColor, sControlLetter, sControlColor, sColorSelect, bLockVisible, bComment, dbit32StsRegs);}",
		makeDynString(), sWarningLetter, sWarningColor, sAlarmLetter, sAlarmColor, sControlLetter, sControlColor, sColorSelect, bLockVisible, bComment, dbit32StsRegs
	);
}

/**Animate widget

di$...$Bits and ds$...$Letters must have the same length. dyn arrays must be defined in priority order: lowest -> highest priority .

@author Thijs van Winden

@par Constraints
None

@par Usage
Internal

@par PVSS managers
UI

@param sWidgetType widget type = dpType + widgetType. ex: AnaDigHeater
@param sDpType object
@param bLocked for select animation
@param sSelectedManager for select animation
@param dbit32StsReg Dyn list of status registers
@param dbWarningBits List of boolean related with the warning letters
@param dsWarningLetters associated warning letters to be displayed.
@param dbAlarmBits Alarm bit positions in status register (StsReg01).
@param dsAlarmLetters associated alarm letters to be displayed (StsReg01).
@param dbControlBits list of boolean related with the letters
@param diControlBitLetters list of StsReg constants for controls
@param bInvalid true if invalid letter as to be shown
@param bAlarmUnack If the alarm is unacknowledged
@param bIsRestartNeeded True if a restart is needed
@param iSystemIntegrityAlarmValue the value of the systemIntegrity alarm for the PLC:
@param bSystemIntegrityAlarmEnabled the systemIntegrity alarm is enabled
*/
void unVacObject_WidgetBodyAnimation(string sWidgetType, string sDpType,
dyn_bit32 dbit32StsRegs,
dyn_bool dbBodyColorBools, dyn_string dsBodyColors,
bool bOn, bool bUndefined) {
	string sBodyColor, sFunction;

	// Do nothing if not connected
	if (!g_bSystemConnected){
		return;
	}

	// 6. Animate body
	unVacObject_WidgetAnimateBodyColor(dbBodyColorBools, dsBodyColors, sBodyColor);

	sFunction = cpcGenericObject_GetAnimationFunction(sWidgetType);

	if(!isFunctionDefined(sFunction)){
		sFunction = sDpType+"_WidgetBodyAnimation";
	}

	if (isFunctionDefined(sFunction) && sFunction != "") {
		execScript("main(bool bOn, bool bUndefined, string sBodyColor, dyn_bit32 dbit32StsRegs) {" +
			sFunction + "(bOn, bUndefined, sBodyColor, dbit32StsRegs);}",
			makeDynString(), bOn, bUndefined, sBodyColor, dbit32StsRegs);

	}
}


/**
Animates the widget status letters

@param sWarningLetter The warning letter
@param sWarningColor The warning color
@param sAlarmLetter The alarm letter
@param sAlarmColor The alarm color
@param sControlLetter The control letter
@param sControlColor The control letter color
@param sColorSelect The color select area
@param bLockVisible Check if the lock must be shown
@param bComment Wheter the comment icon must be shown

@return int
*/

int unVacObject_animateWidgetStatusLetters(string sWarningLetter, string sWarningColor,
string sAlarmLetter, string sAlarmColor, string sControlLetter, string sControlColor,
string sColorSelect, bool bLockVisible, bool bComment ){
	return setMultiValue("WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
		"AlarmText", "text", sAlarmLetter, "AlarmText", "foreCol", sAlarmColor,
		"ControlStateText", "text", sControlLetter, "ControlStateText", "foreCol", sControlColor,
		"SelectArea", "foreCol", sColorSelect, "LockBmp", "visible", bLockVisible,
		"commentsButton", "visible", bComment,
		"WidgetArea", "visible", true
	);
}

/**
  Find all the values corresponding to listed indices

  @param daArray The data array of which the values with the indices are selected from
  @param diKeys The list of keys for which the values will be selected
  @return Filtered array only consisting of values corresponding to keys in diKeys
*/
dyn_anytype unVacObject_dynWithoutIndex(dyn_anytype daArray, dyn_int diKeys){
  dyn_anytype daResult;
  for (int i = 1; i<=dynlen(daArray); i++){
    if(dynContains(diKeys, i) > 0) continue;
    dynAppend(daResult, daArray[i]);
  }
  return daResult;

}

/**
  Find all the values corresponding to listed indices

  @param daArray The data array of which the values with the indices are selected from
  @param diKeys The list of keys for which the values will be selected
  @return Filtered array only consisting of values corresponding to keys in diKeys
*/
dyn_anytype unVacObject_dynFindIndex(dyn_anytype daArray, dyn_int diKeys){
  dyn_anytype daResult;
  for (int i = 1; i<=dynlen(diKeys); i++){
    dynAppend(daResult, daArray[diKeys[i]]);
  }
  return daResult;

}

/**
  Slice a selected copy from the array

  @param daArray The data array of which the values with the indices are selected from
  @param iStart The start index from which the array will be sliced
  @param iLength The length of the sliced portion of the array
  @return Filtered array only consisting of the sliced portion of the original array
*/
dyn_anytype unVacObject_dynSlice(dyn_anytype daArray, int iStart = 1, int iLength = -1){
  // Check whether the index is within bounds and select the full array otherwise
  if(iStart > dynlen(daArray)){
    iStart = dynlen(daArray);
  }
  if(iLength < 0 || iStart+iLength > dynlen(daArray)){
    iLength = dynlen(daArray)-iStart;
  }

  dyn_anytype daResult;
  for (int i = iStart; i<iStart+iLength; i++){
    dynAppend(daResult, daArray[i]);
  }
  return daResult;
}


/**
Fetch a values from a list of status registers based on a StsReg0*_ Constant name

@param dbit32StsReg Dyn list of status registers
@param name The bit names constant as an array of string
@return set of bit values
*/
dyn_bool unVacObject_fetchBitsFromStsRegs(dyn_bit32 dbit32StsRegs, dyn_string dsNames, bool bDefault = false){
	dyn_bool result;
	// Loop over all the bit names and fetch their value als a boolean
	for(int i = 1; i<=dynlen(dsNames); i++){
		if(dsNames[i] == ""){
			result[i] = bDefault;
		} else {
			result[i] = unVacObject_fetchBitFromStsRegs(dbit32StsRegs, dsNames[i]);
		}
	}
	return result;
}

/**
Fetch a values from a list of status registers based on a StsReg0*_ Constant name

@param dbit32StsReg Dyn list of status registers
@param name The bit name constant as a string
@return value of the bit
*/
bool unVacObject_fetchBitFromStsRegs(dyn_bit32 dbit32StsRegs, string name){
	if (name == "") {
    errClass error = makeError("unVacuum", PRIO_SEVERE, ERR_IMPL, 100, "unVacObject_fetchBitFromStsRegs: Bit name is empty");
    throw( error );
	}

  // @TODO New component 1.0.1


  string sKey, sConstant;
  dyn_string exceptionInfo;


  dyn_string names = strsplit(name, "_");

  if (dynlen(names) != 3) {
    errClass error = makeError("unVacuum", PRIO_SEVERE, ERR_IMPL, 100, "unVacObject_fetchBitFromStsRegs: Wrong bit name _ " + name);
    throw( error );
  }

  // Split the constant to query the cache: key + constant
  if( substr(name, 0, 4) == "CPC_" ) {
    sKey = "CPC";
    sConstant = substr(name, 4);
  }
  else {
    errClass error = makeError("unVacuum", PRIO_SEVERE, ERR_IMPL, 100, "cpcGenericObject_WidgetControlStateAnimation() -> constant doesn't belong to CPC: " + name, "");
    throw( error );
  }



  int pos = cpcConfigGenericFunctions_getConstantByName(sKey, sConstant);

	// int pos = cpcConfigGenericFunctions_getConstantByName(name); // old interface
  // @TODO End of New component 1.0.1
	if (pos < 0) {
    errClass error = makeError("unVacuum", PRIO_SEVERE, ERR_IMPL, 101, "unVacObject_fetchBitFromStsRegs: Constant with name: " + name + " is not found or negative");
    throw( error );
	}

	dyn_string names = strsplit(name, "_");

	if (dynlen(names) != 3) {
    errClass error = makeError("unVacuum", PRIO_SEVERE, ERR_IMPL, 102, "unVacObject_fetchBitFromStsRegs: wrong constant name - " + name);
    throw( error );
	}

	string stsReg = names[2];
	strreplace(stsReg, "StsReg", "");
	int reg = stsReg;

	if(dynlen(dbit32StsRegs) < reg){
    errClass error = makeError("unVacuum", PRIO_SEVERE, ERR_IMPL, 103, "unVacObject_fetchBitFromStsRegs: No status register " + names[2]);
    throw( error );
	}

	return getBit(dbit32StsRegs[reg], pos) == 1;
}

/**
Determine the warning letter
@param dbit32StsReg Dyn list of status registers
@param dsBits warning bit positions constant name in status register, "StsReg0*_"
@param sLetter The warning letter
@param sColor The warning color
*/
void unVacObject_WidgetWarningTextAnimation(dyn_bool dbBits, dyn_string dsLetters, string &sLetter, string &sColor){
	sColor =  "cpcColor_Widget_Warning";
	unVacObject_WidgetBoolMapping(dbBits, dsLetters, sLetter);
}

/**
Determine the alarm letter
@param dbit32StsReg Dyn list of status registers
@param dsBits warning bit positions in status register
@param sLetter The warning letter
@param sColor The warning color
*/
void unVacObject_WidgetAlarmTextAnimation(dyn_bool dbBits, dyn_string dsLetters, string &sLetter, string &sColor){
	sColor =  "unFaceplate_AlarmActive";
	unVacObject_WidgetBoolMapping(dbBits, dsLetters, sLetter);
}

/**
Determine the alarm letter
@param dbit32StsReg Dyn list of status registers
@param dsBits warning bit positions in status register
@param sLetter The warning letter
@param sColor The warning color
*/
void unVacObject_WidgetAlarmRestartAnimation(bool bIsRestartNeeded, string &sLetter, string &sColor){
	sColor =  "unFaceplate_AlarmActive";
	if (bIsRestartNeeded) {
		sLetter = CPC_WIDGET_TEXT_MANUAL_RESTART;
		sColor = "cpcColor_Widget_Warning";
	}
}

/**
Determine the control letter animation
@param dbit32StsReg Dyn list of status registers
@param dsBits warning bit positions in status register
@param sLetter The warning letter
@param sColor The warning color
*/
void unVacObject_WidgetControlStateAnimation(dyn_bool dbBits, dyn_string dsLetters, string &sLetter, string &sColor){
	sColor = "unWidget_ControlStateManualLocal";
	unVacObject_WidgetBoolMapping(dbBits, dsLetters, sLetter);
	if (sLetter == CPC_WIDGET_TEXT_CONTROL_FORCED) {
		sColor = "unWidget_ControlStateForced";
	}
}

/**
Map bits to widget letters

@param dbBits Bit booleans in status register corresponding to the letters in dsletters
@param sLetter The warning letter
@param sColor The warning color
*/

void unVacObject_WidgetBoolMapping(dyn_bool dbBits, dyn_string dsLetters, string &letter){
	if(dynlen(dbBits) != dynlen(dsLetters)){
    errClass error = makeError("unVacuum", PRIO_SEVERE, ERR_IMPL, 103, "unVacObject_WidgetBoolMapping: Boolean list["+dynlen(dbBits)+"] and string list["+dynlen(dsLetters)+"] have not the same length");
    throw( error );
	}
	// Loop over all the bits and return the letter for which it is valid
	for(int i = 1;i<=dynlen(dbBits); i++){
		if(dbBits[i]){
			letter = dsLetters[i];
			return;
		}
	}
}

/**
Map bits to widget letters

@param dbit32StsReg Dyn list of status registers
@param dsBits warning bit positions in status register
@param sLetter The warning letter
@param sColor The warning color
*/

void unVacObject_WidgetBitLetterMapping(dyn_bit32 dbit32StsRegs, dyn_string dsBits, dyn_string dsLetters, string &letter){
	// Loop over all the bits and return the letter for which it is valid
	for(int i = 1;i<=dynlen(dsBits); i++){
		if(unVacObject_fetchBitFromStsRegs(dbit32StsRegs, dsBits[i])){
			letter = dsLetters[i];
			break;
		}
	}
}

/**
  Determine the color based on a list of booleans. First boolean that is true will correspond to the color select from dsColors
  Example if bStatus[1] = false , bStatus[2] = true, then color dsColors[2] will be selected

  @param dbStatus Dyn list of booleans
  @param dsColors Dyn list of colors
  @param sColor The body color
*/

void unVacObject_WidgetAnimateBodyColor(dyn_bool dbStatus, dyn_string dsColors, string &sColor){
        sColor = "cpcColor_Widget_Status";					// Auto mode (default)
        unVacObject_WidgetBoolMapping(dbStatus, dsColors, sColor);
}


/**
  Add an item to the menu with proper access

  @param dsMenuList         dyn_string, INPUT/OUTPUT, the menu to which an item will be attached
  @param sMenuLabel         string,     INPUT, the label of the item
  @param iMenuReturnValue   int,        INPUT, the return number of the item (has to be unique)
  @param bGranted           bool,       INPUT, flag to enable or disable the item
  @param iPos               int         INPUT, position to insert the menu item at, defaults to appending at the end

  @return The position at which the menu item is inserted
*/
int unVacObject_addItemToPopupMenu(dyn_string &dsMenuList, string sMenuLabel, int iMenuReturnValue, bool bEnabled, int iPos = 0){
  // If no position is supplied, append
  if(iPos == 0){
    iPos = dynlen(dsMenuList)+1;
  }

  string sAccess = bEnabled ? "1" : "0";
  dynInsertAt(dsMenuList, "PUSH_BUTTON, " + sMenuLabel + ", " + iMenuReturnValue + ", "+sAccess, iPos);

  return iPos;
}

/**
  Add the analog values to the context menu.
  For a proper action a constant CPC_POPMENU_%DPNAME% uppercase must be defined

  @param sDeviceName input, device DP name
  @param sDpType input, device type
  @param mDpeNames input, mapping where the key is the label of the menu item and the value is the dpe
  @param dsAccessOk input, the access control
  @param dsMenuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
  @param iPos input, position to insert the menu item at, defaults to appending at the end

  @return The position at which the menu item(s) is/are inserted
*/
int unVacObject_addAnalogActionToMenu(string sDeviceName, string sDpType, mapping mDpeNames, dyn_string dsAccessOk, dyn_string &dsMenuList, int iPos = 0){
  // If no position is supplied, append
  if(iPos == 0){
    iPos = dynlen(dsMenuList)+1;
  }

  string sFormat, sUnit, sFormattedValue, sDpeName, sMenuItem;
	float fValue;
	int iAction;
	// Loop over all the analog dpe elements
	for(int i = 1; i<=mappinglen(mDpeNames); i++){
		sDpeName = sDeviceName + ".ProcessInput."+mappingGetValue(mDpeNames, i);
		sUnit = dpGetUnit(sDpeName);
		sFormat = dpGetFormat(sDpeName);
		dpGet(sDpeName, fValue);

		// Format the value
		sFormattedValue = unGenericObject_FormatValue(sFormat, fValue);
		// Create the menu item where the key
		sMenuItem = mappingGetKey(mDpeNames, i)+": " +sFormattedValue + " " + sUnit;



   // @TODO New component 1.0.1


  string sKey = "CPC_";
  string sConstant = "POPUPMENU_"+strtoupper(mappingGetValue(mDpeNames, i));
  iAction = cpcConfigGenericFunctions_getConstantByName(sKey, sConstant);
	// iAction = cpcConfigGenericFunctions_getConstantByName("CPC_POPUPMENU_"+strtoupper(mappingGetValue(mDpeNames, i))); // old interface
  // @TODO End of New component 1.0.1





		iPos = unVacObject_addItemToPopupMenu(dsMenuList, sMenuItem, iAction, false, iPos+i-1);
	}

  return iPos;
}

/**
  Add the disable/enable buttons to the context menu. List of menu items is given as

  @param sDeviceName input, device DP name
  @param sDpType input, device type
  @param dsAccessOk input, the access control
  @param dsMenuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
  @param iPos input, position to insert the menu item at, defaults to appending at the end

  @return The position at which the menu item(s) is/are inserted
*/
int unVacObject_addDisableEnableActionToMenu(string sDeviceName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList, int iPos = 0){
  // If no position is supplied, append
  if(iPos == 0){
    iPos = dynlen(dsMenuList)+1;
  }

	// Add a seperator to the context menu
	dynInsertAt(dsMenuList, "SEPARATOR", iPos);
	// add Enable/Disable button
	bool bDisabled;
	dpGet(sDeviceName + ".disabled", bDisabled);
	// Determine label of context item
	string sName = bDisabled ? "Enable" : "Disable";
	// Add item to the menu
	iPos = unVacObject_addItemToPopupMenu(dsMenuList, sName, CPC_POPUPMENU_DISABLE, true, ++iPos);
	dynInsertAt(dsMenuList, "SEPARATOR", ++iPos);

  return iPos;
}

/**
  Add the datapoint alias as button to the context menu


  @param sDeviceName input, device DP name
  @param sDpType input, device type
  @param dsAccessOk input, the access control
  @param dsMenuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
  @param iPos input, position to insert the menu item at, defaults to appending at the end

  @return The position at which the menu item(s) is/are inserted
*/
int unVacObject_addAliasActionToMenu(string sDeviceName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList, int iPos = 0) {
	// add device name to the menu
	string sAliasName= unGenericDpFunctions_getAlias(sDeviceName);
	return unVacObject_addItemToPopupMenu(dsMenuList, sAliasName + "- (click to copy)", CPC_POPUPMENU_ALIAS, true, iPos);
}

/**
  Add the help as button to the context menu

  @param sDeviceName input, device DP name
  @param sDpType input, device type
  @param dsAccessOk input, the access control
  @param dsMenuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
  @param iPos input, position to insert the menu item at, defaults to appending at the end

  @return The position at which the menu item(s) is/are inserted
*/
int unVacObject_addHelpActionToMenu(string sDeviceName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList, int iPos = 0) {
	// add device name to the menu
	return unVacObject_addItemToPopupMenu(dsMenuList, "Help", CPC_POPUPMENU_HELP, true, iPos);
}


/**
  Add the state history as button to the context menu


  @param sDeviceName input, device DP name
  @param sDpType input, device type
  @param dsAccessOk input, the access control
  @param dsMenuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
  @param iPos input, position to insert the menu item at, defaults to appending at the end

  @return The position at which the menu item(s) is/are inserted
*/
int unVacObject_addVacuumExtensionActionToMenu(string sDeviceName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList, int iPos = 0) {
  // If no position is supplied, append
  if(iPos == 0){
    iPos = dynlen(dsMenuList)+1;
  }

	// add device name to the menu
  dynInsertAt(dsMenuList, "SEPARATOR", iPos);

  // Add the state history help and trend history
  iPos = unVacObject_addHelpActionToMenu(sDeviceName, sDpType, dsAccessOk, dsMenuList, iPos);

  bool bEnabled = 0 == fwInstallation_checkInstalledComponent(UNVACSTATEHISTORY_COMPONENT);
  iPos = unVacObject_addItemToPopupMenu(dsMenuList, "StateHistory", CPC_POPUPMENU_STATEHISTORY, bEnabled, iPos+1);
  iPos = unVacObject_addTrendHistoryActionToMenu(sDeviceName, sDpType, dsAccessOk, dsMenuList, false, iPos+1);

  // Add standard vacuum shortcuts to menu
  dynInsertAt(dsMenuList,"CASCADE_BUTTON, Tools, 1", ++iPos);
  dynInsertAt(dsMenuList, "SEPARATOR", ++iPos);

  dynAppend(dsMenuList,"Tools");
 	unVacObject_addItemToPopupMenu(dsMenuList, "Create Work Order", CPC_POPUPMENU_WORKORDER, true);
	unVacObject_addItemToPopupMenu(dsMenuList, "Link to Position/Asset in inforDB", CPC_POPUPMENU_INFORDB, true);
	unVacObject_addItemToPopupMenu(dsMenuList, "Link to Functional Position in vacDB", CPC_POPUPMENU_VACDB, true);

  return iPos;
}

/**
  Add the unicos menu as a process submenu


  @param sDeviceName input, device DP name
  @param sDpType input, device type
  @param dsAccessOk input, the access control
  @param dsMenuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
  @param iPos input, position to insert the menu item at, defaults to appending at the end

  @return The position at which the menu item(s) is/are inserted
*/
int unVacObject_addProcessActionToMenu(string sDeviceName, string sDpType, dyn_string dsMenuConfig, dyn_string dsAccessOk, dyn_string &dsMenuList, int iPos = 0) {
  // If no position is supplied, append
  if(iPos == 0){
    iPos = dynlen(dsMenuList)+1;
  }

  // Add the faceplate
  if (dynContains(dsMenuConfig, UN_POPUPMENU_FACEPLATE_TEXT) > 0) {
    unVacObject_addItemToPopupMenu(dsMenuList, "Faceplate", CPC_POPUPMENU_FACEPLATE_NUMBER, true, iPos);
    dynRemove(dsMenuConfig, dynContains(dsMenuConfig, UN_POPUPMENU_FACEPLATE_TEXT));
  }
  // Add the comments
  unVacObject_addItemToPopupMenu(dsMenuList, UN_POPUPMENU_COMMENTS_LABEL, UN_POPUPMENU_COMMENTS_ID, TRUE, ++iPos);
	// Add the process parent item
  dynInsertAt(dsMenuList,"CASCADE_BUTTON, Process, 1", ++iPos);
  // Add the expert parent item
  dynInsertAt(dsMenuList,"CASCADE_BUTTON, Expert, 1", ++iPos);


  dynAppend(dsMenuList,"Process");
  cpcGenericObject_addUnicosActionToMenu(sDeviceName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
  cpcGenericObject_addDefaultUnicosActionToMenu(sDeviceName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);

  // Split the last menu in 2. Expert and process
  for(int i = dynlen(dsMenuList); i>1; i--){
    if(strpos(dsMenuList[i], UN_POPUPMENU_COMMENTS_LABEL) < 0) continue;

    dsMenuList[i] = "Expert";
    break;
  }

  return iPos;
}


const int UN_POPUPMENU_UNVACTREND = 7000;

/**
  Add the state history as button to the context menu


  @param sDeviceName input, device DP name
  @param sDpType input, device type
  @param dsAccessOk input, the access control
  @param dsMenuList output, pop-up menu to show, dyn_string to be given to the popupMenu function\
  @param bUseAlias, input, Determine wheter the alias or the real name should be used
  @param iPos input, position to insert the menu item at, defaults to appending at the end

  @return The position at which the menu item(s) is/are inserted
*/
int unVacObject_addTrendHistoryActionToMenu(string sDeviceName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList, bool bUseAlias = false, int iPos = 0) {
  // If no position is supplied, append
  if(iPos == 0){
    iPos = dynlen(dsMenuList)+1;
  }

	int iMenuValue, iCste = UN_POPUPMENU_UNVACTREND;
  dyn_string dsAnalog, dsRange, exceptionInfo, dsDPEAnalog, dsDPEAlias, dsProxy, dsDPE,
              dsDPERange, dsCurveDPE, dsFaceplateDPE, dsCurveColor;
  bool bProxy;
  dyn_bool dbCurveVisible;
  dyn_float dfCurveMin, dfCurveMax;
  string sLabel, sVersion;

  unGenericObject_GetTrendDPEList(sDpType, dsAnalog, dsRange, exceptionInfo);
  unGenericDpFunctions_getProxyDeviceList(sDeviceName, bProxy, dsProxy);
  unGenericObject_deviceGetAllTrendDPE(sDeviceName, bProxy, bUseAlias, dsProxy,
                                         dsAnalog, dsRange, makeDynBool(), makeDynString(),
                                         makeDynFloat(), makeDynFloat(),
                                         dsFaceplateDPE, dsDPEAnalog, dsDPE, dsDPEAlias, dsDPERange, dsCurveDPE,
                                         dbCurveVisible, dsCurveColor,
                                         dfCurveMin, dfCurveMax);

  // Add the trend when unVacuum component is not installed
  if(0 != fwInstallation_checkInstalledComponent(UNVACTRENDING_COMPONENT)){
    unGenericObject_addTrendActionToMenu(sDeviceName, sDpType, makeDynString(UN_POPUPMENU_TREND_TEXT), dsAccessOk, dsMenuList);
    return iPos-1;
  }

  // Add all the analog values
  dynInsertAt(dsMenuList,"CASCADE_BUTTON, Trend, 1", iPos);
	dynAppend(dsMenuList,"Trend");
  for(int i = 1; i <= dynlen(dsDPEAnalog); i++){
    // Calculate the action identifier
    iMenuValue = iCste+i;
    // Determine the label
    sLabel = (bUseAlias) ? dsDPEAlias[i] : dsDPEAnalog[i];
    // Add the item to the popup menu
    unGenericObject_addItemToPopupMenu(dsMenuList, sLabel, iMenuValue, TRUE);
  }

  return iPos;
}


/**
Handle the answer of the popup menu for the vacuum actions. Will return false if the action is not found.
This is different than the normal unicos, but usefull because the total context menu of unicos can be
overwritten

@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param menuList input, the access control
@param menuAnswer input, selected menu value
@param helpFile input, The help file for the widget

@return false
*/


bool unVacObject_handleCustomUnicosMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer, string helpFile, dyn_string &exceptionInfo){
	dyn_string exceptionInfo;
	string sAliasName, sUrl;
	bool bDisabled;

	//begin_TAG_SCRIPT_DEVICE_TYPE_HandleMenu
	sAliasName= unGenericDpFunctions_getAlias(deviceName);
	switch(menuAnswer){
	case CPC_POPUPMENU_ALIAS:
		setClipboardText(sAliasName);
		break;
	case CPC_POPUPMENU_DISABLE:
		dpGet(deviceName + ".disabled", bDisabled);
		// Toggle disabled and inverse toggle the eventmask
		dpSet(deviceName + ".disabled", !bDisabled);
		dpSet(deviceName + ".eventMask", bDisabled);
		break;
	case CPC_POPUPMENU_NAME:
		setClipboardText(deviceName);
		break;
	case CPC_POPUPMENU_HELP:
		std_help(helpFile);
		break;
  case CPC_POPUPMENU_STATEHISTORY:
    vacStateHistory_addDevice(makeDynString(deviceName), true, exceptionInfo);
    break;
  case CPC_POPUPMENU_TRENDHISTORY:
    unVacObject_handleTrendHistoryMenu();
    break;
  case CPC_POPUPMENU_WORKORDER:
		sUrl = "https://apex-sso.cern.ch/pls/htmldb_edmsdb/f?p=300:150:6590923767320::NO::P150_WO_STATUS,P150_WO_DESC,P150_MRC,P150_WO_TYPE,P150_STANDARD_WO,P150_OBJ_CODE:R,Failure%20Eqp%20" + sAliasName + ",V01,INSP,V-CO-HW," + sAliasName;
		std_help(sUrl);
		break;
	case CPC_POPUPMENU_INFORDB:
		sUrl = "https://apex-sso.cern.ch/pls/htmldb_edmsdb/f?p=300:220:6590923767320::NO::P220_EQP_CODE,P220_FETCHED_WS_CUST_FIELD:" + sAliasName + ",N";
		std_help(sUrl);
		break;
	case CPC_POPUPMENU_VACDB:
		sUrl = "https://apex-sso.cern.ch/pls/htmldb_edmsdb/f?p=300:220:6590923767320::NO::P220_EQP_CODE,P220_FETCHED_WS_CUST_FIELD:" + sAliasName + ",N";
		std_help(sUrl);
		break;
	default:
		return unVacObject_handleTrendHistoryMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
	}
	return true;
}

/**
  Handle the addition of a trend to the UnVacTrending tool

  @param sDeviceName input, device DP name
  @param sDpType input, device type
  @param dsMenuList input, the access control
  @param iMenuAnswer input, selected menu value
  @param exceptionInfo input, Exception info

  @throws
  @throws

  @return True if one of the trend actions was actually chosen, false otherwise
*/
bool unVacObject_handleTrendHistoryMenu(string sDeviceName, string sDpType, dyn_string dsMenuList, int iMenuAnswer, dyn_string &exceptionInfo){
  int iTrend = iMenuAnswer-UN_POPUPMENU_UNVACTREND;

  // Check wheter the trend action is actually selected
  if(iTrend < 1){
    return false;
  }

  dyn_string dsAnalog, dsRange, exceptionInfo, dsDPEAnalog, dsDPEAlias, dsProxy, dsDPE,
              dsDPERange, dsCurveDPE, dsFaceplateDPE, dsCurveColor;
  bool bProxy, bUseAlias = false;
  dyn_bool dbCurveVisible;
  dyn_float dfCurveMin, dfCurveMax;
  string sLabel;

  // Load the trends from the object
  unGenericObject_GetTrendDPEList(sDpType, dsAnalog, dsRange, exceptionInfo);
  unGenericDpFunctions_getProxyDeviceList(sDeviceName, bProxy, dsProxy);
  unGenericObject_deviceGetAllTrendDPE(sDeviceName, bProxy, bUseAlias, dsProxy,
                                         dsAnalog, dsRange, makeDynBool(), makeDynString(),
                                         makeDynFloat(), makeDynFloat(),
                                         dsFaceplateDPE, dsDPEAnalog, dsDPE, dsDPEAlias, dsDPERange, dsCurveDPE,
                                         dbCurveVisible, dsCurveColor,
                                         dfCurveMin, dfCurveMax);

  // Check wheter the trend action is actually selected
  if(iTrend > dynlen(dsDPEAnalog)){
    return false;
  }

  // Open or focuse trend
  if(!unVacTrending_open()){
    // Throw error
  }

  // Add the trend to the graph
  unVacTrending_add(dsDPE[iTrend]);

  // Succesfully added trend
  return true;
}



/**
Generic animation of  the widget letter. Parameter bitnames should be a array of strings which represent the bit constants,
while the parameters letters represents the corresponding array of letters. By the name of the constant the bit is fetched from
the registers. The first entry in the array is the most important and with increasing index the priority decreases.

@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values
@param bitNames input, the array of bit name strings, like "CPC_StsReg01_ALST"
@param letters input, the array of letters, like "LI", "PI"
@param letter output, the resulting letter or default as input
*/
void unVacObject_widgetLetterMapping(dyn_string dpes, dyn_anytype values, dyn_string bitNames, dyn_string letters, string &letter){
	bool invalid, value, result, resultInvalid;
	for(int i = 1;i<=dynlen(bitNames); i++){
		unFaceplate_fetchStatusBitAndValidness(dpes, values, bitNames[i], value, invalid);
		if(!invalid && value){
			letter = letters[i];
			break;
		}
	}
}

/**
Generic animation of the vacuum faceplate device status section. The unit and format of the online value can be overwritten via
g_params[%name%Unit]. If not they will automatically loaded from the configuration.

@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
void unVacObject_animateFaceplateOnlineValues(dyn_string dpes, dyn_anytype values, dyn_string dsDpeNames) {
	string sUnitName, sFormatName;
	int k;
	for(int i = 1; i<=dynlen(dsDpeNames);i++){
		// Create globals so only has to be loaded once
		sUnitName = dsDpeNames[i]+"Unit";
		sFormatName = dsDpeNames[i]+"Format";

		// Populate the globals for the units
		if(!mappingHasKey(g_params, sUnitName)){
			k = unFaceplate_fetchAnimationCBIndex(dpes, values, dsDpeNames[i]);
			if(i >= 0){
				g_params[sUnitName] = dpGetUnit(dpes[k]);
			}
		}
		// Populate the globals for the format
		if(!mappingHasKey(g_params, sFormatName)){
			k = unFaceplate_fetchAnimationCBIndex(dpes, values, dsDpeNames[i]);
			if(k >= 0){
				g_params[sFormatName] = dpGetFormat(dpes[k]);
			}
		}
		// Animate the faceplate
		unFaceplate_animateOnlineValue(dpes, values, dsDpeNames[i], g_params[sUnitName], g_params[sFormatName], "cpcColor_Faceplate_Status");
	}
}

/**
Generic animation of the vacuum faceplate device status section. The mapping of the ObjectSt can be overwritten via
g_params[ObjectStLabels]. If not they will automatically loaded from the configuration.

@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
void unVacObject_animateFaceplateObjectStatus(dyn_string dpes, dyn_anytype values) {
	//Load mapping if not yet set
	if(!mappingHasKey(g_params, "ObjectStLabels") && dynlen(dpes)>=0){
		mapping objectStLabels;
		dyn_string dsParts = strsplit(dpes[dynlen(dpes)], '.');
		cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(dsParts[1], ".ProcessInput.ObjectSt"), objectStLabels);
		g_params["ObjectStLabels"] = objectStLabels;
	}

	// Device status animation
	unFaceplate_animateTranslatedValue(dpes, values, "ObjectSt", "", "", g_params["ObjectStLabels"], "cpcColor_Faceplate_Status");
}

/**
Generic animation of the vacuum faceplate device status section. The mapping of the ObjectSt can be overwritten via
g_params[ObjectStLabels]. If not they will automatically loaded from the configuration.

@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
void unVacObject_animateFaceplateDeviceStatus(dyn_string dpes, dyn_anytype values) {
	unVacObject_animateFaceplateObjectStatus(dpes, values);
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_ONST", true, "cpcColor_Faceplate_Status");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_OFFST", true, "cpcColor_Faceplate_Status");
}
/**
Generic animation of the vacuum faceplate status device operating mode section

@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
void unVacObject_animateFaceplateOperationModes(dyn_string dpes, dyn_anytype values) {
	// Generic device modes
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUMOST", true, "cpcColor_Faceplate_Status");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_MMOST", true, "cpcColor_Faceplate_Status");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUINHMMO", true, "cpcColor_Faceplate_Status");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_FOMOST", true, "cpcColor_Faceplate_Status");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUINHFMO", true, "cpcColor_Faceplate_Status");
}

/**
Generic animation of the vacuum faceplate status device interlocks

@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
void unVacObject_animateFaceplateInterlocks(dyn_string dpes, dyn_anytype values) {
	// Generic device interlocks
	unFaceplate_animateInterlock(dpes, values, "CPC_StsReg01_STARTIST",   "StartISt",  "CPC_StsReg01_ALUNACK");
	unFaceplate_animateInterlock(dpes, values, "CPC_StsReg01_TSTOPIST",  "TStopISt",  "CPC_StsReg01_ALUNACK");
	unFaceplate_animateInterlock(dpes, values, "CPC_StsReg02_FUSTOPIST", "FuStopISt", "CPC_StsReg01_ALUNACK");
	unFaceplate_animateInterlock(dpes, values, "CPC_StsReg01_ALST",      "AlSt",      "CPC_StsReg01_ALUNACK");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_ALUNACK", true, "cpcColor_Alarm_Bad");
}

/**
Generic animation of the vacuum faceplate status device warnings

@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
void unVacObject_animateFaceplateWarnings(dyn_string dpes, dyn_anytype values) {
	// Generic device interlocks
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOERRORW", true, "cpcColor_Faceplate_Warning");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOSIMUW", true, "cpcColor_Faceplate_Warning");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_FODIPRO", true, "cpcColor_Faceplate_Warning");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_MIOERBRST", true, "cpcColor_Faceplate_Warning");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_ALBW", true, "cpcColor_Faceplate_Warning");
}

/**
Generic animation of the vacuum faceplate device request status

@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
void unVacObject_animateFaceplateRequestStatus(dyn_string dpes, dyn_anytype values) {
	// Generic device request status
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_AUOFFPRST", true, "cpcColor_Faceplate_Status");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_AUONPRST", true, "cpcColor_Faceplate_Status");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_MOFFPRST", true, "cpcColor_Faceplate_Status");
	unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_MONPRST", true, "cpcColor_Faceplate_Status");
}

/**Animate status bit.

Function accept array of bit's constant name (e.g. "UN_StsReg01_IOERRORW") as bitName. Constant's value should correspond to bit position in status dpe (e.g. StsReg01).
Faceplate should contain unFaceplate_Colorbox.pnl graphical object named '$BIT_NAME$' ("IOERRORW" in given example). In this case the first element constant of the
list is used for the bitname

Bit's order starts from 0 (i.e. first bit's index is 0).

@par Constraints
None

@par Usage
Public

@par PVSS managers
UI

@param dpes connected dpe names
@param values connected dpe values
@param bitName constants as a string array; see detailed description for more info
@param enabled if it's enabled
@param color field color on valid state
@param mode The bit check mode, can be OR or AND
*/
const string CPC_VAC_ANIMATE_MULTI_MODE_OR = "OR";
const string CPC_VAC_ANIMATE_MULTI_MODE_AND = "AND";
void unVacObject_animateStatusMultiBit(dyn_string dpes, dyn_string values, dyn_string bitNames, bool enabled, string color, string mode = CPC_VAC_ANIMATE_MULTI_MODE_OR) {
	if (dynlen(bitNames) == 0){
		return;
		//throw(makeError("At least one bitname must be specified", PRIO_SEVERE, ERR_PARAM));
	}

	if(!g_bSystemConnected){
		return;
	}

	dyn_string parts = strsplit(bitNames[1], "_");
	if (dynlen(dpes) == 0) { // online animation
		unGenericObject_ColorBoxDisconnect(makeDynString(parts[3]));
		return;
	}
	bool invalid, value, result, resultInvalid;
	result = (mode == CPC_VAC_ANIMATE_MULTI_MODE_AND);
	for(int i=1; i<=dynlen(bitNames); i++){
		unFaceplate_fetchStatusBitAndValidness(dpes, values, bitNames[i], value, invalid);
		resultInvalid = resultInvalid && invalid;
		result =( mode == CPC_VAC_ANIMATE_MULTI_MODE_OR) ? result || value : result && value;
	}
	unGenericObject_ColorBoxAnimate(parts[3], result, enabled, color, invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
	//  unGenericObject_ColorBoxAnimate(parts[3], result, enabled, color, false);
}

/**
Animate interlock based on multiple bits

Function accept array of bit's constant name (e.g. "UN_StsReg01_IOERRORW") as bitName. Constant's value should correspond to bit position in status dpe (e.g. StsReg01).
Faceplate should contain unFaceplate_Colorbox.pnl graphical object named '$BIT_NAME$' ("IOERRORW" in given example). In this case the first element constant of the
list is used for the bitname

Bit's order starts from 0 (i.e. first bit's index is 0).

@par Constraints
None

@par Usage
Public

@par PVSS managers
UI

@param dpes connected dpe names
@param values connected dpe values
@param bitName constants as a string array; see detailed description for more info
@param enabled if it's enabled
@param color field color on valid state
@param mode The bit check mode, can be OR or AND
*/
void unVacObject_animateInterlockMultiBit(dyn_string dpes, dyn_string values, dyn_string bitNames, string interlockName, string unAckBitName = "UN_STSREG01_ALUNACK", string mode = CPC_VAC_ANIMATE_MULTI_MODE_OR) {
	if (dynlen(bitNames) == 0){
		return;
		//throw(makeError("At least one bitname must be specified", PRIO_SEVERE, ERR_PARAM));
	}

	if(!g_bSystemConnected){
		return;
	}

	if (dynlen(dpes) == 0) { // online animation
		unGenericObject_ColoredSquareDisconnect(interlockName);
		return;
	}

	bool invalid, value, result, resultInvalid, unAck, unAckInvalid;
	int interlockState;
	dyn_string parts;
	value = (mode == CPC_VAC_ANIMATE_MULTI_MODE_AND);

	// Fetch aknowledge bit
	unFaceplate_fetchStatusBitAndValidness(dpes, values, unAckBitName, unAck, unAckInvalid);
	for(int i=1; i<=dynlen(bitNames); i++){
		// Fetch the bit values and interlock state value
		parts = strsplit(bitNames[i], "_");
		interlockState = unFaceplate_fetchAnimationCBValue(dpes, values, parts[3]);
		unFaceplate_fetchStatusBitAndValidness(dpes, values, bitNames[i], value, invalid);

		// Detect validness and calculate end result
		resultInvalid = resultInvalid && invalid;
		result = (mode == CPC_VAC_ANIMATE_MULTI_MODE_OR) ? result || value : result && value;
	}

	unFaceplate_ColorBoxAnimateInterlock(interlockName, interlockState, result, unAck,
	resultInvalid || unAckInvalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
}


// BUTTONS IN FACEPLATE
//--------------------------------

/** Return button configuration including access level

@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL

@param deviceName input, the name of the device
@param buttons input, the existing buttons

*/
mapping unVacObject_genericButtonConfig(string deviceName, mapping &buttons) {
	buttons[UN_FACEPLATE_MASKEVENT] = UN_ACCESS_RIGHTS_EXPERT;
	buttons[UN_FACEPLATE_BUTTON_SELECT] = UN_ACCESS_RIGHTS_OPERATOR;
	buttons[UN_FACEPLATE_BUTTON_AUTO_MODE] = UN_ACCESS_RIGHTS_OPERATOR;
	buttons[UN_FACEPLATE_BUTTON_MANUAL_MODE] = UN_ACCESS_RIGHTS_OPERATOR;
	buttons[UN_FACEPLATE_BUTTON_FORCED_MODE] = UN_ACCESS_RIGHTS_EXPERT;

	buttons[UN_FACEPLATE_BUTTON_ONOPEN_REQUEST] = UN_ACCESS_RIGHTS_OPERATOR;
	buttons[UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST] = UN_ACCESS_RIGHTS_OPERATOR;

	buttons[UN_FACEPLATE_BUTTON_ACK_ALARM] = UN_ACCESS_RIGHTS_OPERATOR;
}


/** Set the state of the contextual button of the device

@par Constraints
None

@par Usage
Public

@par PVSS managers
Ui, CTRL

@param deviceName input, the device DP name
@param dpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
void unVacObject_handleGenericButtonState(string deviceName, string dpType, dyn_string dsUserAccess, dyn_string dsData) {
	bool bSelected, buttonEnabled, bit1, bit2, bit3;
	string localManager;
	dyn_string exceptionInfo;

	bool bLocked = dsData[1];
	string sSelectedUIManager = dsData[2];

	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
	bSelected = (localManager == "S"); // Selection state
	dyn_string dsButtons = unSimpleAnimation_ButtonList(deviceName);

	//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
	mixed stsReg01Value = dsData[3];
	bool bStsReg01Bad = dsData[4];

	for (int i = 1; i <= dynlen(dsButtons); i++) {
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0); // User access
		switch (dsButtons[i]) {
		case UN_FACEPLATE_BUTTON_FORCED_MODE:
			bit1 = !(getBit(stsReg01Value, CPC_StsReg01_FOMOST) == 0);
			bit2 = !(getBit(stsReg01Value, CPC_StsReg01_AUINHFMO) == 0);
			buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2) && !bStsReg01Bad;
			break;
		case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
			//case UN_FACEPLATE_BUTTON_ONFORCED_REQUEST: //
		case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
		case UN_FACEPLATE_BUTTON_AUTO_MODE:
		case UN_FACEPLATE_BUTTON_SET_VALUE:
			bit1 = !(getBit(stsReg01Value, CPC_StsReg01_AUMOST) == 0);
			buttonEnabled = buttonEnabled && bSelected && !(bit1) && !bStsReg01Bad;
			break;
		case UN_FACEPLATE_BUTTON_MANUAL_MODE:
			buttonEnabled = buttonEnabled && bSelected &&
			!getBit(stsReg01Value, CPC_StsReg01_MMOST) &&
			!getBit(stsReg01Value, CPC_StsReg01_AUINHMMO) &&
			!bStsReg01Bad;
			break;
		case UN_FACEPLATE_BUTTON_OPERATION_MODES:
			buttonEnabled = buttonEnabled && bSelected &&
			getBit(stsReg01Value, CPC_StsReg01_MMOST) &&
			!bStsReg01Bad;
			break;
		case UN_FACEPLATE_BUTTON_RESETERRWARN_REQUEST:
			buttonEnabled = buttonEnabled && bSelected &&
			!bStsReg01Bad;
		default:
			break;
		}
		if (dsButtons[i] == UN_FACEPLATE_BUTTON_SELECT) {
			unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
		} else {
			cpcButton_setButtonState(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], buttonEnabled);
		}
	}
	//end_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
}

/**
  Get the help link for a device.

  Only supports vacuum objects and assumes the following format
  Device types is defined as VR_AH then the documentation for that object is
  in /operator/objects/vrah/

  @param sDeviceType The device type for this object
  @return The link to the documentation
*/
string unVacObject_getHelpLink(string sDeviceType){
  strreplace(sDeviceType, "_", "");
  strreplace(sDeviceType, "CPC", "");
  string sFileName = strtolower(sDeviceType) + "/";

  string sBaseUrl = "https://unicos-vacuum.web.cern.ch/operator/objects";
  string sHelpLink = sBaseUrl + "/" +sFileName;

  return sHelpLink;
}

/** Return the main state string for vacuum object

@par Constraints
Object with dpe .ProcessInput.ObjectSt

@par Usage
Public

@par PVSS managers
Ui, CTRL

@param string sDeviceName input, the device DP name
@param int iObjectSt input, the object status integer value
*/

string unVacObject_getStateString(string sDeviceName, int iObjectSt) {
  //return value is the meaning of ObjectSt
  mapping objectStLabels;
  string sState;
  cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(sDeviceName, ".ProcessInput.ObjectSt"), objectStLabels);//get the meaning of ObjectSt
  string sObjectSt = iObjectSt;
  if(mappingHasKey(objectStLabels, sObjectSt)) {
    sState = objectStLabels[sObjectSt];
    if (sState=="")
      sState = "Not defined";
    if (sState=="-1")
      sState = "ObjectSt="+iObjectSt+" Not defined";
  }
  else {
      sState = "ObjectSt="+sObjectSt+" Not defined";
  }
  return sState;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unGenericObject_OpenFaceplate
/**
Purpose: open faceplate

Parameters:
	- sDpName, string, input, deviceName
	- exceptionInfo, dyn_string, output, for errors

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unVacObject_OpenFaceplate(string sDpName, dyn_string dsDollarParams, dyn_string &exceptionInfo)
{
	string sFaceplateName, deviceName, sPanelName, sModule1, sModule2;
	int xPos, yPos, openMarge = 5;
	float widgetHeight, widgetWidth;
	dyn_int diSize;
	bool bModule1, bModule2, bPanel1, bPanel2;

  if(strpos(myModuleName(), "_Widget") > 0) {
    return;
  }
	deviceName = unGenericDpFunctions_getDpName(sDpName);
  dynAppend(dsDollarParams, "$sDpName:" + deviceName);
	if (dpExists(deviceName))
		{
// 1. Get faceplate path
		unGenericDpFunctions_getFaceplate(deviceName, sFaceplateName, exceptionInfo);
		if ((dynlen(exceptionInfo) <= 0) && (sFaceplateName != ""))
			{
// 2. Build panel name
			sPanelName = unGenericDpFunctions_getAlias(deviceName) + " " + unGenericDpFunctions_getDescription(deviceName) + "   ";
// 3. Check if panel is already opened in unicos graphical frame
			sModule1 = myModuleName();
			if (strpos(sModule1, ";unicosGFPanel") > 0)
				{
				sModule2 = sModule1;
				strreplace(sModule2, ";unicosGFPanel", "");
				}
			else
				{
				sModule2 = sModule1 + ";unicosGFPanel";
				}
                        bModule1 = (bool)isModuleOpen(sModule1);
                        bModule2 = (bool)isModuleOpen(sModule2);
                        if(bModule1)
                          bPanel1 = unGraphicalFrame_isPanelOpen(sPanelName, sModule1);
                        if(bModule2)
                          bPanel2 = unGraphicalFrame_isPanelOpen(sPanelName, sModule2);
			if (!bPanel1 && !bPanel2)
				{
// 4. Get open position
				if (this.name == "WidgetArea")
					{
					getValue("", "position", xPos, yPos);
					getValue("", "size", widgetWidth, widgetHeight);
					diSize = getPanelSize(sFaceplateName);
					widgetWidth = widgetWidth + 15;	// Add lock bitmap width
					xPos = (int) (xPos + widgetWidth + openMarge);
					yPos = (int) (yPos + widgetHeight + openMarge);
					if (dynlen(diSize) == 2)
						{
						if ((xPos + diSize[1]) >= (UN_BASE_PANEL_LEFT + UN_BASE_PANEL_WIDTH))
							{
							xPos = (int)(xPos - diSize[1] - widgetWidth - openMarge);
							}
						if ((yPos + diSize[2]) >= (UN_BASE_PANEL_TOP + UN_BASE_PANEL_HEIGHT))
							{
							yPos = (int)(yPos - diSize[2] - widgetHeight - openMarge);
							}
						}
// 5. Open faceplate
					unGraphicalFrame_ChildPanelOn(sFaceplateName, sPanelName, dsDollarParams, xPos, yPos);
					}
				else
					{
					unGraphicalFrame_ChildPanelOnCentral(sFaceplateName, sPanelName, dsDollarParams);
					}
				}
			else
				{
				if (bPanel1)
					{
					setInputFocus(sModule1, sPanelName, "Button1");
					}
				else if(bPanel2)
					{
					setInputFocus(sModule2, sPanelName, "Button1");
					}
				}
			}
		}
	else
		{
		fwException_raise(exceptionInfo, "ERROR", "unGenericObject_OpenFaceplate:" + getCatStr("unGeneration", "DPDOESNOTEXIST"), "");
		}
}


/**
Gets name of the controller - first object on "parents" list

@param deviceName input, device DP name
@return result string
*/
string unVacObject_getControllerName(string deviceName){
  dyn_string parents;
  dyn_string exceptionInfo;
  unGenericDpFunctions_getKeyDeviceConfiguration(deviceName, CPC_CONFIG_PARENTS_KEY, parents, exceptionInfo);
  if(dynlen(parents) > 0) {
    return parents[1];
  }
  else {
    return "";
  }
}


