

class DeviceListCriteria {

  // Constructor
  public DeviceListCriteria() {
    setDefault();
  }

  // Reset check result
  public void resetResult() {
    result = false;
  }

  // Check if criteria is empty; any device matches empty criteria
  public bool isEmpty() {
    //print();
	  return !
		  (normal || notNormal || abnormal || notAbnormal || error || warning ||
		  alert || accessErr || notConnected ||
		  (usePressureLimit && (pressureLimit > 0.0)) ||
		  (usePressureGrowLimit && (pressureGrowLimit > 0.0) && (pressureStartTime < pressureEndTime)) ||
		  (useTemperatureLimit && (temperatureLimit > 0.0)) ||
		  (useTemperatureGrowLimit && (temperatureGrowLimit > 0.0) && (temperatureStartTime < temperatureEndTime)));
  }

  // Set results of criteria check, returns boolean: if result is clear or not (yet)
  // NOTE: methods return NOT result itself, but only an indication: if result is clear or not.
  
  public bool matchNotConnected() {
    result = notConnected;
    return true;  // Check is finished anyway
  }

  public bool macthAccessErr() {
    result = accessErr;
    return true;  // Check is finished anyway
  }

  public bool matchNormalState(bool stateIsNormal, bool stateIsAbnormal) {
    if(stateIsNormal) {
      if(normal) {
        result = true;
      }
      else if(notAbnormal && (!notNormal)) {
        result = true;
      }
    }
    else if(stateIsAbnormal) {
      if(abnormal) {
        result = true;
      }
      else if(notNormal && (!notAbnormal)) {
        result = true;
      }
    }
    else {
      result = notNormal || notAbnormal;
    }
    return result;
  }

  public bool matchError() {
    if(warning) {
      result = true;
    }
    return result;
  }

  public bool matchWarning() {
    if(error || warning) {
      result = true;
    }
    return result;
  }

  public bool matchAlert() {
    if(alert) {
      result = true;
    }
    return result;
  }

  public bool matchPressureLimit() {
    if(usePressureLimit) {
      result = true;
    }
    return result;
  }

  // Get parameters for 'pressure grow' criteria check
  public bool getPressureGrowParams(time &startTime, time &endTime, float &factor, int &spikeInterval) {
    if(!usePressureGrowLimit) {
      return false;
    }
    if(pressureGrowLimit <= 0) {
      return false;
    }
    if(pressureStartTime >= pressureEndTime) {
      return false;
    }

    if(pressureEndTimeNow) {
      endTime = getCurrentTime();
      float delta = pressureEndTime - pressureStartTime;
      startTime = endTime - delta;
    }
    else {
      startTime = pressureStartTime;
      endTime = pressureEndTime;
    }
    factor = pressureGrowLimit;
    spikeInterval = pressureSpikeInterval;
    return true;
  }

  // Get parameters for 'temperature grow' criteria check
  public bool getTemperatureGrowParams(time &startTime, time &endTime, float &factor, int &spikeInterval) {
    if(!useTemperatureGrowLimit) {
      return false;
    }
    if(temperatureGrowLimit <= 0) {
      return false;
    }
    if(temperatureStartTime >= temperatureEndTime) {
      return false;
    }

    if(temperatureEndTimeNow) {
      endTime = getCurrentTime();
      float delta = temperatureEndTime - temperatureStartTime;
      startTime = endTime - delta;
    }
    else {
      startTime = temperatureStartTime;
      endTime = temperatureEndTime;
    }
    factor = temperatureGrowLimit;
    spikeInterval = temperatureSpikeInterval;
    return true;
  }

  // Set all parameters of criteria to default
  public void setDefault() {
    normal = false;
    notNormal = true;
    abnormal = true;
    notAbnormal = false;
    error = true;
    warning = false;
    alert = false;
    accessErr = true;
    notConnected = false;

    usePressureLimit = false;
    pressureLimit = 1E-6;
    usePressureGrowLimit = false;
    pressureEndTimeNow = true;
    pressureEndTime = getCurrentTime();
    pressureStartTime = pressureEndTime - 24 * 60 * 60;
    pressureGrowLimit = 2;
    pressureSpikeInterval = 5;

    useTemperatureLimit = false;
    temperatureLimit = 5;
    useTemperatureGrowLimit = false;
    temperatureEndTimeNow = true;
    temperatureEndTime = getCurrentTime();
    temperatureStartTime = temperatureEndTime - 24 * 60 * 60;
    temperatureGrowLimit = 2;
    temperatureSpikeInterval = 5;
    
    result = false;
  }

  // Access for members
  public bool isNormal() { return normal; }
  public void setNormal(bool value) { normal = value; notNormal = false; }
  public bool isNotNormal() { return notNormal; }
  public void setNotNormal(bool value) { notNormal = value; normal = false; }
  public bool isAbnormal() { return abnormal; }
  public void setAbnormal(bool value) { abnormal = value; notAbnormal = false; }
  public bool isNotAbnormal() { return notAbnormal; }
  public void setNotAbnormal(bool value) { notAbnormal = value; abnormal = false; }
  public bool isError() { return error; }
  public void setError(bool value) { error = value; }
  public bool isWarning() { return warning; }
  public void setWarning(bool value) { warning = value; }
  public bool isAlert() { return alert; }
  public void setAlert(bool value) { alert = value; }
  public bool isAccessError() { return accessErr; }
  public void setAccessError(bool value) { accessErr = value; }
  public bool isNotConnected() { return notConnected; }
  public void setNotConnected(bool value) { notConnected = value; }
  
  public bool isUsePressureLimit() { return usePressureLimit; }
  public void setUsePressureLimit(bool value) { usePressureLimit = value; }
  public float getPressureLimit() { return pressureLimit; }
  public void setPressureLimit(float value) { pressureLimit = value; }
  public bool isUsePressureGrowLimit() { return usePressureGrowLimit; }
  public void setUsePressureGrowLimit(bool value) { usePressureGrowLimit = value; }
  public bool isPressureEndTimeNow() { return pressureEndTimeNow; }
  public void setPressureEndTimeNow(bool value) { pressureEndTimeNow = value; }
  public time getPressureStartTime() { return pressureStartTime; }
  public void setPressureStartTime(time value) { pressureStartTime = value; }
  public time getPressureEndTime() { return pressureEndTime; }
  public void setPressureEndTime(time value) { pressureEndTime = value; }
  public float getPressureGrowLimit() { return pressureGrowLimit; }
  public void setPressureGrowLimit(float value) { pressureGrowLimit = value; }
  public int getPressureSpikeInterval() { return pressureSpikeInterval; }
  public void setPressureSpikeInterval(int value) { if(value < 0) value = 0; pressureSpikeInterval = value; }
  
  public bool isUseTemperatureLimit() { return useTemperatureLimit; }
  public void setUseTemperatureLimit(bool value) { useTemperatureLimit = value; }
  public float getTemperatureLimit() { return temperatureLimit; }
  public void setTemperatureLimit(float value) { temperatureLimit = value; }
  public bool isUseTemperatureGrowLimit() { return useTemperatureGrowLimit; }
  public void setUseTemperatureGrowLimit(bool value) { useTemperatureGrowLimit = value; }
  public bool isTemperatureEndTimeNow() { return temperatureEndTimeNow; }
  public void setTemperatureEndTimeNow(bool value) { temperatureEndTimeNow = value; }
  public time getTemperatureStartTime() { return temperatureStartTime; }
  public void setTemperatureStartTime(time value) { temperatureStartTime = value; }
  public time getTemperatureEndTime() { return temperatureEndTime; }
  public void setTemperatureEndTime(time value) { temperatureEndTime = value; }
  public float getTemperatureGrowLimit() { return temperatureGrowLimit; }
  public void setTemperatureGrowLimit(float value) { temperatureGrowLimit = value; }
  public int getTemperatureSpikeInterval() { return temperatureSpikeInterval; }
  public void setTemperatureSpikeInterval(int value) { if(value < 0) value = 0; temperatureSpikeInterval = value; }
  
  public bool isResult() { return result; }

  // Save/restore criteria, storage is DP
  public bool save(string sDpName, dyn_string &exceptionInfo) {
    if(!dpExists(sDpName)) {
      fwException_raise(exceptionInfo, "ERROR", "DeviceListCriteria.save(): DP does not exist <" +
                        sDpName + ">", "");
      return false;
    }
    dpSet(sDpName + ".Criteria.normal", normal,
          sDpName + ".Criteria.notNormal", notNormal,
          sDpName + ".Criteria.abnormal", abnormal,
          sDpName + ".Criteria.notAbnormal", notAbnormal,
          sDpName + ".Criteria.error", error,
          sDpName + ".Criteria.warning", warning,
          sDpName + ".Criteria.alert", alert,
          sDpName + ".Criteria.accessError", accessErr,
          sDpName + ".Criteria.notConnected", notConnected,
          sDpName + ".Criteria.Pressure.useLimit", usePressureLimit,
          sDpName + ".Criteria.Pressure.limit", pressureLimit,
          sDpName + ".Criteria.Pressure.useGrowLimit", usePressureGrowLimit,
          sDpName + ".Criteria.Pressure.endTimeNow", pressureEndTimeNow,
          sDpName + ".Criteria.Pressure.startTime", pressureStartTime,
          sDpName + ".Criteria.Pressure.endTime", pressureEndTime,
          sDpName + ".Criteria.Pressure.growLimit", pressureGrowLimit,
          sDpName + ".Criteria.Pressure.spikeInterval", pressureSpikeInterval,
          sDpName + ".Criteria.Temperature.useLimit", useTemperatureLimit,
          sDpName + ".Criteria.Temperature.limit", temperatureLimit,
          sDpName + ".Criteria.Temperature.useGrowLimit", useTemperatureGrowLimit,
          sDpName + ".Criteria.Temperature.endTimeNow", temperatureEndTimeNow,
          sDpName + ".Criteria.Temperature.startTime", temperatureStartTime,
          sDpName + ".Criteria.Temperature.endTime", temperatureEndTime,
          sDpName + ".Criteria.Temperature.growLimit", temperatureGrowLimit,
          sDpName + ".Criteria.Temperature.spikeInterval", temperatureSpikeInterval);
    dyn_errClass deErr = getLastError();
    if(dynlen(deErr) < 0) {
      fwException_raise(exceptionInfo, "ERROR", "DeviceListCriteria.save(): write criteria to <" +
                        sDpName + "> failed", "");
      return false;
    }
    return true;
  }
  
  public bool load(string sDpName, dyn_string &exceptionInfo) {
    if(!dpExists(sDpName)) {
      fwException_raise(exceptionInfo, "ERROR", "DeviceListCriteria.load(): DP does not exist <" +
                        sDpName + ">", "");
      return false;
    }
    dpGet(sDpName + ".Criteria.normal", normal,
          sDpName + ".Criteria.notNormal", notNormal,
          sDpName + ".Criteria.abnormal", abnormal,
          sDpName + ".Criteria.notAbnormal", notAbnormal,
          sDpName + ".Criteria.error", error,
          sDpName + ".Criteria.warning", warning,
          sDpName + ".Criteria.alert", alert,
          sDpName + ".Criteria.accessError", accessErr,
          sDpName + ".Criteria.notConnected", notConnected,
          sDpName + ".Criteria.Pressure.useLimit", usePressureLimit,
          sDpName + ".Criteria.Pressure.limit", pressureLimit,
          sDpName + ".Criteria.Pressure.useGrowLimit", usePressureGrowLimit,
          sDpName + ".Criteria.Pressure.endTimeNow", pressureEndTimeNow,
          sDpName + ".Criteria.Pressure.startTime", pressureStartTime,
          sDpName + ".Criteria.Pressure.endTime", pressureEndTime,
          sDpName + ".Criteria.Pressure.growLimit", pressureGrowLimit,
          sDpName + ".Criteria.Pressure.spikeInterval", pressureSpikeInterval,
          sDpName + ".Criteria.Temperature.useLimit", useTemperatureLimit,
          sDpName + ".Criteria.Temperature.limit", temperatureLimit,
          sDpName + ".Criteria.Temperature.useGrowLimit", useTemperatureGrowLimit,
          sDpName + ".Criteria.Temperature.endTimeNow", temperatureEndTimeNow,
          sDpName + ".Criteria.Temperature.startTime", temperatureStartTime,
          sDpName + ".Criteria.Temperature.endTime", temperatureEndTime,
          sDpName + ".Criteria.Temperature.growLimit", temperatureGrowLimit,
          sDpName + ".Criteria.Temperature.spikeInterval", temperatureSpikeInterval);
    dyn_errClass deErr = getLastError();
    if(dynlen(deErr) < 0) {
      fwException_raise(exceptionInfo, "ERROR", "DeviceListCriteria.load(): read criteria from <" +
                        sDpName + "> failed", "");
      return false;
    }
    
    // Correct possible inconsistencies
    if(normal && notNormal) {
      normal = false;
    }
    if(abnormal && notAbnormal) {
      notAbnormal = false;
    }

    if(pressureStartTime >= pressureEndTime) {
      pressureEndTime = pressureStartTime + 24 * 60 * 60;  // Default is one day
    }
    if(pressureEndTimeNow) {
      float fDeltaTime = pressureEndTime - pressureStartTime;
      pressureEndTime = getCurrentTime();
      pressureStartTime = pressureEndTime - fDeltaTime;
    }

    if(temperatureStartTime >= temperatureEndTime) {
      temperatureEndTime = temperatureStartTime + 24 * 60 * 60;  // Default is one day
    }
    if(temperatureEndTimeNow) {
      float fDeltaTime = temperatureEndTime - temperatureStartTime;
      temperatureEndTime = getCurrentTime();
      temperatureStartTime = temperatureEndTime - fDeltaTime;
    }
    
    return true;
  }

  public void print() {
    DebugN("Criteria is:\n    normal: " + normal +
          "\n    notNormal: " +  notNormal +
          "\n    abnormal: " +  abnormal +
          "\n    notAbnormal: " +  notAbnormal +
          "\n    error: " +  error +
          "\n    warning: " +  warning +
          "\n    alert: " +  alert +
          "\n    accessErr: " +  accessErr +
          "\n    notConnected: " +  notConnected +
          "\n    usePressureLimit: " +  usePressureLimit +
          "\n    pressureLimit: " +  pressureLimit +
          "\n    usePressureGrowLimit: " +  usePressureGrowLimit +
          "\n    pressureGrowLimit: " +  pressureGrowLimit +
          "\n    useTemperatureLimit: " +  useTemperatureLimit +
          "\n    temperatureLimit: " +  temperatureLimit +
          "\n    useTemperatureGrowLimit: " +  useTemperatureGrowLimit +
          "\n    temperatureGrowLimit: " +  temperatureGrowLimit);
  }

  // Data storage - all members are private by default
  bool normal;       // Device is in NORMAL state
  bool notNormal;    // Device is NOT in NORMAL state
  bool abnormal;     // Device is in abnormal state
  bool notAbnormal;  // Device is NOT in ABNOMAL state
  bool error;        // There is error for device
  bool warning;      // There is warning for device
  bool alert;        // Device produces alert
  bool accessErr;  // Access error for device
  bool notConnected; // Device is not connected

  // Pressure value criteria
  bool usePressureLimit;
  float pressureLimit;
  bool usePressureGrowLimit;
  bool pressureEndTimeNow;
  time pressureStartTime;
  time pressureEndTime;
  float pressureGrowLimit;
  int pressureSpikeInterval;

  // Temperature value criteria
  bool useTemperatureLimit;
  float temperatureLimit;
  bool useTemperatureGrowLimit;
  bool temperatureEndTimeNow;
  time temperatureStartTime;
  time temperatureEndTime;
  float temperatureGrowLimit;
  int temperatureSpikeInterval;

  // Overall result of check
  bool  result;
};

void someFunc() {
  DeviceListCriteria criteria;
}
