/**@file
 *
 * This package contains functions to support bit information for DPEs of UNICOS devices
 *
 * @author Leonid Kopylov (TE-VSC)
 * @date   March 2017
 */

// How the value of status register shall be interpreted
const int STATUS_VALUE_TYPE_BITS = 0;    // Interpret as a collection of individual bits
const int STATUS_VALUE_TYPE_VALUE = 1;   // Interpret as a value (enum in most of cases)


mapping vacDpeBitInfo_getDeviceBitInfo(string sDeviceName) {

  mapping mResult;
  string sTypeName = vacDpeBitInfo_getTypeName(sDeviceName);
  if(sTypeName == "") {
    return mResult;
  }
  
  if(!globalExists("g_mDpeBitInfo")) {
    addGlobal("g_mDpeBitInfo", MAPPING_VAR);
  }
  
  dyn_anytype daKeys = mappingKeys(g_mDpeBitInfo);
  if(dynContains(daKeys, sTypeName) > 0) {
    mResult = g_mDpeBitInfo[sTypeName];
  }
  else {
//    DebugTN("New type", sTypeName);
    string sFunctionName = "_vacDpeBitInfo_initTypeBitInfo_" + sTypeName;
    if(isFunctionDefined(sFunctionName)) {
      evalScript(mResult, "mapping main() { return " + sFunctionName + "(); }",
                 makeDynString());
    }
    g_mDpeBitInfo[sTypeName] = mResult;
  }
  DebugN(g_mDpeBitInfo);
  
  return mResult;
}

mapping _vacDpeBitInfo_initTypeBitInfo_CPC_VG_PT() {
  mapping mResult;

  // Info for each bit is dyn_string, containing 2 items:
  //  1) short name of bit
  //  2) long name of bit
  
  // StsReg01
  mapping mBitInfoStsReg01;
  mBitInfoStsReg01["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg01[0] = makeDynString("SwOffOV", "Switch Off Order Value");
  mBitInfoStsReg01[1] = makeDynString("SwOnOV", "Switch On Order Value");
  mBitInfoStsReg01[2] = makeDynString("AuMoSt", "Auto Mode Status");
  mBitInfoStsReg01[3] = makeDynString("MMoSt", "Manual Mode Status");
  mBitInfoStsReg01[4] = makeDynString("FoMoSt", "Forced Mode Status");
  mBitInfoStsReg01[5] = makeDynString("AutoOV", "Automatic Order Value");
  mBitInfoStsReg01[6] = makeDynString("IOErrorW", "Input/Output Error Warning");
  mBitInfoStsReg01[7] = makeDynString("IOSimuW", "Input/Output Simulated Warning");

  mBitInfoStsReg01[8] = makeDynString("FoDiProW", "Forced Differs from Process Warning");
  mBitInfoStsReg01[9] = makeDynString("MIOErBRSt", "Manual Input/Output Error Block Request Status");
  mBitInfoStsReg01[10] = makeDynString("StartISt", "Start Interlock Status");
  mBitInfoStsReg01[11] = makeDynString("TStopISt", "Temporary Stop Interlock Status");
  mBitInfoStsReg01[12] = makeDynString("AlUnAck", "Alarm UnAcknowledged");
  mBitInfoStsReg01[13] = makeDynString("AuIhFoMoSt", "Auto Inhibit Forced Mode Status");
  mBitInfoStsReg01[14] = makeDynString("AlSt", "Alarm Status");
  mBitInfoStsReg01[15] = makeDynString("AuIhMMoSt", "Auto Inhibit Manual Mode Status");
  
  mResult["StsReg01"] = mBitInfoStsReg01;

  // StsReg02
  mapping mBitInfoStsReg02;
  mBitInfoStsReg02["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg02[1] = makeDynString("AuOnPRSt", "Auto On Protect Request Status");
  mBitInfoStsReg02[2] = makeDynString("MOnPRSt", "Manual On Protect Request Status");
  mBitInfoStsReg02[3] = makeDynString("AuOffRSt", "Auto Off Request Status");
  mBitInfoStsReg02[4] = makeDynString("MOffRSt", "Manual Off Request Status");
  mBitInfoStsReg02[5] = makeDynString("AuOnFRSt", "Auto On Force Request Status");
  mBitInfoStsReg02[6] = makeDynString("MOnFRSt", "Manual On Force Request Status");
  mBitInfoStsReg02[7] = makeDynString("AuMRW", "Auto Manual Request Warning");

  mBitInfoStsReg02[10] = makeDynString("FuStopISt", "Full Stop Interlock Status");
  mBitInfoStsReg02[11] = makeDynString("EnRstartSt", "Enable Restart Status");
  mBitInfoStsReg02[13] = makeDynString("AlBW", "Alarm Blocked Warning");
  
  mResult["StsReg02"] = mBitInfoStsReg02;

  // Info for each bit is string with description

  // ObjectSt
  mapping mInfoObjectSt;
  mInfoObjectSt["type"] = STATUS_VALUE_TYPE_VALUE;
  mInfoObjectSt[0] = "0=Off without error";
  mInfoObjectSt[1] = "1=Over Range";
  mInfoObjectSt[2] = "2=Measurement OK";
  mInfoObjectSt[3] = "3=Under Range";
  mInfoObjectSt[4] = "4=No Hardware";
  mInfoObjectSt[5] = "5=Hardware Error";
  mInfoObjectSt[6] = "6= Profibus Slave Error";
  mInfoObjectSt[7] = "7=Profibus Slave Not present";
  mInfoObjectSt[8] = "8=High Pressure";

  mResult["ObjectSt"] = mInfoObjectSt;

  return mResult;
}

mapping _vacDpeBitInfo_initTypeBitInfo_CPC_VP_I() {
  mapping mResult;

  // Info for each bit is dyn_string, containing 2 items:
  //  1) short name of bit
  //  2) long name of bit
  
  // StsReg01
  mapping mBitInfoStsReg01;
  mBitInfoStsReg01["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg01[0] = makeDynString("SwOffOVSt", "Switch Off Order Value Status");
  mBitInfoStsReg01[1] = makeDynString("SwOnOVSt", "Switch On Order Value status");
  mBitInfoStsReg01[2] = makeDynString("AuMoSt", "Auto Mode Status");
  mBitInfoStsReg01[3] = makeDynString("MMoSt", "Manual Mode Status");
  mBitInfoStsReg01[4] = makeDynString("FoMoSt", "Forced Mode Status");
  mBitInfoStsReg01[5] = makeDynString("SwPrOVSt", "Switch Protect Order Value status");
  mBitInfoStsReg01[6] = makeDynString("IOErrorW", "Input/Output Error Warning");
  mBitInfoStsReg01[7] = makeDynString("IOSimuW", "Input/Output Simulated Warning");

  mBitInfoStsReg01[8] = makeDynString("AuMRW", "Auto Manual Request Warning Status");
  mBitInfoStsReg01[9] = makeDynString("PosW", "Position Warning");
  mBitInfoStsReg01[10] = makeDynString("StartISt", "Start Interlock Status");
  mBitInfoStsReg01[11] = makeDynString("TStopISt", "Temporary Stop Interlock Status");
  mBitInfoStsReg01[12] = makeDynString("AlUnAck", "Alarm UnAcknowledged");
  mBitInfoStsReg01[13] = makeDynString("AuIhFoMoSt", "Auto Inhibit Forced Mode Status");
  mBitInfoStsReg01[14] = makeDynString("AlSt", "Alarm Status");
  mBitInfoStsReg01[15] = makeDynString("AuIhMMoSt", "Auto Inhibit Manual Mode Status");
  
  mResult["StsReg01"] = mBitInfoStsReg01;

  // StsReg02
  mapping mBitInfoStsReg02;
  mBitInfoStsReg02["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg02[0] = makeDynString("OutOnOVSt", "Output On Order Value Status");
  mBitInfoStsReg02[1] = makeDynString("AuOnRSt", "Auto On Request Status");
  mBitInfoStsReg02[2] = makeDynString("MOnRSt", "Manual On Request Status");
  mBitInfoStsReg02[3] = makeDynString("AuOffRSt", "Auto Off Request Status");
  mBitInfoStsReg02[4] = makeDynString("MOffRSt", "Manual Off Request Status");
  mBitInfoStsReg02[5] = makeDynString("AuOnFRSt", "Auto On Force Request Status");
  mBitInfoStsReg02[6] = makeDynString("MOnFRSt", "Manual On Force Request Status");
  mBitInfoStsReg02[7] = makeDynString("AuStpMRSt", "Auto Step Mode Request Status");

  mBitInfoStsReg02[8] = makeDynString("MStpMRSt", "Manual Step Mode Request Status");
  mBitInfoStsReg02[9] = makeDynString("AuFixMRSt", "Auto Fixed Mode Request Status");
  mBitInfoStsReg02[10] = makeDynString("FuStopISt", "Full Stop Interlock Status");
  mBitInfoStsReg02[11] = makeDynString("EnRstartSt", "Enable Restart Status");
  mBitInfoStsReg02[13] = makeDynString("AlBW", "Alarm Blocked Warning");
  mBitInfoStsReg02[14] = makeDynString("MFixMRSt", "Manual Fixed Mode Request Status");
  
  mResult["StsReg02"] = mBitInfoStsReg02;

  // Info for each bit is string with description

  // ObjectSt
  mapping mInfoObjectSt;
  mInfoObjectSt["type"] = STATUS_VALUE_TYPE_VALUE;
  mInfoObjectSt[0] = "0: Off without error";
  mInfoObjectSt[1] = "1: Over Current";
  mInfoObjectSt[2] = "2: Measurement OK";
  mInfoObjectSt[3] = "3: Under Range";
  mInfoObjectSt[4] = "4: Open Cable";
  mInfoObjectSt[5] = "5: Error";
  mInfoObjectSt[6] = "6: Profibus Slave Error or Not present";

  mResult["ObjectSt"] = mInfoObjectSt;

  return mResult;
}

mapping _vacDpeBitInfo_initTypeBitInfo_CPC_VV_S() {
  mapping mResult;

  // Info for each bit is dyn_string, containing 2 items:
  //  1) short name of bit
  //  2) long name of bit
  
  // StsReg01
  mapping mBitInfoStsReg01;
  mBitInfoStsReg01["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg01[0] = makeDynString("OnSt", "On Status");
  mBitInfoStsReg01[1] = makeDynString("OffSt", "Off Status");
  mBitInfoStsReg01[2] = makeDynString("AuMoSt", "Auto Mode Status");
  mBitInfoStsReg01[3] = makeDynString("MMoSt", "Manual Mode Status");
  mBitInfoStsReg01[4] = makeDynString("FoMoSt", "Forced Mode Status");
  mBitInfoStsReg01[5] = makeDynString("LDSt", "Local Drive Status");
  mBitInfoStsReg01[6] = makeDynString("IOErrorW", "Input/Output Error Warning");
  mBitInfoStsReg01[7] = makeDynString("IOSimuW", "Input/Output Simulated Warning");

  mBitInfoStsReg01[8] = makeDynString("AuMRW", "Auto Manual Request Warning Status");
  mBitInfoStsReg01[9] = makeDynString("PosW", "Position Warning");
  mBitInfoStsReg01[10] = makeDynString("StartISt", "Start Interlock Status");
  mBitInfoStsReg01[11] = makeDynString("TStopISt", "Temporary Stop Interlock Status");
  mBitInfoStsReg01[12] = makeDynString("AlUnAck", "Alarm UnAcknowledged");
  mBitInfoStsReg01[13] = makeDynString("AuIhFoMoSt", "Auto Inhibit Forced Mode Status");
  mBitInfoStsReg01[14] = makeDynString("AlSt", "Alarm Status");
  mBitInfoStsReg01[15] = makeDynString("AuIhMMoSt", "Auto Inhibit Manual Mode Status");
  
  mResult["StsReg01"] = mBitInfoStsReg01;

  // StsReg02
  mapping mBitInfoStsReg02;
  mBitInfoStsReg02["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg02[0] = makeDynString("OutOnOVSt", "Output On Order Value Status");
  mBitInfoStsReg02[1] = makeDynString("AuOnRSt", "Auto On Request Status");
  mBitInfoStsReg02[2] = makeDynString("MOnRSt", "Manual On Request Status");
  mBitInfoStsReg02[3] = makeDynString("AuOffRSt", "Auto Off Request Status");
  mBitInfoStsReg02[4] = makeDynString("MOffRSt", "Manual Off Request Status");

  mBitInfoStsReg02[10] = makeDynString("FuStopISt", "Full Stop Interlock Status");
  mBitInfoStsReg02[11] = makeDynString("EnRstartSt", "Enable Restart Status");
  mBitInfoStsReg02[13] = makeDynString("AlBW", "Alarm Blocked Warning");
  mBitInfoStsReg02[14] = makeDynString("OutOffOVSt", "Output Off Order Value Status");
  
  mResult["StsReg02"] = mBitInfoStsReg02;

  // StsReg03
  mapping mBitInfoStsReg03;
  mBitInfoStsReg03["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg03[0] = makeDynString("Lcl1ItlSt", "Local interlock status (Three or more pressure sources are bad)");
  mBitInfoStsReg03[1] = makeDynString("OpEn1St", "Open enable status (One of the pressure sources is bad)");
  mBitInfoStsReg03[2] = makeDynString("TempItlSt/Lcl2ItlSt", "Temperature interlock status (LHC) / Local interlock status 2 (SPS)");
  mBitInfoStsReg03[3] = makeDynString("GlbBeforeItlSt", "Interlock from VVS-1 status");
  mBitInfoStsReg03[4] = makeDynString("GlbAfterItlSt", "Interlock from VVS+1 status");
  mBitInfoStsReg03[5] = makeDynString("PrItlEnSt/OpEn2St", "Pressure interlock enabled status (LHC) / Open enable status (SPS)");
  mBitInfoStsReg03[6] = makeDynString("VVSConSt", "Valve connected status");
  mBitInfoStsReg03[7] = makeDynString("BdRSt", "Beam dump request status when valve is closed");

  mBitInfoStsReg03[8] = makeDynString("ClEnSt", "Close enable status");
  mBitInfoStsReg03[9] = makeDynString("RemCtrEnSt", "Remote control enable status (inhibition key not active)");
  mBitInfoStsReg03[10] = makeDynString("PrevClSt", "Preventive close active status (interlock coming from CRYO)");

  mResult["StsReg03"] = mBitInfoStsReg03;

  // Info for each bit is string with description

  // ObjectSt
  mapping mInfoObjectSt;
  mInfoObjectSt["type"] = STATUS_VALUE_TYPE_VALUE;
  mInfoObjectSt[0] = "0: Opened with error (Start Interlock Bad)";
  mInfoObjectSt[1] = "1: Opened without error";
  mInfoObjectSt[2] = "2: Closed without Error (Start Interlock Bad)(Full Interlock Bad)";
  mInfoObjectSt[3] = "3: Closed with Error (Start Interlock Bad)(Full Interlock Bad)";
  mInfoObjectSt[4] = "4: Undefined";
  mInfoObjectSt[5] = "5: Profibus Slave Error or Not present";

  mResult["ObjectSt"] = mInfoObjectSt;

  return mResult;
}

mapping _vacDpeBitInfo_initTypeBitInfo_CPC_VP_TP() {
  mapping mResult;

  // Info for each bit is dyn_string, containing 2 items:
  //  1) short name of bit
  //  2) long name of bit
  
  // StsReg01
  mapping mBitInfoStsReg01;
  mBitInfoStsReg01["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg01[0] = makeDynString("OnSt", "On Status");
  mBitInfoStsReg01[1] = makeDynString("OffSt", "Off Status");
  mBitInfoStsReg01[2] = makeDynString("AuMoSt", "Auto Mode Status");
  mBitInfoStsReg01[3] = makeDynString("MMoSt", "Manual Mode Status");
  mBitInfoStsReg01[4] = makeDynString("FoMoSt", "Forced Mode Status");
  mBitInfoStsReg01[5] = makeDynString("IOWarnW", "Input/Output Warning");
  mBitInfoStsReg01[6] = makeDynString("IOErrorW", "Input/Output Error Warning");
  mBitInfoStsReg01[7] = makeDynString("IOSimuW", "Input/Output Simulated Warning");

  mBitInfoStsReg01[8] = makeDynString("AuMRW", "Auto Manual Request Warning Status");
  mBitInfoStsReg01[9] = makeDynString("PosW", "Position Warning");
  mBitInfoStsReg01[10] = makeDynString("StartISt", "Start Interlock Status");
  mBitInfoStsReg01[11] = makeDynString("TStopISt", "Temporary Stop Interlock Status");
  mBitInfoStsReg01[12] = makeDynString("AlUnAck", "Alarm UnAcknowledged");
  mBitInfoStsReg01[13] = makeDynString("AuIhFoMoSt", "Auto Inhibit Forced Mode Status");
  mBitInfoStsReg01[14] = makeDynString("AlSt", "Alarm Status");
  mBitInfoStsReg01[15] = makeDynString("AuIhMMoSt", "Auto Inhibit Manual Mode Status");
  
  mResult["StsReg01"] = mBitInfoStsReg01;

  // StsReg02
  mapping mBitInfoStsReg02;
  mBitInfoStsReg02["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg02[0] = makeDynString("OutOnOVSt", "Output On Order Value Status");
  mBitInfoStsReg02[1] = makeDynString("AuOnRSt", "Auto On Request Status");
  mBitInfoStsReg02[2] = makeDynString("MOnRSt", "Manual On Request Status");
  mBitInfoStsReg02[3] = makeDynString("AuOffRSt", "Auto Off Request Status");
  mBitInfoStsReg02[4] = makeDynString("MOffRSt", "Manual Off Request Status");
  mBitInfoStsReg02[5] = makeDynString("AuSBRSt", "Auto stand-by Request Status");
  mBitInfoStsReg02[6] = makeDynString("MSBRSt", "Manual stand-by Request Status");
  mBitInfoStsReg02[7] = makeDynString("AuRstRSt", "Auto reset Request Status");

  mBitInfoStsReg02[8] = makeDynString("MRstRSt", "Manual reset Request Status");
  mBitInfoStsReg02[10] = makeDynString("FuStopISt", "Full Stop Interlock Status");
  mBitInfoStsReg02[11] = makeDynString("EnRstartSt", "Enable Restart Status");
  mBitInfoStsReg02[12] = makeDynString("OutRstOVSt", "Manual Reset Request Status");
  mBitInfoStsReg02[13] = makeDynString("AlBW", "Alarm Blocked Warning");
  mBitInfoStsReg02[14] = makeDynString("OutOffOVSt", "Output Off Order Value Status");
  mBitInfoStsReg02[15] = makeDynString("OutSBOVSt", "Output StandBy Order Value Status");
  
  mResult["StsReg02"] = mBitInfoStsReg02;

  // Info for each bit is string with description

  // ObjectSt
  mapping mInfoObjectSt;
  mInfoObjectSt["type"] = STATUS_VALUE_TYPE_VALUE;
  mInfoObjectSt[0] = "0-no rotation: no rotation detected";
  mInfoObjectSt[1] = "1-accelerating: rotation detected and on order";
  mInfoObjectSt[2] = "2-nominal speed: nominal speed rotation reached and on order";
  mInfoObjectSt[3] = "3-recovering: rotation between 50% and nominal speed after pump reached nominal speed (and on order)";
  mInfoObjectSt[4] = "4-decelerating: rotation detected and not on order";

  mResult["ObjectSt"] = mInfoObjectSt;

  return mResult;
}

mapping _vacDpeBitInfo_initTypeBitInfo_CPC_VP_G() {
  mapping mResult;

  // Info for each bit is dyn_string, containing 2 items:
  //  1) short name of bit
  //  2) long name of bit
  
  // StsReg01
  mapping mBitInfoStsReg01;
  mBitInfoStsReg01["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg01[0] = makeDynString("OnSt", "On Status");
  mBitInfoStsReg01[1] = makeDynString("OffSt", "Off Status");
  mBitInfoStsReg01[2] = makeDynString("AuMoSt", "Auto Mode Status");
  mBitInfoStsReg01[3] = makeDynString("MMoSt", "Manual Mode Status");
  mBitInfoStsReg01[4] = makeDynString("FoMoSt", "Forced Mode Status");
  mBitInfoStsReg01[6] = makeDynString("IOErrorW", "Input/Output Error Warning");
  mBitInfoStsReg01[7] = makeDynString("IOSimuW", "Input/Output Simulated Warning");

  mBitInfoStsReg01[8] = makeDynString("AuMRW", "Auto Manual Request Warning Status");
  mBitInfoStsReg01[9] = makeDynString("FuStopISt", "Full Stop InterLock Status");
  mBitInfoStsReg01[10] = makeDynString("StartISt", "Start Interlock Status");
  mBitInfoStsReg01[11] = makeDynString("TStopISt", "Temporary Stop Interlock Status");
  mBitInfoStsReg01[12] = makeDynString("AlUnAck", "Alarm UnAcknowledged");
  mBitInfoStsReg01[13] = makeDynString("AuIhFoMoSt", "Auto Inhibit Forced Mode Status");
  mBitInfoStsReg01[14] = makeDynString("AlSt", "Alarm Status");
  mBitInfoStsReg01[15] = makeDynString("AuIhMMoSt", "Auto Inhibit Manual Mode Status");
  
  mResult["StsReg01"] = mBitInfoStsReg01;

  // StsReg02
  mapping mBitInfoStsReg02;
  mBitInfoStsReg02["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg02[0] = makeDynString("RunOSt", "Run Order Status");
  mBitInfoStsReg02[1] = makeDynString("AuOnRSt", "Auto On Request Status");
  mBitInfoStsReg02[2] = makeDynString("MOnRSt", "Manual On Request Status");
  mBitInfoStsReg02[3] = makeDynString("AuAuDepRSt", "Auto Auto Dependant Request Status");
  mBitInfoStsReg02[4] = makeDynString("AuDepOSt", "Auto Dependant Objects Status");
  mBitInfoStsReg02[5] = makeDynString("AuCStopRSt", "Auto Controlled Stop Request Status");
  mBitInfoStsReg02[6] = makeDynString("MAlBRSt", "Manual Alarm Block Request Status");
  mBitInfoStsReg02[7] = makeDynString("MTSasFSRSt", "Manual Temporary Stop as Full Stop Request Status");

  mBitInfoStsReg02[8] = makeDynString("CStopOSt", "Controlled Stop Order Status");
  mBitInfoStsReg02[11] = makeDynString("EnRstartSt", "Enable Restart Status");
  mBitInfoStsReg02[12] = makeDynString("SoftLDSt", "Software Local Mode Status");
  mBitInfoStsReg02[13] = makeDynString("AlBW", "Alarm Blocked Warning");
  mBitInfoStsReg02[14] = makeDynString("OutOffOVSt", "Output Off Order Value Status");
  mBitInfoStsReg02[15] = makeDynString("OutSBOVSt", "Output StandBy Order Value Status");
  
  mResult["StsReg02"] = mBitInfoStsReg02;

  // Info for each bit is string with description

  // ObjectSt
  mapping mInfoObjectSt;
  mInfoObjectSt["type"] = STATUS_VALUE_TYPE_VALUE;
  mInfoObjectSt[0] = "0=Recover";
  mInfoObjectSt[1] = "1=Nominal";
  mInfoObjectSt[2] = "2=Starting";
  mInfoObjectSt[3] = "3=Error";
  mInfoObjectSt[4] = "4=Off";
  mInfoObjectSt[5] = "5=Venting";
  mInfoObjectSt[6] = "6=Services";

  mResult["ObjectSt"] = mInfoObjectSt;

  return mResult;
}

mapping _vacDpeBitInfo_initTypeBitInfo_CPC_VR_AH() {
  mapping mResult;

  // Info for each bit is dyn_string, containing 2 items:
  //  1) short name of bit
  //  2) long name of bit
  
  // StsReg01
  mapping mBitInfoStsReg01;
  mBitInfoStsReg01["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg01[0] = makeDynString("PosSt", "Hardware alarm is active");
  mBitInfoStsReg01[2] = makeDynString("AuMoSt", "Auto Mode Status");
  mBitInfoStsReg01[4] = makeDynString("FoMoSt", "Forced Mode Status");
  mBitInfoStsReg01[6] = makeDynString("IOErrorW", "Input/Output Error Warning");
  mBitInfoStsReg01[7] = makeDynString("IOSimuW", "Input/Output Simulated Warning");

  mBitInfoStsReg01[8] = makeDynString("FoDiProW", "");
  mBitInfoStsReg01[9] = makeDynString("MIOErBRSt", "");
  mBitInfoStsReg01[10] = makeDynString("LogicSt", "Sources logic status for ConfigType = 3: TRUE = AND logic, FALSE = OR logic");
  mBitInfoStsReg01[13] = makeDynString("AuIhFoMoSt", "Auto Inhibit Forced Mode Status");
  mBitInfoStsReg01[14] = makeDynString("HFSt", "Hardware feedback status");
  mBitInfoStsReg01[15] = makeDynString("MPosRSt", "Manual position request status");
  
  mResult["StsReg01"] = mBitInfoStsReg01;

  // Info for each value is string with description

  // ObjectSt
  mapping mInfoObjectSt;
  mInfoObjectSt["type"] = STATUS_VALUE_TYPE_VALUE;
  mInfoObjectSt[0] = "0=No Alarm";
  mInfoObjectSt[1] = "1=Alarm Active";
  mInfoObjectSt[2] = "2=Relay invalid";
  mInfoObjectSt[3] = "3=Undefined state";

  mResult["ObjectSt"] = mInfoObjectSt;

  // SrcChanSt
  mapping mInfoSrcChanSt;
  mInfoSrcChanSt["type"] = STATUS_VALUE_TYPE_VALUE;
  mInfoSrcChanSt[0] = "0=None";
  mInfoSrcChanSt[1] = "1=Relay #1";
  mInfoSrcChanSt[2] = "2=Relay #2";
  mInfoSrcChanSt[3] = "3=Relay #3";
  mInfoSrcChanSt[4] = "4=Relay #4";

  mResult["SrcChanSt"] = mInfoSrcChanSt;

  return mResult;
}


mapping _vacDpeBitInfo_initTypeBitInfo_CPC_OnOff() {
  mapping mResult;

  // Info for each bit is dyn_string, containing 2 items:
  //  1) short name of bit
  //  2) long name of bit
  
  // StsReg01
  mapping mBitInfoStsReg01;
  mBitInfoStsReg01["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg01[0] = makeDynString("OnSt", "On Status");
  mBitInfoStsReg01[1] = makeDynString("OffSt", "Off Status");
  mBitInfoStsReg01[2] = makeDynString("AuMoSt", "Auto Mode Status");
  mBitInfoStsReg01[3] = makeDynString("MMoSt", "Manual Mode Status");
  mBitInfoStsReg01[4] = makeDynString("FoMoSt", "Forced Mode Status");
  mBitInfoStsReg01[5] = makeDynString("LDSt", "Local Drive Status");
  mBitInfoStsReg01[6] = makeDynString("IOErrorW", "Input/Output Error Warning");
  mBitInfoStsReg01[7] = makeDynString("IOSimuW", "Input/Output Simulated Warning");

  mBitInfoStsReg01[8] = makeDynString("AuMRW", "Auto Manual Request Warning Status");
  mBitInfoStsReg01[9] = makeDynString("PosW", "Position Warning");
  mBitInfoStsReg01[10] = makeDynString("StartISt", "Start Interlock Status");
  mBitInfoStsReg01[11] = makeDynString("TStopISt", "Temporary Stop Interlock Status");
  mBitInfoStsReg01[12] = makeDynString("AlUnAck", "Alarm UnAcknowledged");
  mBitInfoStsReg01[13] = makeDynString("AuIhFoMoSt", "Auto Inhibit Forced Mode Status");
  mBitInfoStsReg01[14] = makeDynString("AlSt", "Alarm Status");
  mBitInfoStsReg01[15] = makeDynString("AuIhMMoSt", "Auto Inhibit Manual Mode Status");
  
  mResult["StsReg01"] = mBitInfoStsReg01;

  // StsReg02
  mapping mBitInfoStsReg02;
  mBitInfoStsReg02["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg02[0] = makeDynString("OutOnOVSt", "Output On Order Value Status");
  mBitInfoStsReg02[1] = makeDynString("AuOnRSt", "Auto On Request Status");
  mBitInfoStsReg02[2] = makeDynString("MOnRSt", "Manual On Request Status");
  mBitInfoStsReg02[3] = makeDynString("AuOffRSt", "Auto Off Request Status");
  mBitInfoStsReg02[4] = makeDynString("MOffRSt", "Manual Off Request Status");
  mBitInfoStsReg02[5] = makeDynString("HOnRSt", "Hardware On/Open request");
  mBitInfoStsReg02[6] = makeDynString("HOffRSt", "Hardware Off/Close request status");


  mBitInfoStsReg02[10] = makeDynString("FuStopISt", "Full Stop Interlock Status");
  mBitInfoStsReg02[11] = makeDynString("EnRstartSt", "Enable Restart Status");
  mBitInfoStsReg02[12] = makeDynString("SoftLDSt", "Software Local Mode Status");
  mBitInfoStsReg02[13] = makeDynString("AlBW", "Alarm Blocked Warning");
  mBitInfoStsReg02[14] = makeDynString("OutOffOVSt", "Output Off Order Value Status");
  
  mResult["StsReg02"] = mBitInfoStsReg02;

  return mResult;
}

mapping _vacDpeBitInfo_initTypeBitInfo_CPC_ProcessControlObject() {
  mapping mResult;

  // Info for each bit is dyn_string, containing 2 items:
  //  1) short name of bit
  //  2) long name of bit
  
  // StsReg01
  mapping mBitInfoStsReg01;
  mBitInfoStsReg01["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg01[0] = makeDynString("OnSt", "On Status");
  mBitInfoStsReg01[1] = makeDynString("OffSt", "Off Status");
  mBitInfoStsReg01[2] = makeDynString("AuMoSt", "Auto Mode Status");
  mBitInfoStsReg01[3] = makeDynString("MMoSt", "Manual Mode Status");
  mBitInfoStsReg01[4] = makeDynString("FoMoSt", "Forced Mode Status");
  mBitInfoStsReg01[6] = makeDynString("IOErrorW", "Input/Output Error Warning");
  mBitInfoStsReg01[7] = makeDynString("IOSimuW", "Input/Output Simulated Warning");

  mBitInfoStsReg01[8] = makeDynString("AuMRW", "Auto Manual Request Warning Status");
  mBitInfoStsReg01[9] = makeDynString("FuStopISt", "Full Stop Interlock Status");
  mBitInfoStsReg01[10] = makeDynString("StartISt", "Start Interlock Status");
  mBitInfoStsReg01[11] = makeDynString("TStopISt", "Temporary Stop Interlock Status");
  mBitInfoStsReg01[12] = makeDynString("AlUnAck", "Alarm UnAcknowledged");
  mBitInfoStsReg01[13] = makeDynString("AuIhFoMoSt", "Auto Inhibit Forced Mode Status");
  mBitInfoStsReg01[14] = makeDynString("AlSt", "Alarm Status");
  mBitInfoStsReg01[15] = makeDynString("AuIhMMoSt", "Auto Inhibit Manual Mode Status");
  
  mResult["StsReg01"] = mBitInfoStsReg01;

  // StsReg02
  mapping mBitInfoStsReg02;
  mBitInfoStsReg02["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg02[0] = makeDynString("RunOSt", "Run Order Status");
  mBitInfoStsReg02[1] = makeDynString("AuOnRSt", "Auto On Request Status");
  mBitInfoStsReg02[2] = makeDynString("MOnRSt", "Manual On Request Status");
  mBitInfoStsReg02[3] = makeDynString("AuAuDepRSt", "Auto Auto Dependant Request Status");
  mBitInfoStsReg02[4] = makeDynString("AuDepOSt", "Auto Dependant Objects Status");
  mBitInfoStsReg02[5] = makeDynString("AuCStopRSt", "Auto Controlled Stop Request Status");
  mBitInfoStsReg02[6] = makeDynString("MAlBRSt", "Manual Alarm Block Request Status");
  mBitInfoStsReg02[7] = makeDynString("MTSasFSRSt", "Manual Temporary Stop as Full Stop Request Status");
  
  mBitInfoStsReg02[8] = makeDynString("CStopOSt", "Controlled Stop Order Status");
  mBitInfoStsReg02[11] = makeDynString("EnRstartSt", "Enable Restart Status");
  mBitInfoStsReg02[12] = makeDynString("SoftLDSt", "Software Local Mode Status");
  mBitInfoStsReg02[13] = makeDynString("AlBW", "Alarm Blocked Warning");
  
  mResult["StsReg02"] = mBitInfoStsReg02;

  return mResult;
}


mapping _vacDpeBitInfo_initTypeBitInfo_CPC_VG_F() {
  mapping mResult;

  // Info for each bit is dyn_string, containing 2 items:
  //  1) short name of bit
  //  2) long name of bit
  
  // StsReg01
  mapping mBitInfoStsReg01;
  mBitInfoStsReg01["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg01[0] = makeDynString("SwOffOV", "Switch Off Order Value");
  mBitInfoStsReg01[1] = makeDynString("SwOnOV", "Switch On Order Value");
  mBitInfoStsReg01[2] = makeDynString("AuMoSt", "Auto Mode Status");
  mBitInfoStsReg01[3] = makeDynString("MMoSt", "Manual Mode Status");
  mBitInfoStsReg01[4] = makeDynString("FoMoSt", "Forced Mode Status");
  mBitInfoStsReg01[5] = makeDynString("AutoOV", "Automatic Order Value");
  mBitInfoStsReg01[6] = makeDynString("IOErrorW", "Input/Output Error Warning");
  mBitInfoStsReg01[7] = makeDynString("IOSimuW", "Input/Output Simulated Warning");

  mBitInfoStsReg01[8] = makeDynString("FoDiProW", "Forced Differs from Process Warning");
  mBitInfoStsReg01[9] = makeDynString("MIOErBRSt", "Manual Input/Output Error Block Request Status");
  mBitInfoStsReg01[10] = makeDynString("StartISt", "Start Interlock Status");
  mBitInfoStsReg01[11] = makeDynString("TStopISt", "Temporary Stop Interlock Status");
  mBitInfoStsReg01[12] = makeDynString("AlUnAck", "Alarm UnAcknowledged");
  mBitInfoStsReg01[13] = makeDynString("AuIhFoMoSt", "Auto Inhibit Forced Mode Status");
  mBitInfoStsReg01[14] = makeDynString("AlSt", "Alarm Status");
  mBitInfoStsReg01[15] = makeDynString("AuIhMMoSt", "Auto Inhibit Manual Mode Status");
  
  mResult["StsReg01"] = mBitInfoStsReg01;

  // StsReg02
  mapping mBitInfoStsReg02;
  mBitInfoStsReg02["type"] = STATUS_VALUE_TYPE_BITS;
  mBitInfoStsReg02[1] = makeDynString("AuOnRSt", "Auto On Request Status");
  mBitInfoStsReg02[2] = makeDynString("MOnRSt", "Manual On Request Status");
  mBitInfoStsReg02[3] = makeDynString("AuOffRSt", "Auto Off Request Status");
  mBitInfoStsReg02[4] = makeDynString("MOffRSt", "Manual Off Request Status");
  mBitInfoStsReg02[7] = makeDynString("AuMRW", "Auto Manual Request Warning");

  mBitInfoStsReg02[10] = makeDynString("FuStopISt", "Full Stop Interlock Status");
  mBitInfoStsReg02[11] = makeDynString("EnRstartSt", "Enable Restart Status");
  mBitInfoStsReg02[13] = makeDynString("AlBW", "Alarm Blocked Warning");
  
  mResult["StsReg02"] = mBitInfoStsReg02;

  // Info for each bit is string with description

  // ObjectSt
  mapping mInfoObjectSt;
  mInfoObjectSt["type"] = STATUS_VALUE_TYPE_VALUE;
  mInfoObjectSt[0] = "0=Off without error";
  mInfoObjectSt[1] = "1=Over Range";
  mInfoObjectSt[2] = "2=Measurement OK";
  mInfoObjectSt[3] = "3=Under Range";
  mInfoObjectSt[4] = "4=No Hardware";
  mInfoObjectSt[5] = "5=Hardware Error";
  mInfoObjectSt[6] = "6= Profibus Slave Error";
  mInfoObjectSt[7] = "7=Profibus Slave Not present";
  mInfoObjectSt[8] = "8=High Pressure";

  mResult["ObjectSt"] = mInfoObjectSt;

  return mResult;
}


private string vacDpeBitInfo_getTypeName(string sDeviceName) {

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "vacDpeBitInfo",
                                "vacDpeBitInfo_getTypeName(" + sDeviceName + "): start");
  if(!dpExists(sDeviceName)) {
    string sDpName = unGenericDpFunctions_dpAliasToName(sDeviceName);
    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "vacDpeBitInfo",
                                  "vacDpeBitInfo_getTypeName(" + sDeviceName + "): DP from alias", sDpName);
    if(!dpExists(sDpName)) {
      unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "vacDpeBitInfo",
                                   "vacDpeBitInfo_getTypeName(" + sDeviceName + "): no DP found");
      return "";
    }
    sDeviceName = sDpName;
  }
  return dpTypeName(sDeviceName);
}
