/*
  Support for device list, introduced for piquet device list
*/

//private const string PIQUET_DEV_LIST_CRIT_DP_TYPE = "_VacPiquetDevListCriteria";
private const string VAC_MONITORING_LIST_CONFIG_DP_TYPE = "_VacMonitoringListCriteria"; 
private const string VAC_MONITORING_LIST_RESULT_DP_TYPE = "_VacMonitoringListResult";
private const int MAX_MONITORING_LIST_RESULTS = 3;

string vacMonitoringList_getCriteriaDp(dyn_string &exceptionInfo) {
  dynClear(exceptionInfo);
  string sDpName = VAC_MONITORING_LIST_CONFIG_DP_TYPE;
  if(!dpExists(sDpName)) {
    dpCreate(sDpName, VAC_MONITORING_LIST_CONFIG_DP_TYPE);
    dyn_errClass err = getLastError();
    if(dynlen(err) > 0)
    {
      fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_getCriteriaDp(): failed to create DP " +
                        sDpName + ": " + err, "");
      return "";
    }
    bool success = false;
    for(int waitCount = 0 ; waitCount < 20 ; waitCount++) {
      if(dpExists(sDpName)) {
        success = true;
        break;
      }
      delay(0, 50);
    }
    if(!success) {
      fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_getCriteriaDp(): DP " +sDdpName +
                        " is not created", "");
      return "";
    }
  }
  return sDpName;
}

dyn_string vacMonitoringList_getConfigs(dyn_string &exceptionInfo) {
  dyn_string dsConfigs;
  string sDpName = vacMonitoringList_getCriteriaDp(exceptionInfo);
  if(sDpName != "") {
    dpGet(sDpName + ".ConfigDpNames", dsConfigs);
    dyn_errClass deErr = getLastError();
    if(dynlen(deErr) > 0) {
      fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_getConfigs(): dpGet(" +
                      sDpName + ") failed: " + deErr, "");
    }
  }
  return dsConfigs;
}

bool vacMonitoringList_saveConfigs(dyn_string dsConfigs, dyn_string &exceptionInfo) {
  string sDpName = vacMonitoringList_getCriteriaDp(exceptionInfo);
  if(sDpName != "") {
    dpSet(sDpName + ".ConfigDpNames", dsConfigs);
    dyn_errClass deErr = getLastError();
    if(dynlen(deErr) > 0) {
      fwException_raise(exceptionInfo, "ERROR", "vacDeviceListPiquet_saveConfigs(): dpGet(" +
                        sDpName + ") failed: " + deErr, "");
      return false;
    }
  }
  return true;
}

string vacMonitoringList_getResultDpType() {
  return VAC_MONITORING_LIST_RESULT_DP_TYPE;
}

// Decide which DP shall be used to save new 'reference', takes the one with
// the latest date of data, or creates new one.
// Checnges after demo for Fabien: at most 1 reference per day, so if there
// is already reference for today - take that one

string vacMonitoringList_getNextResultDp(dyn_string &exceptionInfo) {
  time tNow = getCurrentTime();
  dynClear(exceptionInfo);
  dyn_string dsAllDpNames = dpNames("*", VAC_MONITORING_LIST_RESULT_DP_TYPE);
  string sOldDpName = "";
  time tOldTime = getCurrentTime() + 365 * 24 * 60 * 60;
  for(int idx = dynlen(dsAllDpNames) ; idx > 0 ; idx--) {
    time tValueTime;
    dyn_string dsValueDps;
    string sDpName = dpSubStr(dsAllDpNames[idx], DPSUB_DP);
    dpGet(sDpName + ".DpNames", dsValueDps,
          sDpName + ".DpNames:_online.._stime", tValueTime);
    if((year(tValueTime) == year(tNow)) && (yearDay(tValueTime) == yearDay(tNow))) {
      return sDpName;  // Reference for today, shall be reused
    }
    if(dynlen(dsValueDps) == 0) {
      return sDpName;  // Empty DP, without result - can be used
    }
    if(tValueTime < tOldTime) {
      tOldTime = tValueTime;
      sOldDpName = sDpName;
    }
  }
  if(dynlen(dsAllDpNames) >= MAX_MONITORING_LIST_RESULTS) {
    return oldDpName;
  }

  // New DP shall be created
  string sDpName = "";
  int idx = 0;
  while(true) {
    sprintf(sDpName, "%s_%06d", VAC_MONITORING_LIST_RESULT_DP_TYPE, ++idx);
    if(!dpExists(sDpName)) {
      break;
    }
  }
  
  dpCreate(sDpName, VAC_MONITORING_LIST_RESULT_DP_TYPE);
  dyn_errClass deErr = getLastError();
  if(dynlen(deErr) > 0) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_getResultDp(): failed to create DP " +
                      sDpName + ": " + err, "");
    return "";
  }
  bool success = false;
  for(int waitCount = 0 ; waitCount < 20 ; waitCount++) {
    if(dpExists(sDpName)) {
      success = true;
      break;
    }
    delay(0, 50);
  }
  if(!success) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_getResultDp(): DP " +
                      sDpName + " is not created", "");
    return "";
  }
  return sDpName;
}

bool vacMonitoringList_loadRefData(string sDpName, mapping &mResult, dyn_string &exceptionInfo) {

  mappingClear(mResult);
  dynClear(exceptionInfo);

  dyn_string dsDpNames, dsStateDpeNames, dsStateDpeValues, dsStateNames, dsStateColors,
    dsValues, dsComments;
  time tRefTime;
  dpGet(sDpName + ".DpNames:_original.._stime", tRefTime,
        sDpName + ".DpNames", dsDpNames,
        sDpName + ".StateDpeNames", dsStateDpeNames,
        sDpName + ".StateDpeValues", dsStateDpeValues,
        sDpName + ".StateNames", dsStateNames,
        sDpName + ".StateColors", dsStateColors,
        sDpName + ".Values", dsValues,
        sDpName + ".Comments", dsComments);

  dyn_errClass deErr = getLastError();
  if(dynlen(deErr) > 0) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_loadRefData(): dpGet(" +
                      sDpName + ") failed: " + deErr, "");
    return false;
  }

  int iRefSize = dynlen(dsDpNames);

  // Check sizes of resulting lists - all must have the same size
  if(iRefSize != dynlen(dsStateDpeNames)) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_loadRefData(): broken reference: " +
                      iRefSize + " DP(s), " + dynlen(dsStateDpeNames) + " state DPE name(s)", "");
    return false;
  }
  if(iRefSize != dynlen(dsStateDpeValues)) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_loadRefData(): broken reference: " +
                      iRefSize + " DP(s), " + dynlen(dsStateDpeValues) + " state DPE1 value(s)", "");
    return false;
  }
  if(iRefSize != dynlen(dsStateNames)) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_loadRefData(): broken reference: " +
                      iRefSize + " DP(s), " + dynlen(dsStateNames) + " state name(s)", "");
    return false;
  }
  if(iRefSize != dynlen(dsStateColors)) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_loadRefData(): broken reference: " +
                      iRefSize + " DP(s), " + dynlen(dsStateColors) + " state color(s)", "");
    return false;
  }
  if(iRefSize != dynlen(dsValues)) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_loadRefData(): broken reference: " +
                      iRefSize + " DP(s), " + dynlen(dsValues) + " value(s)", "");
    return false;
  }
  if(iRefSize != dynlen(dsComments)) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_loadRefData(): broken reference: " +
                      iRefSize + " DP(s), " + dynlen(dsComments) + " comment(s)", "");
    return false;
  }

  // Build result
  mResult["time"] = tRefTime;
  mResult["dpNames"] = dsDpNames;
  
  // Part of data shall be decoded
  dyn_dyn_string ddsStateDpeNames;
  dyn_dyn_anytype ddaStateDpeValues;
  for(int iDpIdx = 1 ; iDpIdx <= iRefSize ; iDpIdx++) {
    dyn_string dsSingleDpeNames = jsonDecode(dsStateDpeNames[iDpIdx]);
    dynAppend(ddsStateDpeNames, dsSingleDpeNames);
    dyn_anytype daSingleDpeValues = jsonDecode(dsStateDpeValues[iDpIdx]);
    dynAppend(ddaStateDpeValues, daSingleDpeValues);
    if(dynlen(dsSingleDpeNames) != dynlen(daSingleDpeValues)) {
      fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_loadRefData(): broken reference for DP " +
                        dsDpNames[iDpIdx] + ": " + dynlen(dsSingleDpeNames) +
                        " state DPE name(s), " + dynlen(daSingleDpeValues) + " state DPE value(s)", "");
      return false;
    }
  }
  mResult["stateDpeNames"] = ddsStateDpeNames;
  mResult["stateDpeValues"] = ddaStateDpeValues;

  mResult["stateNames"] = dsStateNames;
  mResult["stateColors"] = dsStateColors;
  mResult["values"] = dsValues;
  mResult["comments"] = dsComments;
  
  return true;
}

bool vacMonitoringList_saveRefData(string sDpName, dyn_string dsDpNames,
                                   dyn_dyn_string ddsStateDpeNames, dyn_dyn_anytype ddaStateDpeValues,
                                   dyn_string dsStateNames, dyn_string dsStateColors,
                                   dyn_string dsValues, dyn_string dsComments,
                                   dyn_string &exceptionInfo) {

  // Check consistency - all list shall have the same length
  int iDpCount = dynlen(dsDpNames);
  if(iDpCount != dynlen(ddsStateDpeNames)) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_saveRefData(): bad data: " +
                      iDpCount + " DP(s), " + dynlen(ddsStateDpeNames) + " state DPE name(s)", "");
  }
  if(iDpCount != dynlen(ddaStateDpeValues)) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_saveRefData(): bad data: " +
                      iDpCount + " DP(s), " + dynlen(ddaStateDpeValues) + " state DPE values(s)", "");
  }
  if(iDpCount != dynlen(dsStateNames)) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_saveRefData(): bad data: " +
                      iDpCount + " DP(s), " + dynlen(dsStateNames) + " state name(s)", "");
  }
  if(iDpCount != dynlen(dsStateColors)) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_saveRefData(): bad data: " +
                      iDpCount + " DP(s), " + dynlen(dsStateColors) + " state color(s)", "");
  }
  if(iDpCount != dynlen(dsValues)) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_saveRefData(): bad data: " +
                      iDpCount + " DP(s), " + dynlen(dsValues) + " analog value(s)", "");
  }
  if(iDpCount != dynlen(dsComments)) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_saveRefData(): bad data: " +
                      iDpCount + " DP(s), " + dynlen(dsComments) + " comment(s)", "");
  }
  
  // Encode state DPE names and values
  dyn_string dsStateDpeNames;
  dyn_string dsStateDpeValues;
  for(int iDpIdx = 1 ; iDpIdx <= iDpCount ; iDpIdx++) {
    dynAppend(dsStateDpeNames, jsonEncode(ddsStateDpeNames[iDpIdx]));
    dynAppend(dsStateDpeValues, jsonEncode(ddaStateDpeValues[iDpIdx]));
  }
  
  // Write to DP
  dpSet(sDpName + ".DpNames", dsDpNames,
        sDpName + ".StateDpeNames", dsStateDpeNames,
        sDpName + ".StateDpeValues", dsStateDpeValues,
        sDpName + ".StateNames", dsStateNames,
        sDpName + ".StateColors", dsStateColors,
        sDpName + ".Values", dsValues,
        sDpName + ".Comments", dsComments);

  dyn_errClass deErr = getLastError();
  if(dynlen(deErr) > 0) {
    fwException_raise(exceptionInfo, "ERROR", "vacMonitoringList_saveRefData(): dpSet(" +
                      sDpName + ") failed: " + deErr, "");
    return false;
  }
  return true;
}

bool vacMonitoringList_matchesDeltaCondition(string sDpName, mapping mAllRefData,
                                             dyn_string dsStateDpeNames, dyn_anytype daStateDpeValues,
                                             string &sDeltaMsg)
{
  if(!mappingHasKey(mAllRefData, "dpNames")) {
    return true;  // Empty reference = not DELTA mode
  }
  int iRefIdx = dynContains(mAllRefData["dpNames"], sDpName);
  if(iRefIdx <= 0) {
    return true;  // DP was not in reference result; or mode is not delta
  }
  dyn_string dsRefStateDpeNames = mAllRefData["stateDpeNames"][iRefIdx];
  dyn_anytype daRefStateDpeValues = mAllRefData["stateDpeValues"][iRefIdx];

  for(int iDpeIdx = iDpeIdx = dynlen(dsStateDpeNames) ; iDpeIdx > 0 ; iDpeIdx--) {
    int iRefDpeIdx = dynContains(dsRefStateDpeNames, dsStateDpeNames[iDpeIdx]);
    if(iRefDpeIdx <= 0) {
      continue;  // Shall not happen, only if code was changed => set of state DPEs was changed
    }
    if(daStateDpeValues[iDpeIdx] != daRefStateDpeValues[iRefDpeIdx]) {
      sDeltaMsg = "State changed";
      return true;
    }
  }
  
  // DP is in both old and new lists, states are equal - analyze change history
  int iChangeCount = vacMonitoringList_countStateChanges(dsStateDpeNames, mAllRefData["time"]);
  if(iChangeCount > 0) {
    sDeltaMsg = "Same state, but changed " + iChangeCount + " time" + (iChangeCount > 1 ? "s" : "");
    return true;
  }
  
  return false;
}

int vacMonitoringList_getNatureOrder(string sDpName, string &sNature) {
  string sDomain;
  unGenericDpFunctions_getGroups(sDpName, sDomain, sNature);
  dyn_string dsAllNatures = makeDynString(
      "Valve_HighVac_Sector",
      "Valve_RoughVac_Sector",
      "Pump_RoughVac_Sector",
      "Pump_HighVac_Sector",
      "Gauge_RoughVac_Sector",
      "Gauge_HighVac_Sector");
  return dynContains(dsAllNatures, sDpName);
}

private int vacMonitoringList_countStateChanges(dyn_string dsDpeNames, time tStartTime) {
  int iResult = 0;
  time tNow = getCurrentTime();
  for(int iDpeIdx = dynlen(dsDpeNames) ; iDpeIdx > 0 ; iDpeIdx--) {
    iResult += vacMonitoringList_countDpeChanges(dsDpeNames[iDpeIdx], tStartTime, tNow);
  }
  return iResult;
}

private int vacMonitoringList_countDpeChanges(string sDpeConfigName, time tStartTime, time tNow) {
  int iResult = 0;
  string sDpeName, sConfig;
  string sDpeToUse = vacMonitoringList_buildDpeForHistoryQuery(sDpeConfigName, sDpeName, sConfig);
// DebugN(sDpeConfigName, sDpeToUse, sDpeName, sConfig);
  if(sConfig == "") {
    sConfig = "_original.._value";
  }
  string sQuery = "SELECT 'COUNT(" + sConfig + ")' FROM '" + sDpeName + 
                  "' TIMERANGE(";
  string sTimeStr = formatTime("%Y.%m.%d %H:%M:%S", tStartTime + 1);
  sQuery += "\"" + sTimeStr + "\",";
  sTimeStr = formatTime("%Y.%m.%d %H:%M:%S", tNow);
  sQuery += "\"" + sTimeStr + "\",1,0)";
  dyn_dyn_anytype ddaQueryResult;
  dpQuery(sQuery, ddaQueryResult);
  if(dynlen(ddaQueryResult) > 1) {
    iResult = ddaQueryResult[2][2];
  }
//DebugN(iResult, sQuery);
  return iResult;
}

private string vacMonitoringList_buildDpeForHistoryQuery(string sDpeConfigName, string &sDpeName, string &sConfig) {
  // The main idea is to replace StsRegXX, which is not archived, with evStsRegXX, which is archived
  sConfig = dpSubStr(sDpeConfigName, DPSUB_CONF_DET_ATT);
  string sPureDpName = dpSubStr(sDpeConfigName, DPSUB_SYS_DP);
  sDpeName = dpSubStr(sDpeConfigName, DPSUB_SYS_DP_EL);
  string sPureDpeName = substr(sDpeName, strlen(sPureDpName));
  if(strpos(sPureDpeName, "evStsReg") == -1) {  // this is not evStsReg
    if(strpos(sPureDpeName, "StsReg") >= 0) { // this is StsReg
      strreplace(sPureDpeName, "StsReg", "evStsReg");
    }
  }
  string sNewDpeName = sPureDpName + sPureDpeName;
  if(dpExists(sNewDpeName)) {
    sDpeName = sNewDpeName;
  }
  else {
    DebugN(sNewDpeName + " does not exist");
  }
  return sDpeName + sConfig;
}

