

enum BarProfileColorMode {
  CurrentState = 0,
  CompareToRef = 1
};

// Name of DP type for rules storage
private const string VALUE_COLOR_RULES_DP_TYPE = "_VacValueColorRules";

int VacVccInitValueColorRules(string sEwoName, string sUsageName) {
  string sDpName = VacVccFindUsageDpName(sUsageName);
  if(sDpName == "") {
    return -1;
  }
  dyn_float dfValues;
  dyn_string dsColors;
  dpGet(sDpName + ".Values", dfValues,
        sDpName + ".Colors", dsColors);
  int iRulesCount = dynlen(dfValues);
  if(iRulesCount > dynlen(dsColors)) {
    iRulesCount = dynlen(dsColors);
  }
  shape ewo = getShape(sEwoName);
  ewo.clearRules();
  for(int idx = 1 ; idx <= iRulesCount ; idx++) {
    ewo.addRule(dfValues[idx], dsColors[idx]);
  }
  return iRulesCount;
}

int VacVccSaveValueColorRules(string sUsageName, dyn_float dfValues, dyn_string dsColors) {
  string sDpName = VacVccFindUsageDpName(sUsageName);
  if(sDpName == "") {
    return -1;
  }
  return dpSet(sDpName + ".Values", dfValues,
        sDpName + ".Colors", dsColors);
}


int VacVccEditValueColorRules(string sEwoName, string sUsageName, string sValueTitle = "") {
  dyn_string dsDollars, dsColors;
  dyn_float dfValues;
  shape ewo = getShape(sEwoName);
  ewo.getRules(dfValues, dsColors);
  string sRules = VacVccRulesToString(dfValues, dsColors);
  dsDollars = makeDynString("$rules:" + sRules, "$usage:" + sUsageName);
  if(sValueTitle != "") {
    dynAppend(dsDollars, "$sValueTitle:" + sValueTitle);
  }
  ChildPanelOnCentralReturn("vision/vacPanels/ValueColorRules.pnl", "Color Rules", dsDollars, dfValues, dsColors);
  int iRulesCount = dynlen(dfValues);
  if(iRulesCount > 0) {
    ewo.clearRules();
    for(int idx = 1 ; idx <= iRulesCount ; idx++) {
      ewo.addRule(dfValues[idx], dsColors[idx]);
    }
  }
  return iRulesCount;
}

string VacVccRulesToString(dyn_float dfValues, dyn_string dsColors) {
  string sResult;
  int iSize = dynlen(dfValues);
  for(int iIdx = 1 ; iIdx <= iSize ; iIdx++) {
    string sValue;
    sprintf(sValue, "%.2e", dfValues[iIdx]);
    if(strlen(sResult) > 0) {
      sResult += "|";
    }
    sResult += sValue;
    sResult += "|";
    string sColor = dsColors[iIdx];
    sResult += sColor;
  }
  return sResult;
}

private string VacVccFindUsageDpName(string sUsageName) {
  nameCheck(sUsageName);  // If input name is not allowed - it will become allowed one
  string sDpName = VALUE_COLOR_RULES_DP_TYPE + "_" + sUsageName;
  if(!dpExists(sDpName)) {
    dpCreate(sDpName, VALUE_COLOR_RULES_DP_TYPE);
  }
  return sDpName;
}
